#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
no warnings;
use List::Util 'min','max';
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;

{
  # integers summed

  sub check_revsorted {
    my ($aref) = @_;
    foreach my $i (0 .. $#$aref-1) {
      if ($aref->[$i] < $aref->[$i+1]) {
        die "not revsorted";
      }
    }
  }
  sub insert_revsorted {
    my ($aref, $new) = @_;
    foreach my $i (reverse 0 .. $#$aref) {
      if ($aref->[$i] >= $new) {
        ### insert: "at ".($i+1)." of $#$aref"
        splice @$aref, $i+1,0, $new;
        return $i+1;
      }
    }
    unshift @$aref, $new;
  }
  sub min_sums {
    my ($aref) = @_;
    my @list = @$aref;
    while (@list >= 2) {
      check_revsorted(\@list);
      if (@list >=3 && $list[-2]==$list[-3]) {
        print "3 same\n";
        die;
      }
      insert_revsorted(\@list, (pop @list) + (pop @list));
    }
  }

  my @list;
  my $n = 1;
  for (;;) {
    check_revsorted(\@list);
    if (@list < 2 || $n <= $list[-2]) {
      if ($n % 100 == 0) {
        print "n=$n under $list[-2] of ",scalar(@list),"\r";
      }
      insert_revsorted(\@list, $n++);
      min_sums(\@list);
    } else {
      insert_revsorted(\@list, (pop @list) + (pop @list));
    }
  }
}
{
  my $WEIGHT = 0;
  my $MINLEN = 1;
  my $MAXLEN = 2;

  sub initial_sames_num {
    my ($aref,$start) = @_;
    $start ||= 0;
    foreach my $i ($start+1 .. $#$aref) {
      if ($aref->[$start]->[$WEIGHT] != $aref->[$i]->[$WEIGHT]) {
        return $i-$start;
      }
    }
    return scalar(@$aref)-$start;
  }
  initial_sames_num([ ]) == 0 || die;
  initial_sames_num([ [1] ]) == 1 || die;
  initial_sames_num([ [1],[1] ]) == 2 || die;
  initial_sames_num([ [1],[2],[2],[2] ]) == 1 || die;
  initial_sames_num([ [1],[2],[2],[2] ],1) == 3 || die;
  initial_sames_num([ [1],[2],[2],[2],[9],[9] ],1) == 3 || die;
  initial_sames_num([ [1],[1],[2] ]) == 2 || die;
  initial_sames_num([ [1],[1],[1],[2],[2],[2] ]) == 3 || die;

  sub final_sames_pos {
    my ($aref,$offset) = @_;
    $offset ||= 0;
    my $i = $#$aref - $offset;
    my $cmp = $aref->[$i]->[$WEIGHT];
    for (;;) {
      if (--$i < 0) {
        return 0;
      }
      if ($aref->[$i]->[$WEIGHT] != $cmp) {
        return $i+1;
      }
    }
  }
  final_sames_pos([ [1] ]) == 0 || die;
  final_sames_pos([ [1],[1] ]) == 0 || die;
  final_sames_pos([ [2],[2],[2],[1] ]) == 3 || die;
  final_sames_pos([ [2],[2],[2],[1] ],1) == 0 || die;
  final_sames_pos([ [1],[2],[2],[2],[9] ],1) == 1 || die;
  final_sames_pos([ [2],[1],[1] ]) == 1 || die;
  final_sames_pos([ [1],[1],[1],[2],[2],[2] ]) == 3 || die;

  sub range_combine {
    return [min(map {$_->[0]} @_),
            max(map {$_->[0]} @_)];
  }

  sub insert {
    my ($pending, $new) = @_;
    foreach my $i (reverse 0 .. $#$pending) {
      if ($pending->[$i]->[$WEIGHT] >= $new->[$WEIGHT]) {
        ### insert: "at ".($i+1)." of $#$pending"
        splice @$pending, $i+1,0, $new;
        return;
      }
    }
    unshift @$pending, $new;
  }

  my $verbose = 1;
  my $attempt;
  my $merge;
  my $count_alts = 0;
  $merge = sub {
    my ($orig_pending, $i, $j) = @_;
    my @pending = @$orig_pending;
    for (;;) {
      ### merge: "$i,$j of ".scalar(@pending)
      $i < $j or die;
      my $y = splice @pending, $j,1;
      my $x = splice @pending, $i,1;
      my $new = [ $x->[$WEIGHT] + $y->[$WEIGHT],
                  1 + min($x->[$MINLEN],$y->[$MINLEN]),
                  1 + max($x->[$MAXLEN],$y->[$MAXLEN]) ];
      insert(\@pending,$new);
      if ($verbose) {
        print "  merge $i $j   $x->[$WEIGHT],$x->[$MINLEN],$x->[$MAXLEN]  $y->[$WEIGHT],$y->[$MINLEN],$y->[$MAXLEN]  -> $new->[$WEIGHT],$new->[$MINLEN],$new->[$MAXLEN]  to pending ",scalar(@pending),"\n";
      }
      if (@pending < 2) {
        return [$pending[0]->[$MINLEN], $pending[0]->[$MAXLEN]];
      }

      ### sorted: @pending
      if (0) {
        foreach my $i (0 .. $#pending-1) {
          $pending[$i]->[$WEIGHT] >= $pending[$i+1]->[$WEIGHT]
            or die "unsorted";
        }
      }
      # @pending = sort {($a->[$WEIGHT] <=> $b->[$WEIGHT])
      #                    || ($a->[$MINLEN] <=> $b->[$MINLEN])
      #                    || ($a->[$MAXLEN] <=> $b->[$MAXLEN])}
      #   @pending;

      my $x_pos = final_sames_pos(\@pending);
      if ($x_pos < $#pending-1) {   # more than -0 and -1
        print "  equal weight $x_pos to $#pending weight $pending[-1]->[$WEIGHT]\n";
        $count_alts++;
        my @ret;
        foreach my $i ($x_pos .. $#pending) {
          foreach my $j ($i+1 .. $#pending) {
            push @ret, $merge->(\@pending,$i,$j);
          }
        }
        return range_combine(@ret);
      }
      my $y_pos = final_sames_pos(\@pending,1);
      if ($y_pos < $#pending-1) {   # more than just -1
        $count_alts++;
        print " equal second weight $y_pos of $#pending weight $pending[-2]->[$WEIGHT]\n";
        my @ret;
        foreach my $i ($y_pos .. $#pending-1) {
          push @ret, $merge->(\@pending,$i,$#pending);
        }
        return range_combine(@ret);
      }

      ### again, end two ...
      $i = $#pending-1;
      $j = $#pending;
    }
  };

  sub log2_floor {
    my ($n) = @_;
    my $ret = 0;
    $n = int($n);
    while ($n >= 2) {
      $n >>= 1;
      $ret++;
    }
    return $ret;
  }
  log2_floor(7) == 2 or die;
  log2_floor(8) == 3 or die;
  log2_floor(7) == int(log(7)/log(2)) or die;

  sub want_min {
    my ($n) = @_;
    # A126235
    # a(n) - a(n-1) = 1 increment if n has the form 3*2^k-1
    return log2_floor(2*($n+1)/3);
  }
  sub want_max {
    my ($n) = @_;
    # A126236
    return log2_floor($n) + log2_floor(2*$n/3);
  }
  {
    foreach my $n (2 .. 16) {
      print want_min($n),",";
    }
    print "\n";

    my @A126235_samples = (1,1,1, 2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,6,6,6,6,6,6,6,6,6);
    foreach my $i (0 .. $#A126235_samples) {
      my $n = $i+2; # A126235 OFFSET = 2
      want_min($n) == $A126235_samples[$i] or die;
    }
  }

  {
    foreach my $n (2 .. 16) {
      print want_max($n),",";
    }
    print "\n";

    my @A126236_samples = (1,2,3, 3,4,4,5,5,5,5,6,6,6,6,7,7,7,7,7,7,7,7,8,8,8,8,8,8,8,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11,11);
    foreach my $i (0 .. $#A126236_samples) {
      my $n = $i+2; # A126236 OFFSET = 2
      want_max($n) == $A126236_samples[$i] or die;
    }
  }

  sub initial_pending {
    my ($n) = @_;
    return (map {[$_,0,0]} reverse 1 .. $n);
  }

  $verbose = 0;
  foreach my $n (
                 2 .. 10000
                ) {
    if ($verbose) {
      print "n=$n\n";
    }
    my @pending = initial_pending($n);
    my $range = $merge->(\@pending, $#pending-1,$#pending);
    my ($lo,$hi) = @$range;
    my $want_lo = want_min($n);
    my $want_hi = want_max($n);
    print "$n  range $lo $hi  cf $want_lo $want_hi  alts $count_alts  \r";
    if ($want_lo != $lo
        || $want_hi != $hi) {
      print "$n  range $lo $hi  cf $want_lo $want_hi  \n";
      die;
    }
  }
}
