/* Copyright 2019 Kevin Ryde

   This file is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3, or (at your option) any later
   version.

   This file is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with this file.  See the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>.
*/

/* #define NDEBUG 1 */
#include <assert.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#define WANT_DEBUG  0  /* set to 1 for debug prints */
#if WANT_DEBUG
#define DEBUG(expr)  expr
#else
#define DEBUG(expr)
#endif


#define LIMIT  100000
#define WANT_PROGRESS  1
typedef unsigned int  elem_t;
#define HALF   (UINT_MAX >> 1)
static elem_t n = 0;

int
check_revsorted (const elem_t *p, int len)
{
  assert(len <= LIMIT);
  int i;
  for (i = 0; i < len; i++) {
    if (p[i]==0) {
      printf("zero entry %d\n", i);
      abort();
    }
  }
  for (i = 1; i < len; i++) {
    if (p[i-1] < p[i]) {
      printf("not revsorted at i=%d  %d %d\n", i, p[i-1], p[i]);
      abort();
    }
  }
  return 1;
}

static inline int
insert_revsorted (elem_t *p, int len, int value)
{
  if (len >= LIMIT) {
    printf("stop at len=%d LIMIT\n", len);
    exit(1);
  }
  int i;
  for (i = len; i >= 1; i--) {
    if (p[i-1] >= value) {
      break;
    }
  }
  DEBUG({ printf("  store %d to i=%d move %d of len %d: ", value, i,len-i,len);
          int i;
          for(i=0; i<len; i++) printf(" %d",p[i]);
          printf("\n");
        });
  assert(i >= 0);
  assert(i < LIMIT);
  memmove(p+i+1, p+i, (len-i)*sizeof(elem_t));
  len++;
  p[i] = value;
  DEBUG({ printf("    gives len=%d  ", len);
          int i;
          for(i=0; i<len; i++) printf(" %d",p[i]);
          printf("\n");
        });
  assert(check_revsorted(p,len));
  return len;
}

static inline void
min_sums (const elem_t *ptr, int len)
{
  static elem_t list[LIMIT];
  assert(check_revsorted(ptr,len));
  memcpy(list, ptr, len*sizeof(elem_t));
  DEBUG({ printf("  min sums len=%d  ", len);
          int i;
          for(i=0; i<len; i++) printf(" %d",list[i]);
          printf("\n");
        });
  while (len >= 2) {
    assert(check_revsorted(list,len));
    if (len >= 3 && list[len-3] == list[len-2]) {
      printf("len=%d 3 same %d %d %d\n",
             len,
             (unsigned) list[len-3],
             (unsigned) list[len-2],
             (unsigned) list[len-1]);
      abort();
    }
    elem_t x = list[--len];
    elem_t y = list[--len];
    assert (x <= HALF);
    assert (y <= HALF);
    len = insert_revsorted(list,len, x+y);
  }
}

int
main (void)
{
  setbuf(stdout,NULL);
  printf("LIMIT %d\n", LIMIT);

  static elem_t list[LIMIT];
  int len = 0;
  int prev_len = 0;
  for (;;) {
    DEBUG({ printf("at n=%d len=%d  ", n, len);
            int i;
            for(i=0; i<len; i++) printf(" %d",list[i]);
            printf("\n");
          });

    if (len <= 2 || n <= list[len-2]) {
      n++;
      if (WANT_PROGRESS && (n & 0x7F) == 0) {
        printf("at n=%d\n", (int) n);
      }
      if(n>=3) {
        if (! (len-prev_len == (n%3==0))) {
          printf("different len incr n=%d\n", n);
          abort();
        }
      }
      /* if(n>=10) */
      /*   printf("%d,",len-prev_len); */
      prev_len=len;
      DEBUG(printf("  new n=%d\n",n));
      len = insert_revsorted(list,len, n);
      min_sums(list,len);
    } else {
      elem_t x = list[--len];
      elem_t y = list[--len];
      assert (x <= HALF);
      assert (y <= HALF);
      len = insert_revsorted(list,len, x+y);
    }
  }
  return 0;
}
