\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);


\\ A193405 primecodes perfect matching
\\ a193405-perfect.gp

\\-----------------------------------------------------------------------------

gpf(n) = if(n==1,1,Vecrev(factor(n)[,1])[1]);

\\      Ur          
\\    / | \         
\\   P  P  Uc       
\\         | \      
\\         P  P    
\\                
\\ P n vertices = Ur x vertices * Uc n-x vertices
\\ want Uc planted

P = List([ [] ]);
P_planted = List([ [] ]);
U = List([ [1] ]);
U_biggest = List([ [1] ]);
U_planted = List([ [2] ]);
limit = 100;
limitpi = primepi(limit);

extend() =
{
  my(n=#P+1, new,new_biggest);

  \\ planted P, any U[n-1] with root above
  new=List(U_planted[n-1]);
  new_biggest=List(U_planted[n-1]);

  \\ non-planted P, U[x] and below U[y]
  for(x=2,n-1,
     my(y=n-x);
     for(i=1,#U[x],
        for(j=1,#U_planted[y],
           my(p=U[x][i]*U_planted[y][j]);
           #vpar_from_primecode(p)==n || error();
           p<=limit || next;
           print("P new ",U[x][i]*U_planted[y][j], " = ",U[x][i]," * ",U_planted[y][j]"  sizes "x" "y);
           listput(new,p))));
  new=vecsort(Vec(new));
  print("P ",new);
  listput(P,new);
  listput(P_planted,[prime(p) | p<-new, p<=limitpi]);
  print("P_planted ",P_planted[n]);

  Set(P[n]) == vecsort(P[n]) || error("P not sorted");

  \\ planted U, any P[n-1] with root above
  new        =List([]);
  new_biggest=List([]);

  \\ not-planted U, P[x] added to non-singleton U[y] 
  for(x=1,n-2,
     my(y=n-x);
     for(i=1,#P_planted[x],
        for(j=1,#U[y],
           P_planted[x][i] >= U_biggest[y][j] || break;
           my(t=P_planted[x][i]*U[y][j]);
           t <= limit || next;
           print("U new ",t, " = ",P_planted[x][i]," * ",U[y][j],"  cf biggest "U_biggest[y][j]);
           #vpar_from_primecode(t)==n || error();
           if(setsearch(Set(new),t),error("already exists"));
           listput(new,        t);
           listput(new_biggest,P_planted[x][i]);
           gpf(t)==P_planted[x][i] || error();
        )));

  \\ planted U, any P[n-1] with root above
  new        =concat(Vec(new),        P_planted[n-1]);
  new_biggest=concat(Vec(new_biggest),P_planted[n-1]);

  print("n="n" made U  ",new);
  print("n="n" biggest ",new_biggest);
  listput(U,new);
  listput(U_biggest,new_biggest);
  listput(U_planted,[prime(u) | u<-U[n], u<limitpi]);

  if(vecsort(U_biggest[n]) != U_biggest[n],
     error("U_biggest not sorted"));
  if(#U[n] != #Set(U[n]),
     error("U not distinct"));
}
{
  for(i=1,28,
     extend();
     my(n=#P);
     \\ if(#P[n]==[] && U[n]==[],
     \\    print("stop i="i);
     \\    break);
  );
}

is_P(n) = vpar_has_perfect_matching(vpar_from_primecode(n));
is_U(n) = vpar_has_perfect_matching(vpar_delete_vertex(vpar_from_primecode(n),1));
want_P = vector(#P,num, select(n->#vpar_from_primecode(n)==num && is_P(n), [1..limit]))
want_U = vector(#P,num, select(n->#vpar_from_primecode(n)==num && is_U(n), [1..limit]))

print("P      ",Vec(P))
print("want P ",want_P)
print("U      ",Vec(U))
print("want U ",want_U)

print("counts ",apply(length,P));

{
  for(i=1,#P,
     if(vecsort(P[i])!=want_P[i],
        print();
        print("got  ",vecsort(P[i]));
        print("want "want_P[i]);
        error("P different i=", i));
     if(vecsort(U[i])!=want_U[i],
        print();
        print("got  ",vecsort(U[i]));
        print("want "want_U[i]);
        error("U different i=", i));

     for(j=2,#U[i],
        gpf(U[i][j]) == U_biggest[i][j] || error());
     );
}
\\ prime(27) == 103
\\ vpar_from_primecode(103)
\\ [0, 1, 2, 3, 2, 5, 2, 7]
\\ 1
\\ |
\\ 2
\\ | \  \
\\ 3  5  7
\\ |  |  |
\\ 4  6  8

\\ 309 == 3*103
\\ prime(27) == 103
\\ vpar=vpar_from_primecode(309)
\\ vpar_to_primecode(vpar_delete_vertices(vpar,[1,2,3])) == 27
\\  1  2  3  4  5  6  7
\\ [0, 1, 2, 1, 4, 5, 4, 7, 4, 9]    sizes 2, 6
\\ 1      
\\ | \ 
\\ 2  4
\\ |  | \ \
\\ 3  5  7  9
\\    |  |  |
\\    6  8  10

\\ 507 = 3*13*13
\\ vpar=vpar_from_primecode(507)
\\ [0, 1, 2, 1, 4, 4, 6, 1, 8, 8, 10]
\\ 1      
\\ | \     \
\\ 2  4     8
\\ |  | \   | \
\\ 3  5  6  9 10
\\       |     |
\\       7    11

quit



\\-----------------------------------------------------------------------------
\\ Perfect matched trees are relatively rare.  To find those up to a given
\\ maximum tree number, a good strategy is to build rather than test
\\ individually.

\\  each n.  One way is to build by height (distance root to
\\ deepest descendent).

\\     U        U[h+1]
\\   / | \      
\\  P  P  P     U <= h
\\        |     multiply prime(P) 
\\        |     of all P = h
\\       h-1

\\      Ur            U         Uc
\\    / | \         -----     -------
\\   P  P  Uc       <= h+1    prime(P = h)      any, big
\\   |     | \       = h+1    prime(P <= h-1)   big, small
\\   |     P  P    
\\   |             
\\   h             

limit = 500;

{
  for(h=1,8,
     print1("h="h" U ");
     for(n=1,limit,
        my(vpar=vpar_from_primecode(n));
        if(vpar_height(vpar)==h-1
             && vpar_has_perfect_matching(vpar_delete_vertex(vpar,1)),
           print1(n,",")));
     print();
     print1("    P ");
     for(n=1,limit,
        my(vpar=vpar_from_primecode(n));
        if(vpar_height(vpar)==h-1 && vpar_has_perfect_matching(vpar),
           print1(n,",")));
     print();
  );
}

U   = List([ [1] ]);
Upr = List([ [2] ]);
P   = List([ []  ]);
Ppr = List([ []  ]);
h = 1;
extend_U() =
{
  my(new=List(concat(U)),
     pre=#new);
  for(i=1,#Ppr[h],
     my(n=1);
     while(n<=#new,
           my(c=Ppr[h][i]*new[n]);
           if(c<=limit, listput(new,c));
           n++));
  new=vecsort(Vec(new)[pre+1..#new]);
  listput(U,new);
  listput(Upr,apply(prime,new));
}

extend_P() =
{
  my(new=List([]),
     le = concat(U));            \\ Ur <= h+1
  for(i=1,#le,
     for(j=1,#Upr[h],            \\ Uc = h
        my(c=le[i]*Upr[h][j]);
        if(c<=limit, listput(new,c))));

  if(h>=2,
     le = concat(Upr[1..h-1]);      \\ Uc <= h-1
     for(i=1,#le,
        for(j=1,#U[h+1],            \\ Ur = h+1      
           my(c=le[i]*U[h+1][j]);
           if(c<=limit, listput(new,c))));
    );

  new=vecsort(Vec(new));
  listput(P,new);
  listput(Ppr,apply(prime,new));
}

{
  while(1,
        extend_U();
        extend_P();
        h++;
        print("U  "U[h]);
        print("   "P[h]);
        if(U[h]==[] && P[h]==[],break));
}

print();
U=vecsort(concat(Vec(U)));
P=vecsort(concat(Vec(P)));
Set(U)==U
Set(P)==P
print(P);
print(U);

want_P = select(n->vpar_has_perfect_matching(vpar_from_primecode(n)),[1..limit]);
want_U = select(n->vpar_has_perfect_matching(vpar_delete_vertex(vpar_from_primecode(n),1)),[1..limit]);

P == want_P
U == want_U

nearly = select(n->vpar_has_perfect_matching(vpar_from_primecode(n),1),[1..limit]);
print(nearly)
quit


\\-----------------------------------------------------------------------------
P=List([]);
Pupto=List([]);
primeP=List([]);
U     =List([1]);
primeU=List([2]);
Uupto =List([1]);
limit = 40;

{
  my(u=1,more=1);
  while(more,
        more=0;
        for(u=1,#Uupto,
           for(i=Uupto[u],#U,
              my(t=U[u]*primeU[i]);
              if(t<=limit,
                 print("new P ",t," from u="u" i="i);
                 listput(P,t);
                 listput(primeP,prime(t));
                 listput(Pupto,1);
                 more=1));
           Uupto[u]=#U+1);
        
        for(p=1,#Pupto,
           my(i=Pupto[p]);
           while(i<=#U,
                 my(t=U[i]*primeP[p]);
                 if(t<=limit,                 
                    print("new U ",t," = "U[i]"*"primeP[p]" at Ui="i" Pp="p,if(setsearch(Set(U),t),"  ***duplicate",""));
                    listput(U,t);
                    listput(primeU,prime(t));
                    listput(Uupto,1);
                    more=1);
                 i++);
           Pupto[p]=i));
}
print("P upto   ",Vec(Pupto));
print("P primes ",Vec(primeP));
print("U primes ",Vec(primeU));
P=Vec(P);
U=Vec(U);
want_P = select(n->vpar_has_perfect_matching(vpar_from_primecode(n)),[1..limit]);
want_U = select(n->vpar_has_perfect_matching(vpar_delete_vertex(vpar_from_primecode(n),1)),[1..limit]);
print();
print("got  P ",vecsort(P));
print("want P ",want_P);
print("got  U ",vecsort(U));
print("want U ",want_U);
vecsort(P) == want_P
vecsort(U) == want_U
vpar_from_primecode(3)
vpar_from_primecode(5)
quit




\\-----------------------------------------------------------------------------
           
\\ {
\\   my(v=A193405_vector(200000));
\\   print(#v);
\\   system("rm /tmp/b.txt");
\\   for(n=1,#v,
\\      write("/tmp/b.txt",n,v[n]));
\\   system("ls -l /tmp/b.txt");
\\ }
\\ {
\\   system("rm /tmp/b.txt");
\\   my(n=1,count=0);
\\   while(count < 5000,
\\         if(is_A193405(n),
\\            count++;
\\            write("/tmp/b.txt",Str(count" "n)));
\\         n++);
\\   system("ls -l /tmp/b.txt");
\\ }
