\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(parisizemax,500*10^6);
default(recover,0);
default(strictargs,1);

option_verbose=0;
option_progress=1;

read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("a347966-maximal-matchings.gp");


\\-----------------------------------------------------------------------------
\\ Quantities Combinations

quantities_singleton = [0,1,1];

quantities_validate(v) =
{
  #v==3 || error();
  my(with,without_max,without);
  [with,without_max,without] = v;

  type(with)        == "t_INT"   || error();
  type(without_max) == "t_INT"   || error();
  type(without)     == "t_INT"   || error();

  with >= 0               || error();
  without >= without_max  || error();
}
quantities_validate(quantities_singleton);

quantities_total(v) = v[1]+v[2];
{
  check_equal(quantities_total(quantities_singleton), 1,
              "quantities_total() of singleton");
}


quantities_accumulate(v,c) =
{
  \\ print("quantities_accumulate() "v" + "c);
  quantities_validate(v);
  quantities_validate(c);

  my(c_with,c_without_max,c_without);
  [c_with,c_without_max,c_without] = c;
  my(c_max = c_with + c_without_max);

  my(v_with,v_without_max,v_without);
  [v_with,v_without_max,v_without] = v;

  my(new_without_max = v_without_max * c_with,
     new_without     = v_without     * c_max,
     new_with        = v_with    * c_max
                     + v_without * c_without);

  my(ret = [new_with, new_without_max, new_without]);
  \\ print("  ret ",ret);
  quantities_validate(ret);
  ret;
}

{
  my(v=['f,'g,'h],
     c=['x,'y,'z]);
  local(quantities_validate=(v)->);
  my(acc1=quantities_accumulate(v,c),
     xy='x+'y);
  check_equal(simplify(acc1 -
                       [f*xy + z*h,
                        g* x,
                        h*xy]),
              [0,0,0]);
  my(acc2=quantities_accumulate(acc1,c));
  check_equal(simplify(acc2 -
                       [(f*xy + z*h)*xy + h*xy*z,
                        g* x^2,
                        h*xy^2]),
              [0,0,0],
              "acc2");
  my(acc3=quantities_accumulate(acc2,c));
  check_equal(simplify(acc3 -
                       [((f*xy + h*z)*xy + h*xy*z)*xy + h*xy^2*z,
                        g* x^3,
                        h*xy^3]),
              [0,0,0],
              "acc3");
  check_equal(simplify(acc3 -
                       [f*xy^3 + 3*h*xy^2*z,
                        g* x^3,
                        h*xy^3]),
              [0,0,0],
              "acc3");

  my(acc4=quantities_accumulate(acc3,c));
  check_equal(simplify(acc4 -
                       [f*xy^4 + h*4*xy^3*z,
                        g* x^4,
                        h*xy^4]),
              [0,0,0],
              "acc4");

  my(v=['v_with,'v_without_max,'v_without],
     c=['c_with,'c_without_max,'c_without],
     c_max='c_with+'c_without_max);
  check_equal(simplify(quantities_accumulate(v,c) - 
                       ['v_with    * c_max
                      + 'v_without * 'c_without,
                        'v_without_max * 'c_with,
                        'v_without     * c_max]),
              [0,0,0],
              Str("acc symbolic one"));

  my(acc=v);
  for(rep=1,20,
     acc=quantities_accumulate(acc,c);
     check_equal(simplify(acc -
                          ['v_with    * c_max^rep
                           + 'v_without * rep * c_max^(rep-1) * 'c_without,
                           'v_without_max * 'c_with^rep,
                           'v_without     * c_max^rep]),
                 [0,0,0],
                 Str("acc symbolic rep=",rep));
  );
}

quantities_of_vpar_tree(vpar) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     qvec=vector(#vpar,v, quantities_singleton));
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v]);
     if(p,
        qvec[p] = quantities_accumulate(qvec[p], qvec[v]);
        ,
        return(qvec[v])));
  error();
}
check_equal(quantities_of_vpar_tree([0]), [0,1,1]);

test_maximal_matchings_count_by_quantities(vpar) = \
  quantities_total(quantities_of_vpar_tree(vpar));

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     #vpar || next;
     \\ print(vpar);
     check_equal(test_maximal_matchings_count_by_quantities(vpar),
                 vpar_maximal_matchings_count(vpar),
                 "test_maximal_matchings_count_by_quantities() sample");
  );
}
{
  for(rep=1,1000,
     my(len=random(100),
        vpar=vpar_make_random(len));
     #vpar || next;
     check_equal(test_maximal_matchings_count_by_quantities(vpar),
                 vpar_maximal_matchings_count(vpar),
                 "test_maximal_matchings_count_by_quantities() random"));
}


\\-----------------------------------------------------------------------------
\\ A347966()

{
  my(want=OEIS_data("A347966"));
  check_equal(vector(#want,n, A347966(n)),
              want,
              "A347966() vs samples");

  if(0,
     system("rm -f /tmp/b347966.txt");
     for(n=1,10000, 
        write("/tmp/b347966.txt", Str(n," ",A347966(n))));
     system("ls -l /tmp/b347966.txt");
     quit);

  \\ ~/OEIS/b347966.txt
  if(0,
     my(want=OEIS_data("A347966",'bfile));
     print("b347966.txt length ",#want);
     check_equal(vector(#want,n, A347966(n)),
                 want,
                 "A347966() vs ~/OEIS/b347966.txt"));
}

A347966_by_vpar(n) = vpar_maximal_matchings_count(vpar_from_primecode(n));
{
  check_equal(A347966_by_vpar(15), 4);
  check_equal(A347966(15),         4);

  check_equal(vector(1000,n, A347966(n)),
              vector(1000,n, A347966_by_vpar(n)),
              "A347966() vs vpar");

  for(rep=1,5000,
     my(len=random(10),
        n=random(10^len)+1);
     n>=1 || error();
     \\ print(n);
     check_equal(A347966(n), A347966_by_vpar(n),
              "A347966() vs vpar random"));
}


\\-----------------------------------------------------------------------------
check_equal(b,'b);
check_equal(c,'c);
check_equal(f,'f);
check_equal(n,'n);
check_equal(idx,'idx);
check_equal(got,'got);
check_equal(ret,'ret);
check_equal(upto,'upto);
check_equal(vec,'vec);
check_equal(vpar,'vpar);
check_equal(want,'want);
check_equal(with,'with);
print("end a347966-maximal-matchings--test.gp");


\\ Alternative with p*c for scalar product accumualte, though this is then 5
\\ multiplies instead of 4.
  \\ my(p=[1,1,1],      \\ all_max, without, without_max
  \\    f=factor(n));
  \\ for(i=1,matsize(f)[1],
  \\    my(c=self()(primepi(f[i,1]))~,c_with=c[1]-c[3]); c[3]=-c[3];
  \\    for(j=1,f[i,2],
  \\       p[1] = p*c;        \\ scalar product
  \\       p[2] *= c[1];      \\ child "all_max"
  \\       p[3] *= c_with;    \\ child "with"
  \\    ));
  \\ p;
  \\ maximal_matchings_count(n) = triplet(n)[1];

\\ one-by-one child repetitions
\\
     \\ my(c=A347966_maximal_matchings_quantities(primepi(f[i,1])),
     \\    c_max = c[1] + c[2]);     \\ c maximals = with + without_max
     \\ \\ accumulate repetitions of the child
     \\ for(j=1,f[i,2],
     \\    with = with * c_max       \\ v already matched, c any maximal
     \\           + without * c[3];  \\ v and c new pair, both "without"
     \\    without     *= c_max;     \\ child must be maximal
     \\    without_max *= c[1]);    \\ child must be "with"

