#!/usr/bin/perl -w

# Copyright 2022 Kevin Ryde
# 
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
# 
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.

use strict;
use Math::Factor::XS 'prime_factors';
use List::Util 'uniqstr';
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;

my @want = (1, 2, 3, 5, 11, 31, 127, 527, 1829, 7493, 35179, 196393, 1266983, 6297041,
           35154347);

# vpar_from_primecode(35154347)
# [0,1,2,3,4,4, 6,
#  1,8,9,10,11,11,
#  1,14,15,16,17,18,19]

my $maxterm   = 1_000_000_000;
my $cache_max =     5_000_000;

# $cache[17] = "4,7,17" means tree n=17 has this list of distinct subtree
# numbers.  Singleton subtree 1 is omitted.
#
my @cache = (undef,
             '');  # row 1

# $primepi{$p} = index of prime $p.
my %primepi = (2 => 1,
               3 => 2,
               5 => 3);

# Return a string of comma separated terms '2,3'.
# Subtree number 1 is omitted.
sub row_cached {
  my ($n,$f) = @_;
  ### row_cached(): $n
  if (defined $cache[$n]) {
    ### from cache(): $cache[$n]
    return $cache[$n];
  }
  my @f = uniqstr(defined $f ? @$f
                  : prime_factors($n));

  my %ret;  # distinct subtree numbers
  foreach my $p (uniqstr(prime_factors($n))) {
    my $child = $primepi{$p};
    ### prime: "p=$p child=$child"
    if ($child > 1) {
      # hash slice
      @ret{split /,/, row_cached($child)} = ();
    }
  }

  ### %ret
  my $ret = join(',', keys(%ret), $n);
  if ($n <= $cache_max) {
    ### store cache: "$n = $ret"
    $cache[$n] = $ret;
  }
  return $ret;
}

{
  foreach my $n (1 .. 6) {
    print "$n = \"",row_cached($n),"\"\n";
  }
  row_cached(2) eq '2' or die;
}

my $target = 0;
my $primepi_seen = 0;
N: foreach my $n (1 .. $maxterm) {
  my @f = prime_factors($n);
  if (@f == 1) {  # a prime
    $primepi{$n} = ++$primepi_seen;
  }
  foreach my $i (0 .. $#f-1) {
    next N if $f[$i] == $f[$i+1];  # not square-free
  }
  my $str = row_cached($n,\@f);
  my $num = $str =~ tr/,/,/;
  if ($num == $target) {
    $target++;
    print "found $target at $n\n";
  }
}

print "end\n";
