\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);

read("a357702-CP-depths-total.gp");

read("OEIS-data.gp");
read("OEIS-data-wip.gp");
OEIS_check_verbose = 1;
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("various-CP.gp");
option_verbose = 0;


\\-----------------------------------------------------------------------------

got = vector(1000,n,a(n));
{
  check_equal(got,
              vector(#got,n,vecsum(vpar_depths_vector(vpar_from_CPcode(n)))),
              "CP_depths_total() vs vpar_depths_vector()");
}


\\-----------------------------------------------------------------------------
\\ a(n)

{
  \\ formula
  check_equal(vector(100,n,n++; a(n)),
              vector(100,n,n++; 
    my(x=A002024_k_rep_k(n-1),
       y=A002260_runs_1_to_k(n-1));
    a(x) + a(y) + A064002_CP_num_vertices(n) - 1));

  \\ DATA
  my(want=[0, 2, 6, 10, 12, 16, 22, 18, 22, 28, 34, 20, 24, 30, 36, 38, 26, 30, 36, 42, 44, 50, 34, 38, 44, 50, 52, 58, 66, 28, 32, 38, 44, 46, 52, 60, 54, 34, 38, 44, 50, 52, 58, 66, 60, 66, 42, 46, 52, 58, 60, 66, 74, 68, 74, 82, 50, 54, 60, 66, 68, 74, 82, 76, 82, 90]);
  check_equal(vector(#want,n, a(n)), want);
  if(0,
     for(n=1,70,
        print1(a(n),",");
        if(n==25||n==50,print()));
     print();
     quit);

  \\ example picture
  my(n=3,
     vpar=vpar_from_CPcode(n));
  check_equal(vpar,[0,1,2,2,1]);
  check_equal(vpar_depths_vector(vpar),
              [0,1,2,2,1]);
  check_equal(a(3), 6);
  
}

\\-----------------------------------------------------------------------------
\\ By Total and Num Vertices

CP_total_depths_and_num_vertices(n) =
{
  if(n==1, [0,1],
     my(v=vecsum(apply(CP_total_depths_and_num_vertices,
                      CP_children_z(n))));
     [v[1]+v[2], v[2]+1]);
}
{
  check_equal(vector(1000,n, CP_total_depths_and_num_vertices(n)),
              vector(1000,n, [a(n), A064002_CP_num_vertices(n)]));
}
  

\\-----------------------------------------------------------------------------
\\ Extra Ideas

{
  if(0,
     \\ num ways k = a(n) occurs
     my(histogram=vector(1+2+3+4+5)); 
     for(n=2,oo,
        if(CP_height(n) > 5, break);
        my(t=a(n)/2);
        if(t<=#histogram, histogram[t]++));
     print(histogram);
     \\ not in OEIS: 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 2, 1, 1, 2, 3
     quit);

  if(0,
     print(vector(20,n,n++; CP_total_depths(n)));
     \\ not in OEIS: 2, 6, 10, 12, 16, 22, 18, 22, 28, 34, 20, 24, 30, 36, 38, 26, 30, 36, 42, 44

     \\ record highs
     my(m=0,nlist=List([]),tlist=List([]));
     for(n=1,100, my(t=a(n));
        if(t>m, listput(nlist,n); listput(tlist,t); m=t));
     print(Vec(nlist));
     print(Vec(tlist));
     \\ not in OEIS: 2, 3, 4, 5, 6, 7, 10, 11, 15, 16, 20, 21, 22, 27, 28, 29, 53, 56, 66, 67
     \\ not in OEIS: 2, 6, 10, 12, 16, 22, 28, 34, 36, 38, 42, 44, 50, 52, 58, 66, 74, 82, 90, 98
     quit);
}


\\-----------------------------------------------------------------------------
check_equal(c,'c);
check_equal(k,'k);
check_equal(n,'n);
check_equal(v,'v);
check_equal(x,'x);
check_equal(y,'y);
check_equal(vpar,'vpar);
print("end, ",check_count," checks");
