# Copyright 2019 Kevin Ryde

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


use 5.005;
use strict;
use FindBin;
use File::HomeDir;
use File::Spec;
use File::Temp;
use File::Which;
use Graph::Reader::Graph6;
use IPC::Run;

use lib '../../gmaker/devel/lib';
use MyGraphs;

# uncomment this to run the ### lines
use Smart::Comments;


my $all_filename = File::Spec->catfile(File::HomeDir->my_home,'HOG','all.g6');
my $out_filename = '/tmp/all.g6';
my @pickg_options = (
                     '-g0',   # acyclics, girth=0
                    );

my $pickg_program = 'pickg';
if (! defined(File::Which::which($pickg_program))
    && defined(File::Which::which('nauty-pickg'))) {
  $pickg_program = 'nauty-pickg';
}

my $temp_fh = File::Temp->new;
IPC::Run::run([ $pickg_program, @pickg_options ],
              '<',$all_filename,
              '>',$temp_fh);

seek $temp_fh, 0, 0 or die;
open my $out, '>', $out_filename or die;
my $count = 0;
my $reader = Graph::Reader::Graph6->new;
while (my $graph = $reader->read_graph($temp_fh)) {
  # print $graph,"\n";
  # MyGraphs::Graph_view($graph);
  print $out "try(",MyGraphs::Graph_vpar_str($graph),");\n";
  $count++;  
  # $count <= 1 or die;
}
print "$count to $out_filename\n";
close $out or die;

exit 0;
