\\ Copyright 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("memoize.gp");
setrand(getwalltime()%2^32);


progress_prev=-1;
progress(str) =
{
  my(t=getwalltime()\2000);
  if(t!=progress_prev,
     progress_prev=t;
     print(str));
}

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong: "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

min_child_runs(vpar) =
{
  my(min_child=vpar_minmax_child_vector(vpar,min),
     second=vectorsmall(#vpar));
  for(v=1,#vpar,
     if(min_child[v], second[min_child[v]]=1));
  my(ret=List([]));
  for(i=1,#vpar,
     my(v=i);
     if(!second[v], 
        my(l=List([v]));
        while(v=min_child[v], listput(l,v));
        listput(ret,Vec(l))));
  Vec(ret);
}

\\ make_all_sets(n) =
\\ {
\\   if(n==0, [ [] ],
\\      my(ret=Map());
\\      for(k=1,n,
\\         forvec(v=vector(n,i,[1,k]),
\\               my(sets=vector(k,i,List([])));
\\               for(i=1,#v, listput(sets[v[i]],i));
\\               sets=apply(Vec,sets);
\\               sets=vecsort(sets);
\\               if(#sets[1]==0,next());
\\               \\ print(sets);
\\               mapput(ret,sets,1)));
\\      Vec(Mat(ret)[,1]));
\\ }
\\ check_exact(#make_all_sets(0),  1);
\\ check_exact(#make_all_sets(1),  1);
\\ check_exact(#make_all_sets(2),  2);
\\ check_exact(#make_all_sets(3),  5);
\\ check_exact(#make_all_sets(4), 15);
\\ vector(8,n, #make_all_sets(n))


{
  for(n=1,4,
     my(l=test_make_ordered_partitions(n));
     for(i=1,#l,
        my(p=l[i]);
        !test_vector_search(p[1],1) || error()));
}

\\------------------------------------------------------------------------------

\\ f>min then new min must reach min=2

is_partition_root1(p) =
{
  if(#p==0,return(1));
  my(path1=p[1]);
  if(#path1==0,error());
  if(#path1==1,return(#p==1));
  if(#path1<2,return(0));
  if(path1[1] != 1, return(0));
  my(m=vecmin(path1[2..#path1]),
     seen=vector(#p),
     more=1);
  while(more,
        more=0;
        for(i=2,#p,
           if(seen[i],next());
           if(p[i][1]>m,
              m=min(m,vecmin(p[i]));
              seen[i]=1;
              more=1));
        );
  \\ print("final seen "seen);
  vecsum(seen)==#p-1;  
}

test_make_ordered_partitions(2)
select(is_partition_root1,test_make_ordered_partitions(2))

{
check_exact(vector(8,n,n--; #select(is_partition_root1,test_make_ordered_partitions(n))),
            [1, 1,1,3,14,86,646,5694]);
}
\\ {
\\ for(n=0,7,
\\    print(#select(is_partition_root1,test_make_ordered_partitions(n))));
\\ }


\\ labelled ordered partitions where 1 is the first of a list
\\ 1 2 7 34 209 1546 13327 130922
\\ A002720 partial perms of set
{
  for(n=1,8,
     print(#select(p->p[1][1]==1,test_make_ordered_partitions(n))));
}
quit
           




\\------------------------------------------------------------------------------

\\ run containing 1 minimum m
\\ other runs f > m or f > minimum of all


\\ first run 1,m length l
\\ next run f>m

R(n) =
{
  my(debug=1);
  n>=0 || error;
  if(n==1, return(1));
  sum(l=2,n,
      sum(m=2,n,
          if(debug,print("R l="l" m="m));
          binomial(n-m,l-2) * (l-1)! * RM(n-l,m-2)));
}
RM_indent=" ";
RM(n,m) =
{
  my(debug=1);
  local(RM_indent=concat(" ",RM_indent));
  if(debug,print(RM_indent"RM n="n" m="m));
  n>=0 || error("negative n");
  m>=0 || error("negative m");
  \\ m<=n || error("m too big");

  if(n==0,return(1));
  if(n==1,
     if(debug,print(RM_indent" single ",(m==0)));
     return(m==0));
  if(m>n,return(0));

  my(ret =
  sum(f=m+1,n,
      sum(l=2,n,
          sum(t=1,m,
              my(bin = binomial(n-m,l-2) * (l-1)!,
                 r = RM(n-l,t-1));
              if(debug,print(RM_indent" f="f" l="l" t="t" bin="bin
                             " down "n-l","t-1" = r"));
              bin * r)))
  +
  sum(f=m+1,n,
      sum(l=1,n,
              my(bin = binomial(n-m,l-1) * (l-1)!,
                r = RM(n-l,m));
              if(debug,print(RM_indent" f="f" l="l" bin="bin
                             " down "n-l","m" = "r));
          bin * r))
);
  if(debug,print("  ret "ret));
ret;
}
RM=memoize(RM);

check_exact(RM(0,0), 1);
check_exact(RM(1,0), 1);
check_exact(RM(2,0), 3);
RM(2,0)
quit

R(3)
R(4)
\\ R(5)
\\ print("r ",vector(10,n, R(n)));
quit

\\ want root=1  1, 1, 3, 14,  86,  646, 5694, 57354

\\ n=4 root=1
\\ [0, 1, 1, 1]  [[1, 2], [3], [4]]     l=2
\\ [0, 1, 1, 3]  [[1, 2], [3, 4]]
\\ [0, 1, 4, 1]  [[1, 2], [4, 3]]
\\ [0, 4, 1, 1]  [[1, 3], [4, 2]]
\\ [0, 1, 1, 2]  [[1, 2, 4], [3]]       l=3
\\ [0, 1, 2, 1]  [[1, 2, 3], [4]]
\\ [0, 3, 1, 1]  [[1, 3, 2], [4]]
\\ [0, 4, 4, 1]  [[1, 4, 2], [3]]
\\ [0, 1, 2, 3]  [[1, 2, 3, 4]]         l=4
\\ [0, 1, 4, 2]  [[1, 2, 4, 3]]
\\ [0, 3, 1, 2]  [[1, 3, 2, 4]]
\\ [0, 4, 1, 3]  [[1, 3, 4, 2]]
\\ [0, 4, 2, 1]  [[1, 4, 2, 3]]
\\ [0, 3, 4, 1]  [[1, 4, 3, 2]]



\\------------------------------------------------------------------------------
\\ A000262 sets of lists

vpar_INTERNAL_exponential_transform(vector(10,n,n!))

vector(10,n, sum(k=0,n-1, binomial(n,k)*(n-1)!/(n-k-1)!))

a(n) = if(n<=1,1, (2*n-1)*a(n-1) - (n-1)*(n-2)*a(n-2));
vector(10,n, a(n))

\\ Stephen Dalton
D(n,k) =
{
  \\ print("D("n","k")");
  n>=0 || error();
  k>=0 || error();
  if(n==0 && k==0, return(1));
  if(n==0 && k!=0, return(0));
  if(n!=0 && k==0, return(0));
  D(n-1,k-1)             \\ own list
  + (n+k-1) * D(n-1,k);  \\ in other list
}
check_exact(D(0,0), 1, "D(0,0)");
check_exact(D(1,0), 0);
check_exact(D(2,0), 0);
check_exact(D(0,1), 0, "D(0,1)");
check_exact(D(1,1), 1);
check_exact(D(2,1), 2);
a(n) = sum(k=0,n,D(n,k));
vector(10,n, a(n))

F(n,k) = if(n==0||k==0,0, k*D(n-1,k) + D(n-1,k-1));
S(n,k) = if(n==0||k==0,0, (n-1)*D(n-1,k));
FS(n,k) = if(n<2||k<1, 0, k*S(n-1,k) + S(n-1,k-1));
FF(n,k) =
{
  if(n<2,return(0));
  if(k<2,return(0));

  k*F(n-1,k) + F(n-1,k-1);

  \\ binomial(k,2)*D(n-2,k)
  \\ + if(k>=1, 2*(k-1)*D(n-2,k-1))
  \\ + if(k>=2, D(n-2,k-2));
}
vector(10,n, sum(k=0,n, F(n,k)+S(n,k)))

check_exact(matrix(10,10,n,k,n++;k++; FS(n,k)+FF(n,k)), \
            matrix(10,10,n,k,n++;k++; F(n,k)));

trees(n) = if(n==0,1, n==1,1, n==2,2, sum(k=0,n, FS(n,k) + S(n,k)));
trees(n) = if(n==0,1, n==1,1, n==2,2, sum(k=0,n, D(n,k) - FF(n,k)));
print("tr ",vector(10,n, trees(n)))

print("fs ",vector(10,n, sum(k=0,n, FS(n,k))));
print("s  ",vector(10,n, sum(k=0,n, S(n,k))));
print("f  ",vector(10,n, sum(k=0,n, F(n,k))));

print("! ",vector(10,n, sum(f=1,n, f!*a(n-f))))
\\ {
\\   n>=0 || error();
\\   k>=0 || error();
\\   if(n==0, 1,
\\      k==0, 0,
\\      k>n, 0,
\\      k*L(n-1,k) + L(n-1,k-1));
\\ }
\\ A(n) = sum(k=1,n,L(n,k));
\\ vector(10,n, A(n))

\\        n = 1  2  3   4
\\ want trees 1, 2, 9, 53, 378, 3151, 30000
\\ or root=1  1, 1, 3, 14,  86,  646, 5694, 57354
\\               1  6  39  292  2505

\\               n-m-(l-2) after path1 vertices
\\ +-----------+----------+
\\ | 1 ----- m |   n-m    |
\\ +-----------+----------+
\\ run 1,m,... length l vertices
\\   l chosen after m is binomial(n-m,l-2)
\\   l ordered ordered (l-1)!
\\ m-2 initials as seconds, place first under any of n-m-(l-2) = n-m-l+2
\\   last under n-m-(l-2)+(m-2)-1 = n-l-1
\\ no, 3..m can be siblings of 2, so can top runs
\\ top next run > m so under run1
\\ top next run > m preceding so under
root(n) =
{
  my(debug=0);                          \\ WRONG
  if(debug,print("root n="n));
  if(n==1, return(1));
  sum(l=2,n,
     if(debug,print(" l="l" factorial ",(max(1,l-1))!));
     sum(m=2, n,
        my(bin=(max(1,l-1))! * binomial(n-m,l-2),
           p=prod(i=3,m, (n-i-(l-2))),
           rest_len=n-m-l+2,
           rest=a(rest_len),
          term=bin*p*rest);
        if(debug,
           print("  m="m" bin "bin" prod "p" rest len "rest_len" is "rest
                 " term "term));
        term));
}
root(5)
\\ print("r ",vector(10,n, root(n)));



\\ n=3 root=1
\\ [0, 1, 1]  [[1, 2], [3]]      2 -> 1 <- 3
\\ [0, 1, 2]  [[1, 2, 3]]        1 <- 2 <- 3
\\ [0, 3, 1]  [[1, 3, 2]]        1 <- 2 <- 3

\\ n=5 root=1
\\ [[1, 2], [3], [4], [5]]
\\ [[1, 2], [3], [4, 5]]
\\ [[1, 2], [3], [5, 4]]
\\ [[1, 2], [3, 4], [5]]
\\ [[1, 2], [3, 4, 5]]
\\ [[1, 2], [3, 5], [4]]
\\ [[1, 2], [3, 5, 4]]
\\ [[1, 2], [4], [5, 3]]
\\ [[1, 2], [4, 3], [5]]
\\ [[1, 2], [4, 3, 5]]
\\ [[1, 2], [4, 5, 3]]
\\ [[1, 2], [5, 3, 4]]
\\ [[1, 2], [5, 4, 3]]
\\
\\ [[1, 3], [4], [5, 2]]
\\ [[1, 3], [4, 2], [5]]
\\ [[1, 3], [4, 2, 5]]
\\ [[1, 3], [4, 5, 2]]
\\ [[1, 3], [5, 2, 4]]
\\ [[1, 3], [5, 4, 2]]
\\
\\ [[1, 4], [3], [5, 2]]
\\ [[1, 4], [5, 2, 3]]
\\ [[1, 4], [5, 3, 2]]
\\
\\ [[1, 2, 3], [4], [5]]
\\ [[1, 2, 3], [4, 5]]
\\ [[1, 2, 3], [5, 4]]
\\ [[1, 2, 3, 4], [5]]
\\ [[1, 2, 3, 4, 5]]
\\ [[1, 2, 3, 5], [4]]
\\ [[1, 2, 3, 5, 4]]
\\ [[1, 2, 4], [3], [5]]
\\ [[1, 2, 4], [3, 5]]
\\ [[1, 2, 4], [5, 3]]
\\ [[1, 2, 4, 3], [5]]
\\ [[1, 2, 4, 3, 5]]
\\ [[1, 2, 4, 5], [3]]
\\ [[1, 2, 4, 5, 3]]
\\ [[1, 2, 5], [3], [4]]
\\ [[1, 2, 5], [3, 4]]
\\ [[1, 2, 5], [4, 3]]
\\ [[1, 2, 5, 3], [4]]
\\ [[1, 2, 5, 3, 4]]
\\ [[1, 2, 5, 4], [3]]
\\ [[1, 2, 5, 4, 3]]
\\ [[1, 3, 2], [4], [5]]
\\ [[1, 3, 2], [4, 5]]
\\ [[1, 3, 2], [5, 4]]
\\ [[1, 3, 2, 4], [5]]
\\ [[1, 3, 2, 4, 5]]
\\ [[1, 3, 2, 5], [4]]
\\ [[1, 3, 2, 5, 4]]
\\ [[1, 3, 4], [5, 2]]
\\ [[1, 3, 4, 2], [5]]
\\ [[1, 3, 4, 2, 5]]
\\ [[1, 3, 4, 5, 2]]
\\ [[1, 3, 5], [4, 2]]
\\ [[1, 3, 5, 2], [4]]
\\ [[1, 3, 5, 2, 4]]
\\ [[1, 3, 5, 4, 2]]
\\ [[1, 4, 2], [3], [5]]
\\ [[1, 4, 2], [3, 5]]
\\ [[1, 4, 2], [5, 3]]
\\ [[1, 4, 2, 3], [5]]
\\ [[1, 4, 2, 3, 5]]
\\ [[1, 4, 2, 5], [3]]
\\ [[1, 4, 2, 5, 3]]
\\ [[1, 4, 3], [5, 2]]
\\ [[1, 4, 3, 2], [5]]
\\ [[1, 4, 3, 2, 5]]
\\ [[1, 4, 3, 5, 2]]
\\ [[1, 4, 5, 2], [3]]
\\ [[1, 4, 5, 2, 3]]
\\ [[1, 4, 5, 3, 2]]
\\ [[1, 5, 2], [3], [4]]
\\ [[1, 5, 2], [3, 4]]
\\ [[1, 5, 2], [4, 3]]
\\ [[1, 5, 2, 3], [4]]
\\ [[1, 5, 2, 3, 4]]
\\ [[1, 5, 2, 4], [3]]
\\ [[1, 5, 2, 4, 3]]
\\ [[1, 5, 3], [4, 2]]
\\ [[1, 5, 3, 2], [4]]
\\ [[1, 5, 3, 2, 4]]
\\ [[1, 5, 3, 4, 2]]
\\ [[1, 5, 4, 2], [3]]
\\ [[1, 5, 4, 2, 3]]
\\ [[1, 5, 4, 3, 2]]


\\------------------------------------------------------------------------------
\\ vpar_minmax_child_vector() occurring

\\ forests which have increasing max_child, decreasing same
\\ 1, 2, 5, 15, 52, 203, 877,
\\ A000110 Bell numbers, ways to partition n labelled elements
\\
\\ trees which have increasing max_child = same but 1 later
\\ -----

\\ forests distinct max_child vectors
\\ 1, 3, 13, 73, 501, 4051,
\\ A000262 count sets of lists, split 1..n into any number of ordered lists
\\ not A193932, it differs after a few terms
\\
\\ trees distinct max_child vectors
\\ not in OEIS: 1, 2, 9, 53, 378, 3151, 30000
\\ [1, 3, 13, 73, 501, 4051, 37633] - [1, 2, 9, 53, 378, 3151, 30000]
\\ not in OEIS: 0, 1, 4, 20, 123, 900, 7633
\\
\\ trees root=1 distinct max_child vectors
\\              1, 1, 3, 14, 86, 646, 5694, 57354
\\ not in OEIS:       3, 14, 86, 646, 5694, 57354

\\ trees distinct max_child vectors
\\ [0, 1] count 1      2 <- 1
\\ [2, 0] count 1      1 <- 2
\\
\\ [0, 0, 2]
\\ [0, 1, 2]
\\ [0, 3, 0]
\\ [0, 3, 1]
\\ [2, 0, 1]     3 <- 1 <- 2
\\ [2, 3, 0]
\\ [3, 0, 0]
\\ [3, 0, 2]
\\ [3, 1, 0]
\\ 9,
\\
\\ forests distinct max_child vectors
\\ [0, 0] count 1
\\ [0, 1] count 1
\\ [2, 0] count 1
\\            [0, 0, 0]      none have any children, all singletons
\\            [0, 0, 1]      3 <- 1 and 2  only
\\ [0, 0, 2]                 x2 ways
\\            [0, 1, 0]      2 <- 1 and 3  only
\\ [0, 1, 2]
\\ [0, 3, 0]
\\ [0, 3, 1]
\\            [2, 0, 0]      1 <- 2 and 3  only
\\ [2, 0, 1]
\\ [2, 3, 0]
\\ [3, 0, 0]                 2 -> 1 <- 3  and 1 <- 3, 2
\\ [3, 0, 2]
\\ [3, 1, 0]
\\ 13,
\\
\\ each c occur only once
\\ n as a child is always a max child, so n not occurring is root n

{
  for(n=5,5,
     my(m=Map());
     my(l=List([]));
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              minmax_child = vpar_minmax_child_vector(vpar,min),
              count=0);
           if(!mapisdefined(m,minmax_child,&count),
              my(runs=min_child_runs(vpar));
              listput(l,runs);
              print(vpar"  "runs));
           mapput(m,minmax_child,count+1));
     m=Mat(m);
     for(i=1,matsize(m)[1],
        print(m[i,1]" count "m[i,2]));
     print("sorted");
     l=vecsort(l);
     for(i=1,#l, print(l[i]));
  );
}

{
  for(n=1,8,
     my(count=0,
        m=Map());
     forvec(vec=vpar_forvec_from(n,-1), \\ forests or trees
           my(vpar=vpar_from_upascend(vec),
              minmax_child = vpar_minmax_child_vector(vpar,min));
           mapput(m,minmax_child,1);
           count += (minmax_child==vecsort(minmax_child,,0)));
     \\  print(count",");             \\ vpar with increasing
      print(matsize(Mat(m))[1],",");  \\ vectors occurring
     \\ if(n<=5,
     \\    print("n="n"  "Mat(m)[,1]);
     \\   )
  );
  quit;
}

