\\ Copyright 2018, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../examples/most-maximum-matchings.gp");

read("../vpar-wip.gp");
read("../test-helpers.gp");

option_verbose = 1;

\\ /m/graph/match-maximum-most-heuberger-wagner/00.pdf
print("---------------------\n");



\\-----------------------------------------------------------------------------

a(n) = polcoeff(lift(('x-1)*Mod('x,'x^2-11*'x+9)^n), 1);
{
  \\ A333344 b-file
  system("rm /tmp/b333344.txt");
  my(maxn=1000);
  for(n=0,maxn,
     write("/tmp/b333344.txt",Str(n," ",a(n))));
  system("ls -l /tmp/b333344.txt");
  quit;
}

\\-----------------------------------------------------------------------------

{
  \\ 2*574 + 31 == 1179

  my(bfile=OEIS_bfile_gf("A333347"),
     bfilelen=poldegree(bfile));
  print(bfilelen);
  check_equal(bfile == gcount(x) + O(x^(bfilelen+1)),
             1,
             "b333347 vs gcount()");

  \\ with initial values lindep 606 solves, 605 not enough
  \\ skipping initial vlaues, lindep 575 solves, not 574
  \\ 606-575 == 31 initial exceptions
  \\
  my(v=Vecrev(bfile),
     rowlen=574,
     rows=vector(rowlen,n,n+=50; v[n..n+rowlen-1]),
     l=lindep(rows));
  print(l);
  
  \\ recurrence_guess(v);
  quit;
}
{
  \\ A333347 b-file
  system("rm /tmp/b333347.txt");
  my(maxn=1500,
     g=gcount(x) + O(x^(maxn+1)));
  for(n=0,maxn,
     my(c = polcoeff(g,n));
     write("/tmp/b333347.txt",Str(n," ",c)));
  system("ls -l /tmp/b333347.txt");
}




\\-----------------------------------------------------------------------------
\\ CL with 

\\ m0 of C^k L, count maximum matchings where the "root" unmatched
vector(12,k, G(k+1) - 3*G(k))
\\ not in OEIS: 8, 79, 797, 8056, 81443, 823369, 8324072, 84154471, 850782533

\\ m0 of C^k F, count maximum matchings where the "root" unmatched
vector(12,k, 2*G(k+1) - G(k))
\\ not in OEIS: 21, 213, 2154, 21777, 220161, 2225778, 22502109, 227491197

quit

\\-----------------------------------------------------------------------------
\\
\\ 5 mod 7

g = 1/(1 - 99*x^3 + 729*x^6)
\\ subst(1/x*gG(x),'x, 9*x^3)
\\ substpol(g,9*'x^3,'x)
g = substpol(g,'x^3,'x)
Vec(g + O(x^12))
\\ not in OEIS: 1, 99, 9072, 825957, 75156255, 6838346592, 622207402713
g = subst(g,'x,'x/3)
Vec(g + O(x^12))
\\ 1/(1 - 33*x + 81*x^2)
\\ not in OEIS: 1, 33, 1008, 30591, 927855, 28141344, 853508097
g = subst(g,'x,'x/3)
Vec(g + O(x^12))
\\ 1/(1 - 11*x + 9*x^2)
\\ not in OEIS: 1, 11, 112, 1133, 11455, 115808, 1170793, 11836451

\\ spread by x^3, multiply 3^k
Vec(gG(3*x^3) + O(x^18))
1/(1 - 99*x^3 + 729*x^6) + O(x^18)
Ser(vector(18,n,n--; if(n%3==0, 9^(n/3)*G(n/3+1), 0)))
recurrence_guess(vector(180,n,n--; if(n%3==0, 9^(n/3)*simplify(G(n/3)), 0)))

quit




\\-----------------------------------------------------------------------------
\\ Heuberger and Wagner theorem 1.1   fewest maximum mtchings
\\
\\ even n,  fewest 1 maximum matching iff perfect matching (A005751)
\\ odd n,   fewest 2 maximum matchings
\\            iff duplicate a leaf of an n-1 perfect matching
\\
\\ n=1   3   5   7    9    11     13      15       17         19
\\   0,1,1,1,1,2,3,5,10,15,39,49,160,180,702,701,3177,2891,14830,
\\ not in OEIS: 3,5,10,15,39,49,160,180,702,701,3177,
\\
\\ odds
\\ n=1 3 5 7  9 11  13  15       17         19
\\   0,1,1,3,10,39,160,702,3177,14830,
\\ not in OEIS: 3,10,39,160,702,3177,14830,
\\ A005750 planted perfect
\\   (Euler transform A000151
\\   rooted trees n vertices, edges not at root are 2-coloured (perf matching)
\\
\\ cf total automorphism orbits perfect matched free tree
\\ 1,2,7,26,107,458,2058,9498,44947,
\\ = count rooted unlabelled matched, by any one from each orbit as the root
\\
\\ cf total orbits without leaf in perfect matched free tree
\\ 0,1,4,16,68,298,1356,6321,30117,
\\ half of A198760

{
  \\ total free tree automorphism orbits containing a leaf
  forstep(n=2,30,2,   \\ evens
     my(vpar=vpar_free_first(n),
        total=0);
     until((vpar=vpar_free_next(vpar))=='none,
           vpar_has_perfect_matching(vpar) || next;
           \\ my(orbitnum=vpar_free_orbitnum_vector(vpar),
           \\    leaves=vpar_leaves(vpar));
           \\ \\ print(vpar" "orbitnum" "leaves);
           \\ leaves=vpar_set_complement(vpar,leaves);y
           \\ orbitnum=vecextract(orbitnum,leaves);
           \\ total += #Set(orbitnum);
           total++;
          );
     print1(total",");
     );
  quit;
}
{
  \\ rooted n vertices, sans root is perfect matched forest
  for(n=1,20,
     my(count=0,
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           my(root=vpar_root(vpar),
              vpar_sans_root=vpar_delete_vertex(vpar,root));
           count += vpar_has_perfect_matching(vpar_sans_root));
     print1(count",");
     );
  quit;
}
{
  \\ for(n=1,30,   \\ all
  forstep(n=1,30,2,   \\ odds
     my(vpar=vpar_free_first(n),
        total=0,
        target=if(n%2==0,1,2), matchnum,count);
     until((vpar=vpar_free_next(vpar))=='none,
           [matchnum,count] = vpar_matchnum_and_count(vpar);
           if(count==target, total++));
     print1(total",");
     );
  quit;
}


\\-----------------------------------------------------------------------------
\\ HOG

{
  \\ next n=35
  my(n=33,
     vpar=make(n),
     mc=vpar_matchnum_and_count(vpar),
     matchnum=mc[1],
     count=mc[2]);

print("Most Maximum Matchings Tree ",n,"

Heuberger and Wagner determine trees of n vertices with the most maximum matchings and show there is a unique such tree (except two each at n=6 and n=34).  The present tree is n=",n,".  It has ",count," maximum matchings (of size matchnum ",matchnum,").

Clemens Heuberger and Stephan Wagner, \"The Number of Maximum Matchings In a Tree\", Discrete Mathematics, volume 311, issue 21, November 2011, pages 2512-2542.  http://arxiv.org/abs/1011.6554");

  quit;
}



\\-----------------------------------------------------------------------------
{
  for(n=1,16,
     my(vpar=make(n),
        mc=vpar_matchnum_and_count(vpar),
        p=vpar_adjacency_poly(vpar),
        roots=polrootsreal(p));
     roots=select(r->r!=0,roots);
     print(mc"  "vecprod(roots),"  "p));

  quit;
}

\\-----------------------------------------------------------------------------


\\vpar_view(make_CL(3));
\\ vpar_view(make(6));
print("n=181");
vpar=make(181)
#vpar
matchnum_and_count=vpar_matchnum_and_count(vpar)
factor(matchnum_and_count[2])

{
  check_equal(make(1), [0]);
  check_equal(vector(100,n,n--; #make(n)),
              vector(100,n,n--; n),
              "make() length");
}

{
  for(n=6,17,
     my(vpar=make(n));
     print("n="n"  "vpar_matchnum_and_count(vpar)));
}

\\ 0 mod 7 = gf (81 - 72*x)/(1 - 11*x + 9*x^2)
\\ polroots(1 - 11*x + 9*x^2)
\\ poldisc(1 - 11*x + 9*x^2) == 85

\\ 1 mod 7 = gf (112 - 99*x)/(1 - 11*x + 9*x^2)
\\ 2 mod 7 = gf big
\\ 3 mod 7 = gf big
\\ 4 mod 7 = gf (303 - 270*x)/(1 - 11*x + 9*x^2)
\\ 5 mod 7 = gf (35623/85 - 31716/85*x)/(1 - 11*x + 9*x^2)
\\            + (-93/85 - 162/85*x - 189/17*x^2 + 972/17*x^3 + 6561/85*x^4 + 28431/85*x^5)/(1 - 99*x^3 + 729*x^6)
\\ 6 mod 7 = gf big

\\ recurrence_guess(vector(400,n,n--; vpar_matchnum_and_count(make(n))[2]));



\\------------------------------------------------------------------------------

  \\ term in l^k
g(x) =
{
  + (7/9 
     \\ + x + 2191/1445*x^2 + 9212/4335*x^3 + 3*x^4 + 349/85*x^5
     \\ + 699776/122825*x^6
     - 5/9*x^7 
     \\ - 9729/7225*x^9 - 1640/867*x^10 - 3*x^11 - 63/17*x^12
     \\ - 124592/24565*x^13
    )
  / (1 - 11*x^7 + 9*x^14);
}

g(x) + O(x^49)
vector(7,k,my(n=7*k+7); vpar_matchnum_and_count(make(n))[2])

g(x) =
{
  + (7/9 
     - 5/9*x
    )
  / (1 - 11*x + 9*x^2);
}

g(x) + O(x^7)
vector(7,k,my(n=7*k+7); vpar_matchnum_and_count(make(n))[2])

vector(6,k,k--; 7/9*(l^(k+1)-h^(k+1))/(l-h)  - 5/9*(l^k-h^k)/(l-h))

{
  \\ count = c * l^k
  \\ c = count / l^k
  for(k=10,20,
     my(n=7*k+7);
     print(count_0m7(k)/ l^(n/7) *1.0));
}
(7/9*l - 5/9)/(l-h)

(67*l-71)/765
print((67*l-71)/765.0);
quit

read("../test-helpers.gp");
g(x) = (7/9)/(1 - 11*x + 9*x^2);
test_gf_terms(g(x),6)
l*1.0
k=20
test_gf_term(g,n) = test_gf_terms(g,n+1)[n+1];
test_gf_term(g(x),k) / l^k * 1.0

quit






\\------------------------------------------------------------------------------
\\ matchnum stable to any vertex delete
\\ none
\\
\\ matchnum stable to any leaf delete
\\ 1,0,1,1,2,3,6,10,20,39,81,167,365,792,1775,
\\ not in OEIS: 3,6,10,20,39,81,167,365,792,1775,
\\
\\ matchnum stable to any leaf delete, and not bileaves
\\ 0,0,0,0,0,1,1,4,7,19,40,97,221,526,1230,
\\ not in OEIS: 4,7,19,40,97,221,526,1230,

{
  my(vpar=[0, 1, 2, 2, 2, 2, 1, 7]);
  print(vpar"  "vpar);
  for(v=1,#vpar,
     my(sub=vpar_delete_vertex(vpar,v));
     print(vpar_matchnum(sub)"  "sub));
  my(m=vpar_matchnumsets(vpar));
  for(i=1,#m, print(m[i]));
}

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
         count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           if(vpar_is_bileaves(vpar), next);
           my(matchnum=vpar_matchnum(vpar),
             leaves=vpar_leaves(vpar));
           for(i=1,#leaves,
              my(sub=vpar_delete_vertex(vpar,leaves[i]));
              if(vpar_matchnum(sub)!=matchnum,
                 next(2));
           );
           count++;
          );
     print1(count",");
     );
  quit;
}



\\------------------------------------------------------------------------------
\\ type A,B by vpar_matchnumsets()

set_is_subset(subset,superset) = setminus(subset,superset)==[];
type_ABs(vpar) =
{
  \\ B are in every matchnumset
  my(sets=vpar_matchnumsets(vpar),  
     B=[1..#vpar]);
  for(i=1,#sets,
     B=setintersect(B,
                    setunion(sets[i],
                             vecextract(vpar,sets[i]))));
  [vpar_set_complement(vpar,B), B];
}

{
  for(n=1,8,
     my(vpar=vpar_free_first(n),
         count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(AB=type_ABs(vpar),
              A=AB[1],
              leaves=vpar_leaves(vpar),
             is_sub=set_is_subset(leaves,A),
             is_bi=vpar_is_bileaves(vpar));
           print(vpar"  ",AB,
                 " sub=",is_sub,
                 " bi=",is_bi,
                 if(is_sub!=is_bi," **********",""));
          );
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ type A,B

set_is_subset(subset,superset) = setminus(subset,superset)==[];

\\ [ set, set ] of alternate vertices, with v in the first set
bipartition(vpar,v) =
{
  my(dist=vpar_path_lengths_vector(vpar,v));
  [ select(v->dist[v]%2==0, [1..#vpar]),
    select(v->dist[v]%2==1, [1..#vpar]) ];
}

{
  for(n=6,60,
     my(vpar=make(n),
        matchnum=vpar_matchnum(vpar),
        mc=vpar_matchnum_and_count(vpar),
        eccs=vpar_eccentricities_vector(vpar),
        A=select(v->eccs[v]%2==0, [1..#vpar]),  \\ might be omitted
        B=select(v->eccs[v]%2==1, [1..#vpar]),  \\ in every maximum matching
        leaf=vpar_leaves(vpar)[1],
        bip=bipartition(vpar,leaf));
     check_equal(set_is_subset(vpar_leaves(vpar), A), 1);
     check_equal([A,B], bip);
     check_equal(#B,matchnum);

     my(setcount=0);
     forsubset([#vpar,matchnum],set,
               #set == matchnum || error();
               vpar_is_matching_set(vpar,set) || next;
               setcount++;
               my(set=Vec(set),
                  all=vecsort(concat(set,apply(v->vpar[v],set))));
               \\ print("set="set" all="all" A="A" B="B);
               set_is_subset(B,all) || error();
               );
     check_equal(setcount,mc[2]);
     );
  quit;
}


\\------------------------------------------------------------------------------
\\ Prime Codes

\\ prime code for root=centre
\\ not in OEIS: 2, 4, 8, 16, 32, 49, 133, 361, 578, 1156, 4556, 17956, 9826, 113569, 579977, 83521, 1421803, 20593444, 133535188, 17799961
{
  print("primecode");
  vector(20,n,n++;
         my(vpar=make(n));
         vpar=vpar_reroot_forest(vpar,vpar_centres(vpar));
         vpar_to_primecode(vpar))
}

\\ unicentral except n=2 path-2 is bicentral
check_equal(vector(50,n,n--; #vpar_centres(make(n))), \
            vector(50,n,n--; if(n<=2,n, 1)));


\\------------------------------------------------------------------------------
\\ Formula

sqrt85 = quadgen(4*85);
L = (11 + sqrt85)/2;
H = (11 - sqrt85)/2;

sqrt85^2 == 85 || error();

check_equal(H, 9/L);
check_equal(L-H, sqrt85);
check_equal(2*L-11, sqrt85);
check_equal(L-H, 2*L-11);
check_equal(1/(L-H), 1/85*sqrt85);
check_equal(1/(L-H), (2*L-11)/85);
check_equal(L*(2*L-11), 85/2 + 11/2*sqrt85);
check_equal(L*(2*L-11), -18 + 11*L);
check_equal((2*L-11)*9/L, -85/2 + 11/2*sqrt85);
check_equal((2*L-11)*9/L, 11*L -206/2);

check_equal(simplify(L*H), 9);
check_equal(simplify(L*H^2), 99/2 - 9/2*sqrt85);
check_equal(simplify(L^2*H), 99/2 + 9/2*sqrt85);

G(k) = simplify((L^k-H^k)/(L-H));
check_equal(vector(#A190872_samples,n,n--; G(n)), A190872_samples);
CL(k) = G(k+1);
CL0(k) = G(k+1) - 3*G(k);
CF(k) = 3*G(k+1) - 3*G(k);
CF0(k) = 2*G(k+1) - G(k);

M=[8,3;5,3];
{
  check_equal(vector(100,k,k--; ((M^k)*[1;1]) [1,1]),
              vector(100,k,k--; CL(k)),
              "M products for CL");
  check_equal(vector(100,k,k--; ((M^k)*[1;1]) [2,1]),
              vector(100,k,k--; CL0(k)),
              "M products for CL0");

  check_equal(vector(100,k,k--; (M^k)[1,1]),
              vector(100,k,k--; G(k+1) - 3*G(k)),
              "M[1,1] powers");
  check_equal(vector(100,k,k--; (M^k)[2,1]),
              vector(100,k,k--; 5*G(k)),
              "M[2,1] powers");

  check_equal(vector(100,k,k--; (M^k)[1,2]),
              vector(100,k,k--; 3*G(k)),
              "M[1,2] powers");
  check_equal(vector(100,k,k--; (M^k)[2,2]),
              vector(100,k,k--; G(k+1) - 8*G(k)),
              "M[2,2] powers");

  check_equal(vector(10,k,k--; M^k),
              vector(10,k,k--; [G(k+1) - 3*G(k),         3*G(k);
                                5*G(k),         G(k+1) - 8*G(k)]),
              "M powers");


  check_equal(vector(10,k,k--; M^k),
              vector(10,k,k--; simplify(1/85 * [5*(L+3) * L^k
                                                + 5*(14-L) * H^k
                                                ,
                                                3*(2*L-11) * (L^k-H^k)
                                                ;
                                                5*(2*L-11) * (L^k-H^k)
                                                , 
                                                5*(14-L) * L^k
                                                + 5*(L+3) * H^k])),
              "M powers");
}

{
  check_equal(vector(7,k,k--; CL(k)),
              vector(7,k,k--; my(vpar=make_CL(k));
                     vpar_matchnum_and_count(vpar)[2]),
              "CL()");

  \\ with root unmatched, is by deleting that vertex
  check_equal(vector(7,k,k--; CL0(k)),
              vector(7,k,k--; my(vpar=make_CL(k));
                     vpar=vpar_delete_vertex(vpar,vpar_root(vpar));
                     vpar_matchnum_and_count(vpar)[2]),
              "CL0()");
}
{
  check_equal(vector(7,k,k--; CF(k)),
              vector(7,k,k--;  my(vpar=make_CF(k));
                     vpar_matchnum_and_count(vpar)[2]),
              "CF()");

  \\ with root unmatched, is by deleting that vertex
  check_equal(vector(7,k,k--; CF0(k)),
              vector(7,k,k--; 
                     my(vpar=make_CF(k));
                     vpar=vpar_delete_vertex(vpar,vpar_root(vpar));
                     vpar_matchnum_and_count(vpar)[2]),
              "CF0()");
}

\\   ( (L^k-H^k)/(L-H) )^2
check_equal(vector(10,k,k--; G(k)^2), \
            vector(10,k,k--; simplify( ( (L^2)^k-2*(L*H)^k + (H^2)^k )/(L-H)^2 )));
check_equal(vector(10,k,k--; G(k)^2), \
            vector(10,k,k--; simplify( ( (L^2)^k - 2*9^k + (H^2)^k )/(L-H)^2 )));

check_equal(vector(10,k,k--; G(k)^3), \
            vector(10,k,k--; simplify( ( (L^3)^k - 3*(L^2*H)^k + 3*(L*H^2)^k - (H^3)^k )/(L-H)^3 )));
\\ recurrence_guess(vector(200,k,k--; G(k)^3))

\\ combine_pairs(vec) =
\\ {
\\   \\ my(m=apply(CF,ks),
\\   \\    m0=apply(CF0,ks),
\\   \\    p=prod(i=1,#m, m[i]));
\\   \\ vecsum(ks)*7 + 4*4 + 1 == n || error();
\\   \\ sum(j=1,4, m0[j] * p/m[j]);
\\ }

\\-----------

\\ CL    theorem 3.3 case 1
m_1m7(n) = my(k=(n-1)/7); CL(k);
m_1m7(n) = my(k=(n+6)/7); G(k);
\\ vector(10,k, m_1m7(7*k+1))
\\ not in OEIS: 30966, 313059, 3161955, 31996971, 323182119, 3270330513
{
  check_equal(vector(7,i,my(n=7*i+1); m_1m7(n)),
              vector(7,i,my(n=7*i+1);
                     my(vpar=make(n)); vpar_matchnum_and_count(vpar)[2]),
              "m_1m7()");
}

\\       C2F        figure 4(b)  N == 3 mod 7
\\        |
\\ C1F -- m -- C3F
\\        |
\\       C0F
m_3m7(n) =            \\ recurrence length 14
{
  my(ks=vector(4,j,j--; floor((n-17+7*j)/28)),
     m=apply(CF,ks),
     m0=apply(CF0,ks),
     p=prod(i=1,#m, m[i]));
  vecsum(ks)*7 + 4*4 + 1 == n || error();
  sum(j=1,4, m0[j] * p/m[j]);
}
\\ vector(10,k, m_3m7(7*k+3))
\\ not in OEIS: 30966, 313059, 3164955, 31996974, 323482119, 3270330543
 recurrence_guess(vector(200,k,k--; m_3m7(7*k+3)))
\\ recurrence_guess(vector(200,n,n+=34; my(vpar=make(n)); vpar_matchnum_and_count(vpar)[2]))
{
  check_equal(vector(7,i,my(n=7*i+3); m_3m7(n)),
              vector(7,i,my(n=7*i+3);
                     my(vpar=make(n)); vpar_matchnum_and_count(vpar)[2]),
              "m_3m7()");
}

m_4m7(n) = my(k=(n-4)/7); CF(k);
m_4m7(n) = my(k=(n-4)/7); 3*(G(k+1) - G(k));
\\ vector(10,k, m_4m7(7*k+4))
\\ not in OEIS: 30966, 313059, 3164955, 31996974, 323482119, 3270330543
{
  check_equal(vector(7,i,my(n=7*i+4); m_4m7(n)),
              vector(7,i,my(n=7*i+4);
                     my(vpar=make(n)); vpar_matchnum_and_count(vpar)[2]),
              "m_4m7()");
}

\\        C2L         figure 4(c)  N == 5 mod 7
\\         |
\\  C1L -- m -- L
\\         |
\\        C0L
m_5m7(n) =
{
  my(ks=vector(3,j,j--; floor((n-5+7*j)/21)),
     m=apply(CL,ks),
     m0=apply(CL0,ks),
     mp=vecprod(m));
  vecsum(ks)*7 + 4*1 + 1 == n || error();
  sum(j=1,3, m0[j] * mp/m[j]) + vecprod(m0);

    m0[1]*m[2]*m[3]
  + m[1]*m0[2]*m[3]
  + m[1]*m[2]*m0[3]
  + m[1]*m[2]*m[3];
}
\\ vector(10,k, m_5m7(7*k+5))
\\ not in OEIS: 4235, 42823, 432992, 4377856, 44259488, 447456823, 4523720036
\\ recurrence_guess(vector(200,k,k--; m_5m7(7*k+5)))
{
  check_equal(CL(0), 1, "CL(0)");
  check_equal(CL0(0), 1, "CL0(0)");  \\ 1 empty
  check_equal(vector(7,i,my(n=7*i+5); m_5m7(n)),
              vector(7,i,my(n=7*i+5);
                     my(vpar=make(n)); vpar_matchnum_and_count(vpar)[2]),
              "m_5m7()");
}

\\    L        figure 4(e) N == 0 mod 7
\\    |
\\    m -- CF
\\    |
\\    L
m_0m7(n) = my(k=(n-7)/7); 2*(CF(k)-CF0(k)) + 3*CF0(k);
m_0m7(n) = my(k=n/7); 8*G(k) - 7*G(k-1);
\\ vector(10,k, m_0m7(7*k))
\\ not in OEIS: 83709, 846279, 8555688, 86496057, 874455435
{
  check_equal(vector(7,i,my(n=7*i); m_0m7(n)),
              vector(7,i,my(n=7*i);
                     my(vpar=make(n)); vpar_matchnum_and_count(vpar)[2]),
              "m_0m7()");
}

quit



\\------------------------------------------------------------------------------
\\ Sequences

print();

read("../../geom/geom.gp");
p(x) = x^2 - 11*x + 9;
print(polroots(p('x)));
default(realprecision,1000);
{
  my(L=(11 + geom_sqrt_as_quad(85))/2);
  p(L) == 0 || error();

  my(len=3*35+1,
     q=floor(L*10.0^len));
  q < L*10^len || error();
  L*10^len < q+1 || error();
  p(q/10^len) < 0 || error();
  p((q+1)/10^len) > 0 || error();

  print("(11 + geom_sqrt_as_quad(85))/2");
  my(v=digits(q));
  for(i=1,#v,
     print1(v[i]",");
     if(i==36 || i==72, print()));
  print();
  print1("%E ");
  for(i=1,12, print1(v[i]));
  print();
  print();

  len=3*35+2;
  my(R=L^(1/7.0),
     q=floor(R*10.0^len));
  p( (q/10^len) ^7) < 0 || error();
  p( ((q+1)/10^len) ^7) > 0 || error();

  print("(11 + geom_sqrt_as_quad(85))/2  ^(1/7)");
  my(v=digits(q));
  for(i=1,#v,
     print1(v[i]",");
     if(i==36 || i==72, print()));
  print();
  print1("%E ");
  for(i=1,12, print1(v[i]));
  print();

  print();
}
{
  for(n=0,46,
     my(vpar=make(n),
        mc=vpar_matchnum_and_count(vpar),
        count=mc[2]);
     print1(count",");
     if(n==23 || n==36, print()));
  print("\n");

  for(n=0,85,
     my(vpar=make(n),
        matchnum=vpar_matchnum(vpar));
     print1(matchnum",");
     if(n==35 || n==60, print()));
  print();
  quit
}





\\------------------------------------------------------------------------------
printwrap_pos = 0;
printwrap_indent = 5;
printwrap(str) =
{
  if(printwrap_pos + length(str) > 70,
     print();
     printwrap_pos=0);
  if(printwrap_pos==0,
     for(i=1,printwrap_indent,print1(" "));
     printwrap_pos=printwrap_indent);
  print1(str);
  printwrap_pos += length(str);
}
printwrap_newline() =
{
  if(printwrap_pos,
     print(); printwrap_pos=0);
}
{
  print("sample vpars");
  for(n=0,-1,
     my(vpar=make(n),
        num_edges=vpar_num_edges(vpar));
     printwrap("[");
     for(v=1,#vpar,
        if(vpar[v],
           num_edges--;
           my(str=Str("["v","vpar[v]"]",
                      if(num_edges==0,"",",")));
           printwrap(str)));
     printwrap("]");
     printwrap_newline();
     print("     ,");
  );
}
{
  print("sample vpars");
  for(n=0,256,
     my(vpar=make(n));
     print(vpar_to_sparse6(vpar));
  );
}

\\------------------------------------------------------------------------------
\\ equimatchable-ness

\\ pattern 1110110 for n=0 to 6 mod 7
\\ for(n=1,300, if(n%7==0,print()); print1(vpar_is_equimatchable(make(n))));
\\ print();

\\ not stable equimatchable
\\ for(n=1,300, if(n%7==0,print()); print1(vpar_is_stable_equimatchable(make(n))));
\\ print();

\\ vpar_matchnum_and_count(make(34))
\\ vpar_matchnum_and_count(make(34.5))

\\ for(n=1,6, print(vpar_to_sparse6(make(n))))
\\ print(vpar_to_sparse6(make(6.5)))
\\ for(n=7,33, print(vpar_to_sparse6(make(n))))
\\ print(vpar_to_sparse6(make(34)))
\\ print(vpar_to_sparse6(make(34.5)))


\\------------------------------------------------------------------------------

print();
{
  my(v=pol_partial_fractions(gcount(x)));
  for(i=1,#v,print(v[i]));
}
\\ print();
\\ {
\\   my(v=pol_partial_fractions(gcount(x)));
\\   my(g=vecsum(v[2..#v]));
\\   my(v=pol_partial_fractions(g));
\\   for(i=1,#v,print(v[i]));
\\   print("gcount sans initial cases degree ",
\\         poldegree(denominator(g)));
\\ }


{
  \\ 2*7 + 2*14 + 2*28 + 42 + 2*70 + 3*98  == 574
  my(v=[
        (1530*x^7 - 510) ,
        (5202*x^7 + 1734) ,

        \\ max
        (9948825*x^14 - 12159675*x^7 + 1105425) ,

        (459*x^14 + 51) ,
        (13770*x^28 - 5610*x^14 + 170) ,
        (46818*x^28 + 19074*x^14 + 578) ,
        (61965*x^42 - 8415*x^21 + 85) ,
        (3839661225*x^70 - 605122650*x^35 + 65025) ,
        (426629025*x^70 - 6437475*x^35 + 7225) ,
        (587468167425*x^98 - 116745039675*x^49 + 122825) ,
        (587468167425*x^98 - 10287085050*x^49 + 122825) ,
        (587468167425*x^98 - 984933675*x^49 + 122825)],
    g=vecprod(v));
  print("dens only degree ", poldegree(g));
  for(i=1,#v,
     my(p=v[i]);
     p=polrecip(p);
     my(r=polroots(p));
     r=abs(r);
     print(vecmax(r)));
  print(l^(1/7));
}
\\ p = (9948825*x^14 - 12159675*x^7 + 1105425)
\\ p = polrecip(p)
\\ p = 9948825*x^14 - 12159675*x^7 + 1105425
\\ r = polroots(p)
\\ r = abs(r)
\\ vecsort(r)

\\ recurrence_guess(vector(1158,n,n+=99; vpar_matchnum_and_count(make(n))[2]))
\\ 
\\ Vec(gcount(x) + O(x^1000)) == \
\\ vector(1000,n,n--; vpar_matchnum_and_count(make(n))[2])

\\ The constant terms in gcount_6m7(x) are negatives of what the recurrence
\\ parts would give working back.  The actual initial 2 mod 7 trees are
\\
vector(5,k,k--; vpar_matchnum_and_count(make(7*k+2))[2]) == \
  [1, 15, 153, 1560, 15807]         || error();


\\------------------------------------------------------------------------------
