\\ Copyright 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose=0;

read("vpar.gp");
read("vpar-wip.gp");
read("test-helpers.gp");


\\-----------------------------------------------------------------------------
\\ vpar_perfect_domnum() states

\\ No, count of how much different with vs without is any value, probably.


\\                                   with   without_dom  without_undom
test_perfect_domnum_rel_initial = [    1,     'none,          0       ];

test_perfect_domnum_rel_validate(rel) =
{
  my(with,without_dom,without_undom);
  [with,without_dom,without_undom] = rel;

  type(with) == "t_INT" || error();

  for(i=1,#rel,
     my(n=rel[i]);
     if(n!='none,
        type(n) == "t_INT" || error();
        n < 999 || error()));
}

test_perfect_domnum_rel_add(p_rel,v_rel) = \
  test_perfect_domnum_rel_add_and_inc(p_rel,v_rel)[1];

test_perfect_domnum_rel_add_and_inc(p_rel,v_rel) =
{
  \\ print("test_perfect_domnum_rel_add() "p_rel"  "v_rel);
  test_perfect_domnum_rel_validate(p_rel);
  test_perfect_domnum_rel_validate(v_rel);

  my(v_with,v_without_dom,v_without_undom);
  [v_with,v_without_dom,v_without_undom] = v_rel;
  my(p_with,p_without_dom,p_without_undom);
  [p_with,p_without_dom,p_without_undom] = p_rel;

  if(v_without_dom  =='none, v_without_dom  =999);
  if(v_without_undom=='none, v_without_undom=999);
  if(p_without_dom  =='none, p_without_dom  =999);
  if(p_without_undom=='none, p_without_undom=999);

  \\ with[p] += min(with[v], without_undom[v])
  \\
  \\ v able to accept further domination from p.
  \\
  my(ret_with = p_with + min(v_with,v_without_undom));

  \\ without_dom[p] = min(without_dom[p]   + without_dom[v],
  \\                      without_undom[p] + with[v])
  \\
  \\ Choose exactly one with[v], rest with_dom[v].
  \\
  my(ret_without_dom = min(p_without_dom + v_without_dom,
                           p_without_undom + v_with));

  \\ without_undom[p] += without_dom[v]
  \\
  my(ret_without_undom = p_without_undom + v_without_dom);

  my(ret=[ret_with, ret_without_dom, ret_without_undom]);
  ret = apply(n->if(n>=999,'none,n), ret);

  my(inc=vecmin(select(n->n!='none, ret)));
  ret = apply(n->if(n=='none, n, n-inc), ret);

  [ ret, inc ];
}

test_perfect_domnum_rel_list = List([]);

listput(test_perfect_domnum_rel_list, \
        test_perfect_domnum_rel_initial);
{
  \\ 2 = path-2       
  \\    *
  \\    |
  \\    *
  listput(test_perfect_domnum_rel_list,
          test_perfect_domnum_rel_add
          (test_perfect_domnum_rel_list[1],
           test_perfect_domnum_rel_list[1]));
}
{
  \\ 3 = path-3         
  \\    *
  \\    |
  \\    *
  \\    |
  \\    *
  listput(test_perfect_domnum_rel_list,
          test_perfect_domnum_rel_add
          (test_perfect_domnum_rel_list[1],
           test_perfect_domnum_rel_list[2]));
}
{
  \\ 4 = path-4
  \\    *
  \\    |
  \\    *
  \\    |
  \\    *
  \\    |
  \\    *
  listput(test_perfect_domnum_rel_list,
          test_perfect_domnum_rel_add
          (test_perfect_domnum_rel_list[1],
           test_perfect_domnum_rel_list[3]));
}
{
  \\ 5 = path-5
  listput(test_perfect_domnum_rel_list,
          test_perfect_domnum_rel_add
          (test_perfect_domnum_rel_list[1],
           test_perfect_domnum_rel_list[4]));
}
{
  \\ 6 = star-3       
  \\    *
  \\   / \
  \\  *   *
  listput(test_perfect_domnum_rel_list,
          test_perfect_domnum_rel_add
          (test_perfect_domnum_rel_list[2],
           test_perfect_domnum_rel_list[1]));
}
{
  \\ 7 = claw root at arm       
  \\    *
  \\    |
  \\    *
  \\   / \
  \\  *   *
  listput(test_perfect_domnum_rel_list,
          test_perfect_domnum_rel_add
          (test_perfect_domnum_rel_list[1],
           test_perfect_domnum_rel_list[6]));
}
{
  \\ 8 claw + 1 above
  \\    *
  \\    |
  \\    *
  \\    |
  \\    *
  \\   / \
  \\  *   *
  listput(test_perfect_domnum_rel_list,
          test_perfect_domnum_rel_add
          (test_perfect_domnum_rel_list[1],
           test_perfect_domnum_rel_list[7]));
}

\\ Return the state number 1 to 5 for the given rel, or return 0 if not one
\\ of the 5 states in test_perfect_domnum_rel_list.
test_perfect_domnum_rel_find(rel) =
{
  for(i=1,#test_perfect_domnum_rel_list,
     if(test_perfect_domnum_rel_list[i] == rel, return(i)));
  0;
}


{
  \\ extend test_perfect_domnum_rel_list with necessary further states
  my(more=1);
  while(more && #test_perfect_domnum_rel_list < 150,
        more=0;
        for(p=1,#test_perfect_domnum_rel_list,
           for(v=1,#test_perfect_domnum_rel_list,
              my(rel=test_perfect_domnum_rel_add
                     (test_perfect_domnum_rel_list[p],
                      test_perfect_domnum_rel_list[v]));
              if(!test_perfect_domnum_rel_find(rel),
                 print("add "rel);
                 more=1;
                 listput(test_perfect_domnum_rel_list,rel)))));
}
{
  if(1,
     for(i=1,#test_perfect_domnum_rel_list,
        my(rel=test_perfect_domnum_rel_list[i]);
        my(with,without_dom,without_undom);
        [with,without_dom,without_undom] = rel;
        print("state="i"  ",rel,"  delta "with-without_dom));
     );
}

{
  for(p=1,#test_perfect_domnum_rel_list,
     for(v=1,#test_perfect_domnum_rel_list,
        my(rel=test_perfect_domnum_rel_add
               (test_perfect_domnum_rel_list[p],
                test_perfect_domnum_rel_list[v]));

        check_equal(test_perfect_domnum_rel_find(rel)!=0, 1,
                    Str("test_perfect_domnum_rel transition"
                        " p="p" v="v" to "rel))));
}
\\ print("test_perfect_domnum_rel_list  ",#test_perfect_domnum_rel_list);

{
  test_perfect_domnum_rel_table
  = matrix(#test_perfect_domnum_rel_list,
           #test_perfect_domnum_rel_list,
           p,v,
           test_perfect_domnum_rel_find
           (test_perfect_domnum_rel_add
            (test_perfect_domnum_rel_list[p],
             test_perfect_domnum_rel_list[v])));
}

{
  check_equal(test_perfect_domnum_rel_list,
              [['no_with, 0,        0,        'unmatched],   \\ 1
               [    0,   -1, 'no_without_undom, 'matched],     \\ 2
               [    0,    0,        0,        'unmatched],   \\ 3
               [    0,   -1,       -1,        'matched],     \\ 4
               [    0,   -1,        0,        'unmatched],   \\ 5
               [   -1,   -1,       -1,        'matched],
               [    0,   -1,       -1,        'unmatched],
               [   -1,   -2,       -1,        'matched],
               [   -1,   -1,       -1,        'unmatched],
               [   -1,   -2,       -2,        'matched],     \\ 10
               [   -1,   -2,       -1,        'unmatched],   \\ 11

               [   -1,   -1, 'no_without_undom, 'matched],     \\ 12
               [   -1,   -2, 'no_without_undom, 'matched],
               [   -1,   -1,        0,        'unmatched],
               [   -1,   -2,        0,        'unmatched],
               [   -1,   -2,       -2,        'unmatched]    \\ 16
              ]);               
}

\\ Return 1 if rel is not an edge almost equimatchable state as a root.
\\ Per vpar_is_equimatchable() states, cannot have 4 as that is a
\\ non-equimatchable as a root.
\\
test_perfect_domnum_rel_is_ok_at_root(rel) =
{
  test_perfect_domnum_rel_v_max(rel) == -1;
}
{
  test_perfect_domnum_rel_ok_at_root
  = apply(rel-> test_perfect_domnum_rel_is_ok_at_root(rel),
         test_perfect_domnum_rel_list);
}

\\ 11,15 are doomed above or below, but are ok as a root
check_equal(vecsum(test_perfect_domnum_rel_table[11,]), 0);
check_equal(vecsum(test_perfect_domnum_rel_table[,11]), 0);
check_equal(test_perfect_domnum_rel_ok_at_root[11], 1);
\\
check_equal(vecsum(test_perfect_domnum_rel_table[15,]), 0);
check_equal(vecsum(test_perfect_domnum_rel_table[,15]), 0);
check_equal(test_perfect_domnum_rel_ok_at_root[15], 1);

{
  \\ 16 is not at root, and goes to non for anything under or above
  check_equal(vecsum(test_perfect_domnum_rel_table[16,]), 0);
  check_equal(vecsum(test_perfect_domnum_rel_table[,16]), 0);
  check_equal(test_perfect_domnum_rel_ok_at_root[16], 0);

  \\ drop 16 as a target
  check_equal(test_perfect_domnum_rel_table[7,6], 16);
  check_equal(test_perfect_domnum_rel_table[7,12], 16);
  my(count=0);
  for(p=1,#test_perfect_domnum_rel_list,
     for(v=1,#test_perfect_domnum_rel_list,
        my(t=test_perfect_domnum_rel_table[p,v]);
        if(t==16,
           test_perfect_domnum_rel_table[p,v] = 0;
           count++)));
  check_equal(count, 2);   \\ 16 occurs in 2 places
}

test_perfect_domnum_rel_is_equivalent(x,y) =
{
  \\ same final ok-ness
  test_perfect_domnum_rel_ok_at_root[x] ==
  test_perfect_domnum_rel_ok_at_root[y] || return(0);

  my(m=apply(s->if(s==y,x,s), test_perfect_domnum_rel_table));

  \\ same transitions when add new child under
  m[x,] == m[y,] || return(0);

  \\ same transitions when are a child under some parent
  m[,x] == m[,y] || return(0);

  1;
}

{
  for(x=1,#test_perfect_domnum_rel_list,
     for(y=x+1,#test_perfect_domnum_rel_list,
        check_equal(test_perfect_domnum_rel_is_equivalent(x,y),
                       (x==6 && y==12)
                    || (x==8 && y==13)
                    || (x==9 && y==14)
                    || (x==11 && y==15)
                    ,
                    Str("test_perfect_domnum_rel_is_equivalent()"
                        " x="x" y="y))));
}

\\ Might be able to make an argument why these variations in without_max
\\ don't make a difference, not with the conditions of the other fields at
\\ least.  For now easy enough to notice mechanically the equivalence in
\\ test_perfect_domnum_rel_table etc.
\\
check_equal(test_perfect_domnum_rel_list[6],  [-1, -1, -1, 'matched]);
check_equal(test_perfect_domnum_rel_list[12], [-1, -1, 'no_without_undom, 'matched]);

check_equal(test_perfect_domnum_rel_list[8],  [-1, -2, -1, 'matched]);
check_equal(test_perfect_domnum_rel_list[13], [-1, -2, 'no_without_undom, 'matched]);

check_equal(test_perfect_domnum_rel_list[9],  [-1, -1, -1, 'unmatched]);
check_equal(test_perfect_domnum_rel_list[14], [-1, -1,  0, 'unmatched]);

check_equal(test_perfect_domnum_rel_list[11], [-1, -2, -1, 'unmatched]);
check_equal(test_perfect_domnum_rel_list[15], [-1, -2,  0, 'unmatched]);

\\ drop 16
\\ 15 -> 11
\\ 14 -> 9
\\ 13 -> 8
\\ 12 -> 6
\\                                      1,2,3,4,5,6,7,8,9,10,11,12,13 14 15 16
test_perfect_domnum_rel_shrink = [1,2,3,4,5,6,7,8,9,10,11, 6, 8, 9,11, 0];
test_perfect_domnum_rel_shrink_size = \
  #test_perfect_domnum_rel_list - 5;
{
  check_equal(test_perfect_domnum_rel_shrink_size, 11, 
              "test_perfect_domnum_rel_shrink_size");
  check_equal(vecmax(test_perfect_domnum_rel_shrink), 
              test_perfect_domnum_rel_shrink_size, 
              "vecmax test_perfect_domnum_rel_shrink");
}
{
  \\ vpar_INTERNAL_perfect_domnum_table[]
  \\ generated
  my(m=matrix(test_perfect_domnum_rel_shrink_size,
              test_perfect_domnum_rel_shrink_size));
  for(p=1,#test_perfect_domnum_rel_list,
     my(ps=test_perfect_domnum_rel_shrink[p]);
     if(!ps, next());
     for(v=1,#test_perfect_domnum_rel_list,
        my(vs=test_perfect_domnum_rel_shrink[v]);
        if(!vs, next());

        my(t=test_perfect_domnum_rel_find
              (test_perfect_domnum_rel_add
               (test_perfect_domnum_rel_list[p],
                test_perfect_domnum_rel_list[v])));
        if(t, t=test_perfect_domnum_rel_shrink[t]);
        m[ps,vs] = t));

  if(0,
     for(p=1,test_perfect_domnum_rel_shrink_size,
        for(v=1,test_perfect_domnum_rel_shrink_size,
           printf(" %2d%s",
                  m[p,v],
                  if(v==test_perfect_domnum_rel_shrink_size
                     && p==test_perfect_domnum_rel_shrink_size,"",",")));
     print()));

  my(l=List([]));
  for(p=1,test_perfect_domnum_rel_shrink_size,
     for(v=1,test_perfect_domnum_rel_shrink_size,
        listput(l,m[p,v])));
  check_equal(vpar_INTERNAL_perfect_domnum_table,
              Vecsmall(l),
              "vpar_INTERNAL_perfect_domnum_table generated");
}

check_equal(test_perfect_domnum_num_states, \
            test_perfect_domnum_rel_shrink_size);

{
  \\ vpar_perfect_domnum() ok constant, inline in code
  \\ generated
  my(want=0);
  for(i=1,#test_perfect_domnum_rel_list,
     if(test_perfect_domnum_rel_ok_at_root[i],
        my(state=test_perfect_domnum_rel_shrink[i]);
        if(!state, next());
        want=bitor(want, 1 << state)));
  check_equal(3024,
              want,
              "vpar_perfect_domnum() at root bits constant");
  check_equal(test_perfect_domnum_at_root_bits,
              want);
}

test_perfect_domnum_state_is_equivalent(x,y) =
{
  \\ same final ok-ness
  test_perfect_domnum_at_root(x) ==
  test_perfect_domnum_at_root(y) || return(0);

  \\ same transitions when add new child under
  for(v=1,test_perfect_domnum_num_states,
     if(test_perfect_domnum_table_lookup(x,v) !=
        test_perfect_domnum_table_lookup(y,v),
        return(0)));

  \\ same transitions when under some parent
  for(p=1,test_perfect_domnum_num_states,
     if(test_perfect_domnum_table_lookup(p,x) !=
        test_perfect_domnum_table_lookup(p,y),
        return(0)));

  1;
}
{
  for(x=1,test_perfect_domnum_num_states,
     for(y=x+1,test_perfect_domnum_num_states,
        check_equal(test_perfect_domnum_state_is_equivalent(x,y),
                    0,
                    Str("test_perfect_domnum_state_is_equivalent()"
                        " x="x" y="y))));
}

{
  \\ almost equimatchable states are as at the end of respective path-n
  for(n=1,11,
     check_equal(test_perfect_domnum_get_state(vpar_make_path(n)), n));
}
\\ but 6 also second of path-4
check_equal(test_perfect_domnum_get_state([0,1,1,2]), 6);
{
  \\ and 6 also
  \\    *
  \\   / \
  \\  *   *
  \\     / \
  \\    *   *
  my(vpar=[0,1,1,2,2]);
  vpar_validate(vpar);
  check_equal(test_perfect_domnum_get_state(vpar), 6);
}
{
  \\ and 6 also
  \\    *
  \\   /|\
  \\  * * *
  \\      |
  \\      *
  my(vpar=[0,1,1,1,2]);
  vpar_validate(vpar);
  check_equal(test_perfect_domnum_get_state(vpar), 6);
}
{
  \\ and 7 also
  \\     *
  \\     |
  \\     *
  \\   / |
  \\  *  *
  \\     |
  \\     *
  my(vpar=[0,1,2,2,3]);
  vpar_validate(vpar);
  check_equal(test_perfect_domnum_get_state(vpar), 7);
}
{
  \\ and 8 also
  \\     *
  \\     |
  \\     *
  \\     |
  \\     *
  \\   / |
  \\  *  *
  \\     |
  \\     *
  my(vpar=[0, 1, 2, 3, 4, 3]);
  vpar_validate(vpar);
  check_equal(test_perfect_domnum_get_state(vpar), 8);
}  
{
  \\ almost equimatchable state of path-n is not the smallest having its state,
  \\ or perhaps only state<=5
  for(n=2,6,
     my(path=vpar_make_path(n),
        vpar=vpar_unlabelled_first(n));
     until((vpar=vpar_unlabelled_next(vpar)) == 'none,
           my(state=test_perfect_domnum_get_state(vpar));
           if(state==6 && vpar==[0, 1, 2, 1], next());
           if(state==6 && vpar==[0, 1, 2, 2, 1], next());
           if(state==6 && vpar==[0, 1, 2, 1, 1], next());
           if(state==7, next());
           if(state==8, next());
           \\ print("n="n" state="state" in "vpar);
           check_equal(state <= n, 1,
                       Str("vpar_perfect_domnum()"
                           " expect no vpar has n < state\n"
                           " state="state" n="n" vpar="vpar));
          ));
}


