\\ Copyright 2017, 2018, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
\\ default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("memoize.gp");
setrand(getwalltime()%2^32);


check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong: "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

vector_is_nondecreasing(v) = \
  for(i=2,#v, if(v[i]<v[i-1], return(0))); 1;
vector_is_nonincreasing(v) = \
  for(i=2,#v, if(v[i]>v[i-1], return(0))); 1;


al_by_search(n,l) =
{
  my(count=0);
  forvec(vec=vpar_forvec_from(n),
        my(vpar=vpar_from_upascend(vec));
        count += (vpar_num_childless(vpar)==l
                  && vector_is_nonincreasing(vpar_subtree_sizes_vector(vpar))));
  count;
}
\\ {
\\   print("al_by_search");
\\   for(n=1,5,
\\      for(l=1,n,
\\         printf(" %3d",al_by_search(n,l)));
\\      print());
\\ }

\\------------------------------------------------------------------------------
\\ subtree height is non-increasing -- count by search

\\ not in OEIS: 1, 1, 2, 5, 16, 61, 268, 1329, 7342
\\
\\ starts like A322616 but differs here 7342 there 7336

{
  for(n=9,9,
     my(count=0);
     forvec(vec=vpar_forvec_from(n), \\ trees
           my(vpar=vpar_from_upascend(vec));
           my(anc =vpar_subtree_heights_vector(vpar));
           if(vector_is_nonincreasing(anc),
              count++));
     print1(count","););
  quit;
}

\\ OEIS_bfile_gf("A322616") + O(x^10)
\\ g = OEIS_bfile_gf("A322197") + O(x^10)
\\ sqrt(g)

\\ my(x='x+O('x^10), y='y+O('y^10)); c = (cosh(x)*cosh(y) + sinh(x) + sinh(y)) / (1 - sinh(x)*sinh(y))
\\ for(s=0,9, print(sum(i=0,s,my(j=s-i); polcoeff(polcoeff(c,i,'x),j,'y) *i!*j!)));


\\------------------------------------------------------------------------------

W_by_search(n,m,e) =
{
  my(count=0);
  forvec(v=vector(n,i,[1,m+e]),
        my(seen=vector(m+e));
        for(i=1,#v, seen[v[i]]=1);
        count += (vecsum(seen[1..m])==m));
  count;
}

\\ n things into m non-empty and e possibly empty
W(n,m,e) =
{
  if(n==0 && m==0, return(1));
  if(n<0||m<0||e<0, return(0));
  if(m>=1, m*W(n-1,m-1,e+1)) + e*W(n-1,m,e);
}

{
  for(n=0,6,
     printf("n=%d",n);
     for(e=2,3,
        for(m=0,6,
           my(w=W(n,m,e));

           \\ if(w, my(s=stirling(n,m,2)); w = if(s,w/s,-1));
           w /= m!;

           printf("%6d",w));
        printf("  |"));
     print());
}

{
  my(good=0);
  for(n=0,6,
     for(e=0,6,
        for(m=0,6,
           my(r=n^(m+e));
           if(r>1e6,next());
           if(W(n,m,e) != W_by_search(n,m,e), error());
           good++)));
  print("good "good);
}
quit

\\ e=1 allowed empty
\\ A028246  a(n, k) = Stirling2(n,k)*(k-1)!
\\ 1 1 1 3 2 1 7 12 6 1 15 50 60 24 1 31 180 390 360 120 1 63 602 2100 3360 2520 720
\\ 1    1  
\\ 1    3    2  
\\ 1    7   12    6 
\\ 1   15   50   60   24  
\\ 1   31  180  390  360  120  
\\ 1   63  602 2100 3360 2520  720 

\\ e=2 allowed empty
\\ A038719
\\ T(n,k) number of partitions of the set {1,2,...,n+2} into k+2 blocks
\\ where 1 and 2 belong to two distinct blocks and the remaining k blocks
\\ are labelled from a fixed set of k labels.
\\
\\ 1 2 1 4 5 2 8 19 18 6 16 65 110 84 24 32 211 570 750 480 120 64 665 2702 5460 5880 3240 720
\\  1  
\\  2    1
\\  4    5    2  
\\  8   19   18    6 
\\ 16   65  110   84   24 
\\ 32  211  570  750  480  120 
\\ 64  665 2702 5460 5880 3240  720 
\\
\\ e=2 divide stirling2
\\ not in OEIS: 3367,  263,  126,  140,  264
\\
\\ e=2 divide m! = 2-Stirling
\\ A143494 stirling restricted 1,2 in different sets
\\ 1 2 1 4 5 1 8 19 9 1 16 65 55 14 1 32 211 285 125 20 1 64 665 1351 910 245 27 1
\\  1   
\\  2     1
\\  4     5     1 
\\  8    19     9     1 
\\ 16    65    55    14     1  
\\ 32   211   285   125    20     1  
\\ 64   665  1351   910   245    27     1

\\ e=3 allowed empty
\\ not in OEIS: 1, 3, 1, 9, 7, 2, 27, 37, 24, 6, 81, 175, 194, 108, 24, 243, 781, 1320, 1230, 600, 120, 729, 3367, 8162, 11340, 9120, 3960, 720
\\ not in OEIS: 3367,  8162, 11340,  9120,  3960
\\ not in OEIS: 3960,  9120, 11340,  8162,  3367
\\   1  
\\   3     1 
\\   9     7     2  
\\  27    37    24     6 
\\  81   175   194   108    24
\\ 243   781  1320  1230   600   120  
\\ 729  3367  8162 11340  9120  3960   720 
\\
\\ e=3 divide stirling2 column m=1
\\ 7 37 175 781 3367
\\ A005061 a(n) = 4^n - 3^n.
\\ e=3 divide stirling2 column m=2
\\ not in OEIS: 8, 27, 88, 263
\\
\\ e=3 divide m!
\\ A143495 3-Stirling
\\ 1 3 1 9 7 1 27 37 12 1 81 175 97 18 1 243 781 660 205 25 1 729 3367 4081  1890 380 33 1
 \\   1    
 \\   3     1   
 \\   9     7     1   
 \\  27    37    12     1   
 \\  81   175    97    18     1   
 \\ 243   781   660   205    25     1  
 \\ 729  3367  4081  1890   380    33     1 








\\------------------------------------------------------------------------------

\\ f non-empty, n-f-l+1 empty or not, put l.
\\
\\ f + n-f-l+1 = n-l+1 non-empty, put l+(n-f-l+1) = n-f+1 placed
\\ No, the extras can't make the f positions non-empty.

{

  \\ 2 buckets not empty, 3 things
  \\ ab-c, ac-b, bc-a
  my(n=3,l=2,f=1);
  check_exact( (n-l)!*stirling(n-f,n-l,2), 1);
  check_exact(n-l, 1);   \\ places above
  check_exact(n-f, 2);   \\ things
  check_exact(n-f-l, 0);
  check_exact(n-f+1, 3);
  check_exact(n-l+1, 2);
  check_exact( stirling(n-f+1,n-l+1,2), 3);
  check_exact( (n-l+1)! * stirling(n-f+1,n-l+1,2) / ( (n-f+1)! / l! ), 2);
}
quit

\\------------------------------------------------------------------------------
\\ subtree height is non-increasing -- count formula

al(n,l) =
{
  if(n==1 && l==1, 1,
     sum(p=1,n-l,
           al(n-l,p) * sum(s=0,l-p,
                             binomial(l,l-s)
                             * p!*stirling(l-s,p,2)
                             * (n-l-p)^s)));
}

\\ S(n,p) = 1/p! * sum(i=0,p, (-1)^i * binomial(p,i) * (p-i)^n)

al(n,l) =
{
  if(n==1 && l==1, 1,
     sum(p=1,n-l,
           al(n-l,p) * sum(s=0,l-p,
                             binomial(l,l-s)
                             * sum(i=0,p, (-1)^i * binomial(p,i) * (p-i)^(l-s))
                             * (n-l-p)^s)));
}
al=memoize(al);
a(n) = sum(l=1,n, al(n,l));
check_exact(vector(15,n, a(n)), \
            [1, 1, 2, 5, 16, 61, 268, 1329, 7342, 44703, 297314, 2143479, 16643124, 138413449, 1227204644]);


{
  print("al");
  for(n=1,10,
     for(l=1,n,
        printf(" %5d",al(n,l)));
     print());
}

{
  \\ al(n,2)
  \\ A053599 5*2^((n+1)/2)-2*n-7 for odd n,
  \\         7*2^( n   /2)-2*n-7 for even n
  \\ binary palindromes ignoring trailing 0s

  print1("al(n,2) = ");
  for(n=3,12, printf("%d,",al(n,2)));
  print();
}
{
  \\ not in OEIS: 1,7,31,97,271,667,1537,3349,7051
  \\ superseeker nothing
  print1("al(n,3) = ");
  for(n=4,15, printf("%d,",al(n,3)));
  print();
}
{
  \\ not in OEIS: 1,15,115,565,2339,8305,26651,79361,224091
  print1("al(n,4) = ");
  for(n=5,13, printf("%d,",al(n,4)));
  print();
}
{
  \\ 1,1,1,...
  print1("diagonal(n,n-1) = ");
  for(n=5,13, printf("%d,",al(n,n-1)));
  print();
}
{
  \\ 2^(n-2)-1
  print1("diagonal(n,n-2) = ");
  for(n=5,13, printf("%d,",al(n,n-2)));
  print();
}
{
  \\ 2*3^(n-3) - 3*2^(n-3) + 1
  \\ 7,31,115,391,1267,3991,12355,37831,115027
  print1("diagonal(n,n-3) = ");
  for(n=5,13, printf("%d,",al(n,n-3)));
  check_exact(vector(15,n,n+=3; al(n,n-3)),
              vector(15,n,n+=3; 2*3^(n-3) - 3*2^(n-3) + 1));
}
{
  \\ 5*4^(n-4) - 10*3^(n-4) + 3*2^(n-3) - 1
  \\ not in OEIS: 13,97,565,2881,13573,60817,263605,1116961,
  print1("diagonal(n,n-4) = ");
  for(n=0,10, printf("%d,",al(n,n-4)));
  print();
  check_exact(vector(15,n,n+=3; al(n,n-4)),
              vector(15,n,n+=3; 5*4^(n-4) - 10*3^(n-4) + 6*2^(n-4) - 1));
}
{
  \\ 16*5^(n-5) - 40 * 4^(n-5) + 34 * 3^(n-5) - 11 * 2^(n-5) + 1
  \\ not in OEIS: 23,271,2339,16951,110243,667591,3848819,21427831
  print1("diagonal(n,n-5) = ");
  for(n=7,14, printf("%d,",al(n,n-5)));
  print();
  check_exact(vector(15,n,n+=4; al(n,n-5)),
              vector(15,n,n+=4; 16*5^(n-5) - 40*4^(n-5) + 34*3^(n-5) - 11*2^(n-5) + 1));
}
{
  \\ 61*6^(n-6) - 185*5^(n-6) + 206*4^(n-6) - 101*3^(n-6) + 20*2^(n-6) - 1
  \\ not in OEIS: 37,667,8305,83251,726817,5779747,43033825,305435251
  print1("diagonal(n,n-6) = ");
  for(n=8,15, printf("%d,",al(n,n-6)));
  print();
  check_exact(vector(15,n,n+=4; al(n,n-6)),
              vector(15,n,n+=4; 61*6^(n-6) - 185*5^(n-6) + 206*4^(n-6) - 101*3^(n-6) + 20*2^(n-6) - 1));
}
{
  \\ 268*7^(n-7) - 959*6^(n-7) + 1329*5^(n-7) - 887*4^(n-7) + 285*3^(n-7) - 37*2^(n-7) + 1
  \\ not in OEIS: 59,1537,26651,360001,4124699,42164977,397085531
  print1("diagonal(n,n-7) = ");
  for(n=9,15, printf("%d,",al(n,n-7)));
  print();
\\   recurrence_guess(vector(100,n,n+=6; al(n,n-7)));
  check_exact(vector(15,n,n+=6; al(n,n-7)),
              vector(15,n,n+=6; 268*7^(n-7) - 959*6^(n-7) + 1329*5^(n-7) - 887*4^(n-7) + 285*3^(n-7) - 37*2^(n-7) + 1));
}

\\   7     6     5     4     3    2    1
\\                                     1
\\                                1   -1
\\                           2,  -3,   1
\\                     5,  -10,   3,  -1
\\              16,  -40,   34, -11,   1
\\        61, -185,  206, -101,  20,  -1
\\ 268, -959  1329, -887,  285, -37,   1

 \\  1
 \\ -1,   1,
 \\  1,  -3,    2,
 \\ -1,   3,  -10,    5,
 \\  1, -11,   34,  -40,   16,
 \\ -1,  20, -101,  206, -185,  61,

\\ not in OEIS: -11,   34,  -40,   16,
\\ not in OEIS:  20, -101,  206, -185,  61,
\\ not in OEIS:  61, -185,  206, -101,  20
\\ not in OEIS:  2, 5, 16, 61, 268       \\ high coeff
\\ not in OEIS: -3, 3,-11, 20, -37       \\ 2 coeff

quit



\\------------------------------------------------------------------------------
\\ max ancestor is non-decreasing -- view samples

{
  for(n=5,5,
     print("n="n);
     my(count=0);

     \\ for(i=1,100, my(vpar=vpar_make_random(n)); \\ trees

     forvec(vec=vpar_forvec_from(n),my(vpar=vpar_from_upascend(vec));
        my(vec =vpar_subtree_sizes_vector(vpar));
        if(vector_is_nonincreasing(vec),
           print(Vec(vpar)"   vec "vec);
           vpar_view(vpar,1);
           count++));
     print("count "count);
     print(););
  quit;
}

\\------------------------------------------------------------------------------
\\ subtree height is non-increasing -- count formula

\\ n vertices with l many leaves subtree height = 1
\\ below p many previous leaves place l new leaves
\\ must one each, then rest anywhere
al(n,l) =
{
  my(debug=1);
  if(debug,print("n="n" l="l));
  my(ret);
  if(n==0, ret=0,
     l==0, ret=0,
     n==1 && l==1, ret=1,
     n==1, ret=0,
     l>=n, ret=0,

     ret = sum(p=1,n-l,
                 if(debug,print("  p="p" al("n-l","p") = ",al(n-l,p)));
                 my(sum_s=
                    sum(s=0,l-p, \\ others
                          l-s >= p || error();
                          if(debug,print("    s="s
                                         "  binomial("l","s") = ",
                                         binomial(l,s),
                                         "  stirling(",l-s,","p") = ",
                                         stirling(l-s,p,2),
                                         "  factorial p! = ",p!,
                                         "  power "n-l-p,"^s = ",(n-l-p)^s));
                          binomial(l,l-s)
                          * stirling(l-s,p,2)  \\ without order of p
                          * p!                 \\ permute p
                          * (n-l-p)^s));       \\ other placements
                 if(debug,print("    sum over s = "sum_s));
                 al(n-l,p) * sum_s
           );
     );
  if(debug,print("  return "ret));
  ret;
}
al=memoize(al);

check_exact(al(1,0), 0, "al(1,0)");
check_exact(al(1,1), 1, "al(1,1)");
check_exact(al(1,2), 0, "al(1,2)");
check_exact(al(2,1), 1, "al(2,1)");
check_exact(al(2,2), 0, "2,2");

check_exact(al(3,1), 1, "3,1");
check_exact(al(3,2), 1, "3,2");
\\
\\     1         1         al(2,1) = 1
\\     |        / \
\\     2       2   3       al(1,1) = 1
\\     |                   p=1 l=2   n-l-p=0
\\     3

check_exact(al(4,1), 1, "4,1");
check_exact(al(4,2), 3, "4,2");
check_exact(al(4,3), 1, "4,3");

check_exact(al(5,1), 1, "5,1");
check_exact(al(5,2), 7, "5,2");
check_exact(al(5,3), 7, "5,3");
check_exact(al(5,4), 1, "5,4");
check_exact(al(5,5), 0, "5,5");

\\   1    3,2       1                       1
\\  / \   s=0       |    s=0  1           / | \
\\ 2   3  2 of      2    s=1  4 of       2  4  5
\\ |   |            |    total 5         |
\\ 4   5            3                    3      3 of
\\                 / \
\\                4   5                2,1,0,0,0
\\
\\ 2,1,1,0,0    3,2,1,0,0

\\ layers       4
\\             3 3        0s attach anywhere under
\\           2  2  2      so long as all 1s have a 0 under
\\           1  1  1
\\           0  0  0  0

{
  print("al");
  for(n=1,5,
     for(l=1,n,
        printf(" %3d",al(n,l)));
     print());
}

a(n) = sum(l=1,n, al(n,l));
vector(10,n, a(n))
quit



\\ n elements into 2 non-empty sets
\\ S(n,2) = 2^(n-1) - 1
\\ my(n=2); 2^(n-1) - 1 == 1


\\------------------------------------------------------------------------------
\\ subtree height is non-increasing -- count formula

\\ leaving l-t=p-1 is up to t=l-p+1
buckets_choose(p,l) =
{
  my(debug=0);
  if(debug,print("buckets_choose p="p" l="l));
  if(p==0, l==0||error("buckets_choose p=0 l!=0"); return(1));
  l>=p || error("buckets_choose l>=p");
  if(p==1, return(1));
  sum(t=1,l-p+1,
       if(debug,print(" t="t" binomial ",binomial(l,t)," of leaving "p-1","l-t));
       binomial(l,t) * buckets_choose(p-1,l-t));
}
buckets_choose=memoize(buckets_choose);
check_exact(buckets_choose(0,0), 1);
check_exact(buckets_choose(1,1), 1);
check_exact(buckets_choose(1,2), 1, "buckets_choose(1,2)");
check_exact(buckets_choose(1,3), 1, "buckets_choose(1,3)");
check_exact(buckets_choose(2,2), 2, "buckets_choose(2,2)");

\\ 1-23 12-3 13-2 2-23 23-1 3-12
check_exact(buckets_choose(2,3), 6, "buckets_choose(2,3)");

\\ \\ f(p,n-l-p) =
\\ \\   p       * f(p-1,n-l-p-1)   an l onto p
\\ \\ + (n-l-p) * f(p,  n-l-p-1)   an l onto non-p
\\ f(n,p,l) =
\\ {
\\   if(l==0, 1,
\\      p*f(n,p,l-1) + m*f(n,p,l-1));
\\ }

f(p,l) =
{
  if(p==0, 0,
     l<=0, 0,
     p==1, 1,
     sum(t=1,l, binomial(l,t) * f(p-1,l-t)));
}
f=memoize(f);
\\ {
\\   for(p=0,10,
\\      for(l=0,10,
\\         printf(" %8d",f(p,l)));
\\      print());
\\ }
\\ {
\\   \\ A019538  k! * Stirling2(n,k)
\\   print();
\\   for(l=1,10,
\\      for(p=1,l,
\\         print1(f(p,l),",")));
\\   print();
\\   print();
\\ }

g(p,nonp,l) =
{
  p>=0 || error();
  nonp>=0 || error();
  l>=0 || error();
  if(p>l, 0,
     p==0, nonp^l,
     sum(t=1,l, binomial(l,t) * g(p-1,nonp,l-t)));
}
g=memoize(g);
{
  for(p=0,8,
     for(l=0,8,
        printf(" %8d",g(p,l)));
     print());
}
{
  print();
  for(l=1,10,
     for(p=1,l,
        print1(g(p,l),",")));
  print();
  print();
}
check_exact(g(0,1,1),1, "g(0,1,1)");
g
check_exact(g(1),3);
quit

\\ n vertices with l many leaves subtree height = 1
\\ below p many previous leaves place l new leaves
\\ must one each, then rest anywhere
al(n,l) =
{
  \\ WRONG

  my(debug=1);
  if(debug,print("n="n" l="l));
  my(ret);
  if(n==0, ret=0,
     l==0, ret=0,
     n==1 && l==1, ret=1,
     l>=n, ret=0,
     n==1, ret=0,

     \\      *     o others up to l-o=p which is o=l-p
     \\     /|\
     \\    * * *
     \\ ret = sum(p=1,l,
     \\             print("  n-l=",n-l," p="p,
     \\                   " = ",al(n-l,p)
     \\                   " * ",binomial(l,p)
     \\                   " * ",binomial(n-l,l-p)
     \\                  );
     \\             al(n-l,p) * binomial(l,p) * binomial(n-l,l-p)
     \\             \\ * sum(o=0,l-p,
     \\             \\         print("    o="o" l-o=",l-o," n-l-p=",n-l-p,"^o",
     \\             \\               " is buckets("p","l-o,") = ",buckets_choose(p,l-o),
     \\             \\               " * power ",(n-l-p)^o
     \\             \\              );
     \\             \\
     \\             \\         buckets_choose(p,l-o) * buckets_choose(o,n-l-p))
     \\       )

     \\ if(n>l,    print("  end (",n-1,",",l-1") = ",al(n-1,l-1)," * ",(n-l)));
     \\ if(n-l>=1, print("  all ",  al(n-l,l) * l!));
     \\ if(l>=2,   print("  single ",al(n-1,l)));
     \\
     \\ ret = if(n>l,     al(n-1,l-1) * (n-l))
     \\      + if(n-l>=1, al(n-l,l) * l!)
     \\      + if(l>=2,   al(n-1,l))

     ret = sum(p=1,l,
                 print("  p="p" ("n-l","p") ",al(n-l,p)," * ",f(p,l)," f("p","l")");
                 al(n-l,p) * f(p,l))
    );
  if(debug,print("  return "ret));
  ret;
}
al=memoize(al);

\\ another final 1 put onto a non-leaf  al(n,l) -> al(n-1,l-1)*(n-l)
\\ increase all under previous leaves              al(n-l,l)*l!


check_exact(al(1,0), 0, "1,0");
check_exact(al(1,1), 1, "1,1");
check_exact(al(1,2), 0, "1,2");
check_exact(al(2,1), 1, "2,1");
check_exact(al(2,2), 0, "2,2");

check_exact(al(3,1), 1, "3,1");
check_exact(al(3,2), 1, "3,2");
\\
\\     1         1         al(2,1) = 1
\\     |        / \
\\     2       2   3       al(1,1) = 1
\\     |                   p=1 l=2   n-l-p=0
\\     3

check_exact(al(4,1), 1, "4,1");
check_exact(al(4,2), 3, "4,2");
check_exact(al(4,3), 1, "4,3");
\\
\\ to l leaves on n-l previous size tree with p leaves
\\ n-l vertices to put l leaves which are new subtree_height=1
\\ but among the n-l must its p existing leaves
\\ (n-l)^l putting l leaves anywhere
\\ p existing leaves, n-l-p existing non-leaves
\\ p       * f(p-1,n-l-p-1)   an l onto p
\\ (n-l-p) * f(p,  n-l-p-1)   an l onto non-p
\\ t>=p onto p, others l-t onto others n-l-p
\\ buckets_choose(t,p) * (n-l-p)^(l-t)
\\
\\     1         1        1
\\     |        / \      / \     4 goes under first
\\     2       2   4    2   3    prev leaf to inc it
\\    / \      |        |
\\   3   4     3        4
\\ 2,1,0,0   2,1,0,0    2,1,0,0
\\ 2,1,0     2,1,0      1,0,0
\\ 1,0       1,0        1,0

\\ layers       4
\\             3 3        0s attach anywhere under
\\           2  2  2      so long as all 1s have a 0 under
\\           1  1  1
\\           0  0  0  0

a(n) = sum(l=1,n, al(n,l));

  \\ my(t=0);
  \\ if(n==0,t=0,
  \\    n==1,t=1,
  \\
  \\    if(debug,print(" "n"^",n-2," = ",n^(n-2)));  \\ n as root
  \\    t=n^(n-2)
  \\    + sum(p=1,n-1,
  \\            my(o=n-p);  \\ n-1 to 1
  \\            if(debug,print(" p="p" o="o"  "o"^",o-2," * "p" * "f(p)));
  \\            f(p) * p * o^(o-2)));
  \\ if(debug,print(" total "t));
  \\ t;

vector(6,n, a(n))
quit


\\------------------------------------------------------------------------------
