\\ Copyright 2017, 2018, 2019, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.



\\-----------------------------------------------------------------------------

vpar_relabel_upascend(vpar) = \
  vpar_relabel_perm(vpar,Vecsmall(vpar_seq_upascend(vpar))^-1);
{
addhelp(vpar_relabel_upascend,
"vpar = vpar_relabel_upascend(vpar)
Relabel vpar to vertices in leaf-order.");
}

\\ V1..Vn vertices in leaf order
\\ f(V1)..f(Vn) vpar of relabelled in leaf order
\\
\\          5                                5
\\        /   \       3,3,5,5,0            /   \       5,4,4,5,0
\\      3       4     1,2,3,4,5          1       4     1,2,3,4,5
\\     / \                                      / \
\\    1   2                                    2   3
vpar_from_upascend(seq) =
{
  my(num_children=vpar_num_children_vector(vpar),
     seen=vectorsmall(#vpar),
     vpar=vector(#seq),
     pos=1);

  for(v=1,#vpar,
     \\ print("v="v);
     my(l=v);
     while(num_children[l]==0 && l<=v,
           \\ print(" put l="l);
           seq[pos]=l; pos++;
           l=vpar[l]; \\ to parent
           if(!l,break());
           num_children[l]--));
  seq;
}

{
  my(vpar=vpar_make_complete_tree(2,3));
  vpar_validate(vpar);
  check_exact(vpar_INTERNAL_seq_upwards(vpar), [4, 5, 2, 6, 7, 3, 1]);
  check_exact(vpar_seq_upascend(vpar), [4, 5, 2, 6, 7, 3, 1]);

  \\        7
  \\      /   \
  \\    3       6
  \\   / \     / \
  \\  1   2   4   5
  check_exact(vpar_relabel_upascend(vpar), [3, 3, 7, 6, 6, 7, 0]);
}


\\-----------------------------------------------------------------------------

\\ n-1 many values each 0..n
vpar_random_canonical_forest(n) = vector(n,i,if(i==n,0,random(n+1)));
{
addhelp(vpar_random_canonical_forest,
"can = vpar_random_canonical_forest(n)
Return a random canonical vector for a labelled oriented forest.
As described in vpar_from_upascend(), the return is n-1 values each
uniformly distributed 0..n, and last can[#can]=0.
This is a uniformly distributed choice among the (n+1)^(n-1) many distinct
labelled oriented forests."); }


\\-----------------------------------------------------------------------------
\\ vpar_seq_upascend() for vpar_children_vector()

\\ No, neither postorder nor upascend visit children in ascending order.
\\ Would want some depth-first order.
\\
\\ test_children_vectors_by_upascend(vpar) =
\\ {
\\   my(vec=vector(#vpar,i,List([])),
\\      seq=vpar_seq_upascend(vpar));
\\   for(i=1,#seq,
\\      my(v=seq[i], p=vpar[v]);
\\      if(p,listput(vec[p],v));
\\      vec[v]=Vec(vec[v]));
\\   vec;
\\ }
\\
\\ {
\\   for(rep=1,20,
\\      my(vpar=vpar_make_random(10,1));
\\      my(got =vpar_children_vector(vpar),
\\         want=test_children_vectors_by_upascend(vpar));
\\      if(got!=want,print(vpar);vpar_view(vpar));
\\      check_exact(got,want,"vpar_children_vector() vs upascend");
\\   );
\\ }

\\-----------------------------------------------------------------------------

vpar_make_random_forest(n) = \
  vpar_INTERNAL_from_upascend(vpar_INTERNAL_random(n,1));
{
addhelp(vpar_make_random_forest,
"vpar = vpar_make_random_forest(n)
Return a random vpar forest of n vertices.
There are n^(n-1) labelled oriented forests.
The return is a uniformly distributed random choice among them.");
}

\\ vector(n,i, if(i==1,0,random(i-1)+1));

\\-----------------------------------------------------------------------------


  my(num_children = vpar_num_children_vector(vpar),
     num_terminal = sum(v=1,#vpar, num_children[v] <= !vpar[v]),
     num_below    = vectorsmall(#vpar,v, num_children[v]==0),
     W=0);
  for(u=1,#vpar,
     my(v=u);
     while(num_children[v]==0,
           num_children[v]=-1;
           my(p=vpar[v]);
           if(!p,break());
           W += num_below[v] * (num_terminal-num_below[v]); \\ v--p crossings
           num_below[p] += num_below[v];
           num_children[p]--;
           v=p));
  W;

\\-----------------------------------------------------------------------------
\\ vpar_indsets()

  \\ print("u "u);
  \\ print("v "v);
  \\ print("ret "ret);
  \\    print("at v="v" "without[v]" "with[v]);
  \\    print("  "with);
  \\    print("  "without);
  \\       print("with");
  \\       print("without");
  \\       print("final "all);

\\-----------------------------------------------------------------------------
\\ eccentricity and diameter

        \\ print("at v="v" below "b" "prospective_diameter" new diameter "diameter" new ecc "ecc[p]" at "p);

           print("  set "ecc[p]" second "ecc_second[p]);
            print("v="v" above "v_ecc);
        print(v" to "p" v_ecc "v_ecc" cf "ecc[p]" and second "ecc_second[p]);
  print("ecc below  "ecc);
  print("ecc second "Vec(ecc_second));

\\-----------------------------------------------------------------------------

\\ Return a vector of the vertices 1..#vpar in pre-order.
vpar_seq_preorder(vpar) = Vecrev(vpar_INTERNAL_seq_upwards(vpar));

vpar_to_canondown(vpar) = vecextract(vpar,vpar_seq_childless_paths(vpar));
{
addhelp(vpar_to_canondown,
"vec = vpar_to_canondown(vpar)
Return downward canonical form for vpar.
The return is vpar parent numbers in sequence vpar_seq_childless_paths().

The first entry vec[1]=0, since the root is visited first in down leaf
order.  Further vec[2..n] are each in the range 1..n for a single tree, or
0..n for a forest.  All possible values give different vpar, for n^(n-1)
total trees or (n+1)^(n-1) forests.");
}

vpar_to_canondown(vpar) =
{
  my(seen=vectorsmall(#vpar),
     leaves=vpar_leaves(vpar),
     retlist=List([]),
     upto=1);
  for(i=1,#leaves,
     my(v=leaves[i],l=List([]));
     \\ print("leaf "v);
     while(v=vpar[v],
           listput(l,v);
           if(seen[v],break());
           seen[v]=1);
     \\ print(" stop at "v"  "Vecrev(l));
     listput(retlist,Vecrev(l));
     upto+=#l);
  concat(Vec(retlist));
}

forvec_childless_paths(n) = vector(n-1,i,if(i<n,[1,n],[0,0]));

vpar_from_canondown(vec) =
{
  my(vpar=vector(#vec,i,-1),
     supports=List([]),
     p);
  for(i=1,#vec,
     my(v=vec[i]);
     if(v,
        if(vpar[v]>=0,  \\ already seen
           listput(supports,vec[i-1]),
           vpar[v]=p));
     p=v);
  listput(supports,vec[#vec]);
  \\ print("supports "supports);
  \\ print("vpar "vpar);
  my(i=1);
  for(v=1,#vpar,
     if(vpar[v]<0,
        vpar[v]=supports[i];i++));
  vpar;
}

\\-----------------------------------------------------------------------------

vpar_centres_of_component(vpar,v) =
{
  vpar_centres(vpar);
}

vpar_INTERNAL_for_free_isomorphism(vpar) =
{
  my(c=vpar_centres(vpar));
  if(#c==0, vpar,
     #c==1, vpar_reroot(vpar,c[0]);
     ,
     vpar=Vec(vpar,#vpar+1);  \\ new root between the centres
     if(vpar[c[1]]==c[0], c=Vecrev(c));
     vpar[c[0]]=#vpar; vpar[#vpar]=c[1];
     vpar_reroot(vpar,#vpar));
}
vpar_is_isomorphic(vpar1,vpar2,free=0) =
{
  if(#vpar1!=#vpar2, return(0));
  if(free,
     vpar1=vpar_INTERNAL_for_free_isomorphism(vpar1);
     vpar2=vpar_INTERNAL_for_free_isomorphism(vpar2));

  my(num_children1=vpar_num_children_vector(vpar1),
     num_children2=vpar_num_children_vector(vpar2),
     id1=vector(#vpar1,i,1),
     id2=vector(#vpar2,i,1),
     seq1=vectorsmall(#vpar1),
     seq2=vectorsmall(#vpar2),
     upto=0,
     upto2=0,
     upto_id1=0,
     upto_id2=0,
     phase=1);

  for(v=1,#vpar1,
     if(num_children1[v]==0,   \\ leaf vertices vpar1
        num_children1[v]=-1;
        seq1[upto++]=v);      \\ pre-increment
     if(num_children2[v]==0,   \\ leaf vertices vpar2
        num_children2[v]=-1;
        seq2[upto2++]=v));     \\ pre-increment
  if(upto!=upto2, return(0));

  \\   1               1
  \\   |              / \
  \\   2             2   3
  \\  / \            |
  \\ 3   4           4
  while(upto<#seq1,
        \\ print("seq1 "Vec(seq1)" seq2 "Vec(seq2)"  upto="upto);

        \\ print("phase="phase" to upto="upto);
        my(l1=List([]),l2=List([]));
        for(i=phase,upto,
           my(v=seq1[i], p=vpar1[v]);
           \\ print(" i="i" vpar1 v="v" p="p);
           if(p,
              id1[p] = id1[p]*#vpar1 + id1[v];
              \\ print("  set parent id "id1[p]);
              if(num_children1[p]--==0,
                 listput(l1,p)));

           v=seq2[i]; p=vpar2[v];
           \\ print(" i="i" vpar2 v="v" p="p);
           if(p,
              id2[p] = id2[p]*#vpar2 + id2[v];
              \\ print("  set parent id "id2[p]);
              if(num_children2[p]--==0,
                 listput(l2,p)));
        );
        \\ print("lists");
        \\ print(Vec(l1));
        \\ print(Vec(l2));
        if(#l1!=#l2, return(0));  \\ not isomorphic

        phase = upto+1;
        l1=vecsort(l1,(x,y)->lex(id1[x],id1[y]));
        l2=vecsort(l2,(x,y)->lex(id2[x],id2[y]));

        my(prev_big_id=-1, prev_small_id=-1);
        for(i=1,#l1,
           if(id1[l1[i]] != id2[l2[i]], return(0));  \\ not isomorphic
           if(id1[l1[i]]!=prev_big_id,
              prev_big_id = id1[l1[i]];
              prev_small_id = upto_id++);
           id1[l1[i]] = id2[l2[i]] = prev_small_id);

        for(i=1,#l1,
           seq1[upto++]=l1[i];
           seq2[upto]=l2[i]));
  1;
}

\\-----------------------------------------------------------------------------

{
  \\ p0 = 3,5,6,10, 5, 4,  0,  1,  2,    9,  0
  \\ v0 = 1,2,3, 4, 2, 6, 9/5, 8, 7/10, 11, 5/9
  \\  1 2 3  4  5  6   7   8   9   10   11
  my(vpar=apply(p->p+1, [-1,3,5,6,10,0,4,2,1,0,2,9]));  \\ Caminiti example
  check_exact(#vpar,12);
  check_exact(select(v->test_vertex_gt_ancestors(vpar,v), vector(#vpar,v,v)),
              vecsort(apply(n->n+1, [7,10,5,0,9,11])));
  check_exact(test_vertex_gt_ancestors(vpar,12), 1, "12 is stable");

  \\        5 7 9 10 11
  \\ parent 0 2 0  2  9
  \\ new    2 0 2  9  0
  \\             7   9 10 11 5
  \\        1234   6
}
test_seq_blob_by_stable(vpar) =
{
\\print("vpar "vpar);
  my(prev_unstable=0,
     seq=vectorsmall(#vpar),
     stable=vector(#vpar,v,test_vertex_gt_ancestors(vpar,v)),
     next_unstable= (start)->for(z=start+1,#vpar, if(!stable[z],return(z)));0,
     upto=0);
\\print(stable);
  my(z=next_unstable(0)); if(z,seq[upto++]=z);
  for(v=1,#vpar,
\\print(v);
         if(stable[v],
            seq[upto++]=v,
            my(z=next_unstable(v)); if(z, seq[upto++]=z)));
  seq;
}
test_seq_blob_by_blob(vpar) =
{
  my(debug=1,
     blob=Set([]),
     vblob=#vpar,
     seq=vectorsmall(#vpar-1),
     upto=#seq+1);
  forstep(v=#vpar,2,-1,
         if(debug,print("v="v" vblob="vblob));
         my(path=test_path_vertices_to_root(vpar,v),
            inter=setintersect(Set(path),blob));
         if(debug,print(" path "path));
         if(#inter!=0,
            seq[upto--]=v;
            if(debug,print(" non-null, put v="v" parent "vpar[v]" ("vpar[v]-1")"));
            ,
            if(debug,print(" null, put vblob="vblob" parent "vpar[vblob]" ("vpar[vblob]-1")  new vblob "vpar[v]));
            seq[upto--]=vblob;
            vblob=vpar[v]);
         blob=setunion(blob,[v]));
\\  if(vblob,seq[upto--]=vblob);
  seq;
}
test_code_blob_by_blob(vpar) =
{
  my(debug=0,
     blob=Set([]),
     vblob=#vpar,
     seq=vector(#vpar-1),
     upto=#seq+1);
  forstep(v=#vpar,2,-1,
         if(debug,print("v="v" vblob="vblob));
         my(path=test_path_vertices_to_root(vpar,v),
            inter=setintersect(Set(path),blob));
         if(debug,print(" path "path));
         if(#inter!=0,
            seq[upto--]=vpar[v];
            vpar[v]=0;
            ,
            seq[upto--]=vpar[vblob];
            vpar[vblob]=vpar[v];
            vpar[v]=0);
         blob=setunion(blob,[v]));
\\  if(vblob,seq[upto--]=vblob);
  seq;
}
test_seq_blob_by_rotate(vpar) =
{
  my(prev=#vpar,
     first=0,
     stable=vector(#vpar,v,v>1&&test_vertex_gt_ancestors(vpar,v)),
     vs=select(b->b, stable, 1),
     seq=vectorsmall(#vpar,v,v));
  print("vs0 ",apply(n->n-1, Vec(vs)));
  for(i=1,#vs,
     seq[vs[i]]=vs[if(i<#vs,i+1,1)]);
  \\ forstep(v=#vpar,1,-1,
  \\        if(stable[v],
  \\           if(!first,first=v);
  \\           print("set "prev" to "v);
  \\           seq[prev]=v; prev=v));
  \\ if(first,seq[first]=prev);
  seq;
}


test_seq_blob_by_MCD(vpar) =
{
  my(vlast=#vpar,
     seq=vectorsmall(#vpar,v,v),
     upto=#seq+1);
  forstep(v=#vpar-1,2,-1,
         if(vecmax(test_path_vertices_to_root(vpar,v))==v,
            seq[v]=seq[vlast]; seq[vlast]=v));
  seq;
}
vpar=apply(p->p+1, [-1,3,5,6,10,0,4,2,1,0,2,9])
\\vpar_view(vpar);
\\ seq=test_seq_blob_by_stable(vpar);
\\ seq=test_seq_blob_by_blob(vpar);
\\ seq=test_seq_blob_by_MCD(vpar);

seq=test_seq_blob_by_rotate(vpar);
print("seq  ",Vec(seq));
print("seq0 ",apply(n->n-1, Vec(seq)));
seq=vpar_seq_blob(vpar);
print("seq  ",Vec(seq));
print("seq0 ",apply(n->n-1, Vec(seq)));

got=vecextract(vpar,seq);
print("vecx0 ",apply(n->n-1,got));

\\     1 2 3  4  5 6 7  8 9 10 11
want0=[3,5,6,10, 5,4,0, 1,2,9, 0];  \\ their parent in digraph
want0=[3,5,6,10, 2,4,0, 1,2,9, 0];
want=apply(p->p+1, want0);
print("want0     "want0);
\\ vpar_view(got);
quit

got=test_code_blob_by_MCD(vpar);
print("MCD  "got);


got=test_code_blob_by_blob(vpar);
print("got  "got);


quit;

quit




\\vpar_view([0,4,6,7,11,1,5,3,2,1,3,10]);
check_exact(test_seq_blob_by_stable([0,4,6,7,11,1,5,3,2,1,3,10]), \
            );
check_exact(vpar_seq_blob([0,4,6,7,11,1,5,3,2,1,3,10]),[]);



  \\ seq[] start as no-change permutation, then rotate so that
  \\              r  v1 v2 ... vn
  \\     becomes  v1 v2 ... vn r
  \\
  my(above = vectorsmall(#vpar,v,v),
     seq   = above,
     last=0,
     r=#vpar);
  forstep(v=#vpar,1,-1,
         my(p=vpar[v]);
         if(p, if(v>above[p],print(v);
                  seq[v]=last; last=v,  \\ this is biggest ancestor, cycle down
                  above[v]=above[p]),   \\ propagate biggest ancestor
               r=v));                   \\ smallest root
  if(#vpar, seq[#vpar]=r);
  return(seq);

  \\ Caminiti example
  \\ p0 = 3,5,6,10, 5, 4,  0,  1,  2,    9,  0
  \\ v0 = 1,2,3, 4, 2, 6, 9/5, 8, 7/10, 11, 5/9
  \\  1 2 3  4  5  6   7   8   9   10   11


\\-----------------------------------------------------------------------------
\\ vpar_max_ancestor_vector(vpar)
\\ not cycle safe

  my(seq=vpar_seq_upascend(vpar),
     above=vector(#vpar,v,v));
  forstep(i=#seq,1,-1,    \\ downwards
         my(v=seq[i],p=vpar[v]);
         if(p && v<above[p], above[v]=above[p]));
  above;


\\ with inline cycle-safe max ancestor

vpar_from_blob(vec) =
{
  \\ Per
  \\
  \\     Paulius Micikevicius, Saverio Caminiti, Narsingh Deo, "Linear-time
  \\     Algorithms for Encoding Trees as Sequences of Node Labels".
  \\
  \\ the rotation of vertices which makes a vec which is a digraph of blob
  \\ code doesn't change the set of vertex numbers which have v>ancestors.
  \\ Those ancestors might go up to a cycle.  A cycle is taken to have its
  \\ biggest member as above all of them.
  \\
  \\ So the code here is to establish above[v] = biggest ancestor above v,
  \\ inclusive of v itself..
  \\ Then those which have above[v]==v are to be unrotated from what
  \\ vpar_seq_blob() did.
  \\ The code here assumes vec[#vec]==0 and doesn't bother rotating that
  \\ value back to the first above[v]==v.

  my(above=vectorsmall(#vec),
     stack=vectorsmall(#vec),
     sp=0);
  for(i=1,#vec,
     if(above[i],next());
     my(v=i);
     \\ print("i="i);
     while(1,
           stack[sp++]=v;
           above[v]=-sp;
           \\ print(" stack ",Vec(stack[1..sp])," parent "vec[v]);
           v=vec[v];
           if(!v || above[v],break()));

     \\ print("cycle ",if(v,-above[v]),"  ",if(v, stack[-above[v]..sp]));
     my(m=if(v&&above[v]<0, vecmax(stack[-above[v]..sp]),
             v&&above[v]>0, above[v],
             v));
     \\ print(" sp "sp" above "above" v="v" m="m);

     while(sp,
           my(s=stack[sp]); sp--; \\ pop
           above[s]=m=max(s,m);
           \\ print(" pop "s" set "m);
          \\ if(s==v,break());
          ));
  \\ print(above);
  \\ print(vector(#above,v,v==above[v]));

  sp=0;
  for(v=1,#above,
     if(above[v]==v,
        \\ print("v="v" store vpar "sp);
        my(t=vec[v]); vec[v]=sp; sp=t));

  vec;
}


\\-----------------------------------------------------------------------------
\\ subtree_height by explicit bucketing


The buckets are linked lists, re-using the vectors.
  \\ For height h, seq[h]=v is the first vertex, then heights[v]=v2 next,
  \\ heights[v2]=v3, etc, until vn=0.
  \\ heights[v] is re-used for the linked list once its h is taken.
  \\ List is vertices pushed low to high, so high to low in final traversal.
  \\
  \\ Then final seq sequence of vectors built from its end seq[#vpar]
  \\ downwards, so the seq filling doesn't overlap the seq[h] linked list.

  my(heights=vpar_subtree_heights_vector(vpar),
     max_height=if(#vpar, vecmax(heights)+1, 0), \\ max bucket entry
     seq=vectorsmall(#vpar));
  for(v=1,#vpar,1,
     my(h=heights[v]+1);
     heights[v]=seq[h]; seq[h]=v);

  my(upto=#vpar+1);        \\ ready for pre-decrement
  forstep(h=max_height,1,-1,
     my(v=seq[h]);
     while(v,
           seq[upto--]=v;  \\ pre-decrement
           v=heights[v]));
  seq;

The sequence starts with all childless vertices in ascending order.  They
are reckoned removed from the tree.  Then all now childless vertices are
taken in ascending order, and reckoned removed from the tree.  These phases
childless vertices and their removal continue until no vertices remain.
This is an upwards sequence since each child precedes its parent.  This is
the vertex sequence of Neville's second canonical tree representation.

\\-----------------------------------------------------------------------------
\\ rows with own bucket sort

vpar_seq_rows(vpar,up=0) =
{
  my(depths = vpar_depths_vector(vpar),
     rows   = vectorsmall(if(#depths,vecmax(depths),0)+1),
     chain  = vectorsmall(#vpar));

  \\ Bucket sort vertices into linked lists of those at depth d.
  \\ rows[d] = v is first at depth d, then chain[v] = v2, etc, ending vn=0.
  for(v=1,#vpar,
     my(d=depths[v]+1);
     chain[v]=rows[d]; rows[d]=v);

  \\ Discard depths vector and build there the output sequence.
  \\ upto goes from end of sequence to start since linked lists are vertices
  \\ high to low.  Rows are taken low to high or high to low as per "up" param.
  depths = vectorsmall(#vpar);
  my(upto=#vpar+1);
  for(d=1,#rows,
     my(v=rows[if(up,d,#rows+1-d)]);
     while(v,
           depths[upto--]=v;  \\ pre-decrement
           v=chain[v]));
  depths;
}
{
addhelp(vpar_seq_rows,
"seq = vpar_seq_rows(vpar,up=0)
Return a Vecsmall of the vertex numbers 1..#vpar in row sequence,
ie. breadth first.  The return type is Vecsmall since this is a permutation
of the integers 1..#vpar.

seq is vertices sorted by ascending depth (vpar_depth_of_vertex()) and among
equal depths by ascending vertex number.  So seq starts with all root
vertices in ascending order, then all depth 1 vertices in ascending order,
and so on.

Optional parameter up can be 1 for rows upwards from the deepest.  So sort
first by descending depth, and among equal depths by ascending vertex still.

vpar can be a forest.  The sequence goes by depth of whatever component tree
a given vertex might be in.

For reference, a vecextract(vpar,vpar_seq_rows(vpar)) is not a
canonical form for vpar since vpars can have the same extract.  Some with
the same extract are isomorphic, but some are not.  (That's for isomorphism
reckoned either for free trees or rooted trees.)");
}

\\-----------------------------------------------------------------------------
\\ MAYBE: perhaps a 0,1 copy of the vpar_path_lengths_matrix calculation.
\\ Column-wise additions might be faster than looping over rows.

vpar_reachability_matrix(vpar,directed=0) =
{
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     m=matrix(#vpar,#vpar));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     if(p, m[,p]+=m[,v]; m[v,p]=1));
  if(directed==2, m=m~,
     directed==0, forstep(i=#seq,1,-1,
                         my(v=seq[i], p=vpar[v]);
                         if(p,
                            print(m);
                            print(v);
                            print(p);
                            print(m[v,]);
                            print(m[,p]~);
                            m[v,]+=m[,p]~)));
  apply(n->n!=0, m);
}

\\-----------------------------------------------------------------------------
\\ Blob Sequence Own above[]

  \\ Establish above[v] = maximum of vertices v to root inclusive.
  \\ Must do this in an initial descending pass since vertex numbers can be
  \\ in any order parent-child.
  \\
  \\ Initialize above[v]=v for nothing above roots.
  \\ Then down the tree look for parent above[p] being bigger than v to
  \\ propagate that to above[v].
  \\ r is noted as smallest above[r]==r.  This is the smallest root since
  \\ all roots have nothing at all above them (and anything else with
  \\ above[v]==v has root < v).
  \\
  my(seq=vpar_seq_upascend(vpar),
     above=vectorsmall(#vpar,v,v),
     r=#vpar);
  forstep(i=#seq,1,-1,    \\ for downwards by depth
         my(v=seq[i],p=vpar[v]);
         if(p, if(v<above[p],above[v]=above[p]),
               r=v));


vpar_from_blob(vec) =
{
  \\ As noted in
  \\
  \\   Paulius Micikevicius, Saverio Caminiti, Narsingh Deo, "Linear-Time
  \\   Algorithms for Encoding Trees as Sequences of Node Labels",
  \\   Congressus Numerantium, volume 183, 2006, pages 65-75.
  \\   (URLs with vpar_seq_subtree_height() code.)
  \\
  \\ the rotation of vertices which makes a vec a digraph of blob code
  \\ doesn't change the set of vertex numbers which have v>ancestors, though
  \\ their parents may now go into cycles.
  \\
  \\ So the code here is to establish above[v] = biggest ancestor above v,
  \\ inclusive of v itself, in a cycle-safe way.
  \\ Those with above[v]==v are to be unrotated from what vpar_seq_blob() did.
  \\ The code here assumes vec[#vec]==0 and doesn't bother rotating that
  \\ value back to the first above[v]==v.

  my(above=vpar_max_ancestor_vector(vec),
     prev=0);
  for(v=1,#above,
     if(above[v]==v,       \\ print("v="v" store parent "prev);
        my(t=vec[v]); vec[v]=prev; prev=t));
  vec;
}

\\-----------------------------------------------------------------------------

{
  my(n=4,
     seen=List([]));
  forvec(vec=vpar_forvec_from(n,1), \\ forests
        my(vpar=vpar_from_subtree_height(vec));
        vpar_validate(vpar);
        check_exact(#vpar,n);

        check_exact(setsearch(seen,vpar),0,
                    "vpar_from_subtree_height() not already seen");
        listinsert(seen,vpar, setsearch(seen,vpar,1));

        my(vec_again=vecextract(vpar,vpar_seq_subtree_height(vpar)));
        check_exact(vec_again,vec, "subtree_height vec and back again"));
  check_exact(#seen,(n+1)^(n-1));
}


{
  my(n=4,
     seen=List([]));
  forvec(vec=vpar_forvec_from(n,1), \\ forests
        my(vpar=vpar_from_upqueue(vec));
        vpar_validate(vpar);
        check_exact(#vpar,n);

        check_exact(setsearch(seen,vpar),0,
                    "vpar_from_upqueue() not already seen");
        listinsert(seen,vpar, setsearch(seen,vpar,1));

        my(vec_again=vecextract(vpar,vpar_seq_upqueue(vpar)));
        check_exact(vec_again,vec, "upqueue vec and back again"));
  check_exact(#seen,(n+1)^(n-1));
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        seq=vpar_seq_upqueue(vpar),
        got =vecextract(vpar,seq),
        want=Vec(Vecsmall(vpar)*Vecsmall(vpar_seq_upqueue(vpar))));
     check_exact(got,want, "upqueue vec by permute"));
}

{
  for(ahead=0,1,
     my(n=4,
        seen=List([]));
     forvec(vec=vpar_forvec_from(n), \\ tree
           \\ print("ahead="ahead"  vec "vec);
           my(vpar=vpar_from_upascend(vec,ahead));
           vpar_validate(vpar);
           check_exact(#vpar,n);

           check_exact(setsearch(seen,vpar),0,
                       "vpar_from_upascend() not already seen");
           listinsert(seen,vpar, setsearch(seen,vpar,1));

           if(!ahead,
              check_exact(vpar,test_from_canonical_by_not_in_lists(vec)));

           \\ print("vpar "vpar);
           my(vec_again=vecextract(vpar,vpar_seq_upascend(vpar)));
           \\ print("vec_again "vec_again);
           check_exact(vec_again,vec, "upascend vec and back again"));
     check_exact(#seen,n^(n-1)));
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        seq=vpar_seq_upascend(vpar),
        got =vecextract(vpar,seq),
        want=Vec(Vecsmall(vpar)*Vecsmall(vpar_seq_upascend(vpar))));
     check_exact(got,want, "upascend vec by permute"));
}

{
  my(n=4,
     seen=List([]));
  forvec(can=Vecrev(vpar_forvec_from(n)), \\ tree
        \\ print("can "can);
        my(vpar=vpar_from_childless_paths(can));
        check_exact(setsearch(seen,vpar),0, "not already seen");
        check_exact(#vpar,n);
        listinsert(seen,vpar, setsearch(seen,vpar,1));
        vpar_validate(vpar);
        \\ print("vpar "vpar);

        \\ check_exact(vpar_seq_childless_paths(vpar),
        \\             test_seq_childless_paths_by_lists(vpar));

        my(can_back=vecextract(vpar,vpar_seq_childless_paths(vpar)));
        \\ print("can_back "can_back);
        check_exact(can_back,can, "vpar_seq_childless_paths() and back again"));

  check_exact(#seen,n^(n-1));
}

\\-----------------------------------------------------------------------------
\\ subtree_height


  \\ in vec as vec[1..len]
  \\ Vertices with height 1 are parents of
  \\ vec[] gives num_children[v] for all vertices.
  \\ Initial num_children[v]==0 are the leaf vertices of the first phase,
  \\ stored to vertices seq[1..upto].
  \\ For a vertices seq[phase..upto], each v=seq[i] has parent p=vec[i].
  \\ num_children[p] is decremented and if becomes 0 then p in the next phase.
  \\ vpar is built from v=seq[i] parent p=vec[i], but only at the end where
  \\ the num_children vector can be re-used.


  \\ print("vec "vec);
  my(work=vector(#vec),
     heights=work);
  for(i=1,#vec,
     if(vec[i],
        work[vec[i]]=i));
  \\ print("work "Vec(work));

  my(len=sum(v=1,#work, work[v]==0),
     h=1,  \\ next height
     i=1);
  while(i<=#vec,
        my(new_len=0);
        for(j=1,len,
           my(p=vec[i]);
           \\ print(" i="i" p="p" last occurrence "if(p,work[p],""));
           if(p && work[p]==i,
              \\ print("  set "p" height "h);
              heights[p]=h;
              work[p]=-h;
              new_len++);
           i++);
        len=new_len;
        h++);
  \\ print("heights "heights);
  return(vpar_INTERNAL_seq_bucket_sort(work,1));
  return(vpar_INTERNAL_seq_bucket_sort(heights));



  \\ print("vec "vec);
  my(last_occurrence=vectorsmall(#vec),
     heights=last_occurrence);
  for(i=1,#vec,
     if(vec[i],
        last_occurrence[vec[i]]=i));
  \\ print("last_occurrence "Vec(last_occurrence));

  my(phase=1,
     upto= sum(v=1,#last_occurrence, last_occurrence[v]==0),
     h=1);
  while(phase<#vec,
        \\ print("phase "phase" upto "upto);
        my(new_upto=upto);
        for(i=phase,upto,
           my(p=vec[i]);
           \\ print(" i="i" p="p" last occurrence "if(p,last_occurrence[p],""));
           if(p && last_occurrence[p]==i,
              \\ print("  set "p" height "h);
              heights[p]=h;
              new_upto++));
        phase=upto+1;
        upto=new_upto;
        h++);
  \\ print("heights "heights);
  return(vpar_INTERNAL_seq_bucket_sort(heights));
  return(seq);



\\ seq = vectorsmall(#vec),

\\ ;
\\               seq[new_upto++]=p
  \\ for(v=1,#vec,
  \\    if(last_occurrence[v]==0,
  \\       seq[upto++]=v));


  my(num_children=vpar_num_children_vector(vec),
     heights= vectorsmall(#vec),
     lists = vector(#vec,v, List([])),
     seq   = vectorsmall(#vec),
     phase = 1,
     upto  = 0,
     vpar  = vector(#vec));

  for(v=1,#vec,  \\ leaf vertices
     if(num_children[v]==0,
        seq[upto++]=v));
  print("initial ",seq[phase..upto]);

  while(upto<#vec,
        my(new_upto=upto);
        for(i=phase,upto,
           if(num_children[v]==0,
              seq[new_upto++]=v);
           if(p,
              heights[p]=max(heights[p],heights[v]+1);
              if(num_children[p]--==0,  \\ pre-decrement
                 seq[new_upto++]=p)));
        phase=upto+1;
        upto=new_upto;
        \\ my(s=vecsort(Vec(seq[phase..upto])));
        \\ for(i=1,#s, seq[phase+i-1]=s[i]);
       );
  print("heights ",heights);
  seq;

\\-----------------------------------------------------------------------------
\\ vpar_from_subtree_size

  \\ print("vec "vec);
  my(num_children=vpar_num_children_vector(vec),
     sizes = vectorsmall(#vec,v,1),
     lists = vector(#vec,v, List([])),
     seq   = vector(#vec),
     phase = 1,
     upto  = 0,
     vecpos=0,
     vpar=vector(#vec));
  \\ print("  num_children "num_children);

  for(v=1,#vec,  \\ leaf vertices
     if(num_children[v]==0,
        listput(lists[1],v)));

  for(s=1,#vec,
     my(l=lists[s]);
     l=vecsort(l);
     \\ print("s="s" list "Vec(l));
     for(i=1,#l,
        my(v=l[i], p=vec[vecpos++]);
        \\ print("  v="v" at vecpos="vecpos" p="p);
        vpar[v]=p;
        if(p,
           sizes[p] += sizes[v];
           if(num_children[p]--==0,  \\ pre-decrement
              \\ print("    p="p" now childless, size ",sizes[p]);
              listput(lists[sizes[p]], p)));
        );
  );
  vpar;

\\-----------------------------------------------------------------------------

\\ vpar_from_upascend(can) =
\\ {
\\   my(counts=vector(#can));
\\   for(i=1,#can,if(can[i],counts[can[i]]++));
\\   my(notlist=List(Vec(select(c->c==0,counts,1))));
\\   my(vpar=vector(#can));
\\   for(i=1,#can,
\\      my(v=if(#notlist,notlist[1],0));
\\      my(p=can[i]);
\\      \\ print("i="i);
\\      \\ print(" counts  "counts);
\\      \\ print(" notlist "Vec(notlist));
\\      \\ print(" v="v" p="p);
\\      vpar[v]=p;
\\      if(counts[v]==0,
\\         \\ print(" remove v from notlist");
\\         listpop(notlist,setsearch(notlist,v)));
\\      counts[v]++;
\\      if(p,
\\         counts[p]--;
\\         if(counts[p]==0,
\\            \\ print(" insert p into notlist");
\\            listinsert(notlist,p,setsearch(notlist,p,1)))));
\\   vpar;
\\ }

\\-----------------------------------------------------------------------------
\\ vpar_matchings_count() own code rather than using matchpoly

vpar_matchings_count(vpar) =
{
  \\ paired[v] is how many matchings have v already paired.
  \\ unpaired[v] is how many matchings have v not paired.
  \\ The code works upwards from leaf nodes to apply the counts at v to
  \\ parent p.  v_either is all matchings in the subtree at and below v,
  \\ either v paired or unpaired.
  \\
  \\   unpaired[p] = prod v_either of its children
  \\   paired[p] = sum unpaired[v] * others either, ie. it pairs with one
  \\   child v which must be unpaired, and the other children can be anything.
  \\
  \\ Initially all paired[v]=0.  At the leaf nodes there are no paired
  \\ matchings below.  Nodes above the leaves start as no matchings and sum
  \\ the children.  Initially all unpaired[v]=1 which is the empty matching
  \\ as the sole possibility in the tree comprising vertex v alone.
  \\
  \\ This approach roughly the same as
  \\
  \\     Jingyuan Zhang, Xufeng Chen, Weigang Sun, "A Linear-Time Algorithm
  \\     for the Hosoya Index of an Arbitrary Tree", MATCH Communications in
  \\     Mathematical and in Computer Chemistry, volume 75, 2016, pages
  \\     703-714, ISSN 0340-6253.
  \\
  \\ They express the algorithm by a rational alpha(v) at each vertex, which
  \\ is summed at the end.  The paired and unpaired counts are in their beta
  \\ as m(v) and m-(v).
  \\
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     paired  =vector(#vpar),
     unpaired=vector(#vpar,v,1),
     total=1);
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v],
        v_either=paired[v]+unpaired[v]);
     if(p,
        paired[p] = paired[p]*v_either + unpaired[p]*unpaired[v];
        unpaired[p] *= v_either;
        ,
        total *= v_either);  \\ at root
     paired[v]   =     \\ kill bignum intermediate values
     unpaired[v] = 0);
  total;
}

\\-----------------------------------------------------------------------------
\\ Sequences

\\ no, just vecsort is enough
vpar_seq_sort(keys,down=0) = vecsort(keys, , 1+if(down,4));


\\-----------------------------------------------------------------------------


\\ Postorder too?
\\ p>v
\\
\\ last child followed by p

vpar_is_preorder(vpar) =
{
  \\ depths[v] is the depth of vertex v, as an index into ancestors[] vector
  \\ so root is depths[r]=1.  depths[v]=1 if not yet determined.
  \\ ancestors[d] is the vertex number which is most recently seen at depth d,
  \\ starting d=1 for root vertices.
  \\
  \\ prev_depth is the depth of vertex v-1, or 1 initially.
  \\ Each vertex v must have its parent depth d in the range 1..prev_depth,
  \\ and then its parent be p==ancestors[d] meaning most recent seen at that d.
  \\
  \\ Each v must have preceding parent p<v.  That p must be the most
  \\ recently seen vertex at its depth d.  The depth of v is then d+1.

  my(depths    = vectorsmall(#vpar,v,1),
     ancestors = vectorsmall(#vpar),
     prev_depth=1, p);
  for(v=1,#vpar,
     my(d=1);
     if((p=vpar[v]),
        d=depths[p];
        if(d>prev_depth || ancestors[d]!=p,
           \\ print("  p="p" depth "depths[p]" "depths);
           return(0));
        d++);
     prev_depth=depths[v]=d;
     ancestors[d]=v);
  1;
}
{
addhelp(vpar_is_preorder,
"bool = vpar_is_preorder(vpar)
Return 1 if vpar is labelled in pre-order sequence, or return 0 if not.
This means vpar_seq_porder(vpar) is the identity permutation 1..#vpar.");
}


check_exact(vpar_is_preorder([]), 1);
check_exact(vpar_is_preorder([0]), 1);
check_exact(vpar_is_preorder([0,0,0]), 1);
check_exact(vpar_is_preorder([0,0,0,1]), 0);

check_exact(vpar_is_preorder([0,1]), 1);
check_exact(vpar_is_preorder([0,0]), 1);

test_is_preorder_by_seq(vpar) = \
  (vpar_seq_preorder(vpar) == vectorsmall(#vpar,v,v));

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_exact(vpar_is_preorder(vpar), test_is_preorder_by_seq(vpar),
                "vpar_is_preorder() vs test_is_preorder_by_seq() on sample"));
}
{
  my(want_count_forests = [1,2,5,14,42,132]); \\ A000108 Catalan
  my(want_count_trees   = [1,1,2,5,14,42,132]);  \\ same one later
  for(n=1,5,
     my(count_trees=0, count_forests=0);
     forvec(vec=vpar_forvec_from(n,1), \\ forest
           my(vpar=vpar_INTERNAL_from_upascend(vec));
           my(got=vpar_is_preorder(vpar));
           check_exact(got, test_is_preorder_by_seq(vpar),
                       "vpar_is_preorder() vs test_is_preorder_by_seq()");
           count_forests += got;
           count_trees   += (got && vpar_is_tree(vpar)));
     check_exact(count_forests, want_count_forests[n],
                 Str("count_forests n="n));
     check_exact(count_trees,   want_count_trees[n], "count_trees");
  );
}


\\-----------------------------------------------------------------------------
\\ vpar_relabel_map()
\\
\\ FIXME: Convenient for making a few changes in a single go, but maybe
\\ vecsmall perm is more usual and good enough efficiency-wise and building
\\ perms by multiplying etc.

{
\\ MY-CHECK-GP-FUNCTION-DEFINITION: vpar_relabel_map(vpar,m)
if(type(mapisdefined)=="t_CLOSURE",  \\ when Map() available, gp 2.9 up
eval("
vpar_relabel_map(vpar,m) =
  my(inv=Map(Mat(Vecrev(Mat(m)))));
  vector(#vpar,v,
         mapisdefined(inv,v,&v);
         my(p=vpar[v]);
         mapisdefined(m,p,&p);
         p);
"));
}
{
if(type(mapisdefined)=="t_CLOSURE",  \\ when Map() available, gp 2.9 up
addhelp(vpar_relabel_map,
"vpar = vpar_relabel_map(vpar,m)
Return vpar with its vertices relabelled according to the permutation
in Map m.  Each vertex v which appears in m is becomes number mapget(m,v).
Vertices not appearing as keys in m are unchanged.
m must be a permutation of those v appearing in its keys.");
);
}

\\ vpar_relabel_map()
test_have_Map=(type(mapisdefined)=="t_CLOSURE");
{
  if(test_have_Map,
     check_exact(vpar_relabel_map([0,1,2], Map([2,1; 1,2])),
                 [2,0,1],
                 "vpar_relabel_map()");
    );
}


\\-----------------------------------------------------------------------------
\\ vpar_from_successive_paths, upwards only

  \\ my(vpar = vector(#vec,v,-1),
  \\    upto = 0,
  \\    v    = 0);
  \\ for(i=1,#vec,
  \\    while(!v || vpar[v]>=0,   \\ top of this chain
  \\          v=upto++);          \\ pre-increment next successive
  \\    \\ print("i="i" v="v" parent "vec[i]" (seen "if(vec[i],vpar[vec[i]])")");
  \\    vpar[v]=vec[i];
  \\    v=vec[i]);                \\ parent
  \\ vpar;



\\-----------------------------------------------------------------------------
\\ downleaves - now down=1 option of vpar_seq_childless_paths()

\\ path down from root or highest to successive leaf 1..n

vpar_seq_downleaves(vpar) =
{
  my(c=0,
     chain=vpar_INTERNAL_num_children_vecsmall(vpar),
     prev_leaf=0);
  for(l=1,#vpar,
     if(chain[l]==0,  \\ leaf
        \\ print("l="l);
        my(v=l);
        while(1,
              my(p=vpar[v]);
              \\ print(" v="v" p="p);
              if(!p || chain[p]<0,
                 if(!c,c=v);
                 if(prev_leaf,chain[prev_leaf]=-v);
                 prev_leaf=l;
                 break());
              chain[p]=-v;
              v=p;
             )));
  \\ print("chain "Vec(chain));
  my(seq=vectorsmall(#vpar), upto=0);
  while(c,
        \\ print("seq "c);
        seq[upto++]=c;
        c=-chain[c]);
  seq;
}
{
addhelp(vpar_seq_downleaves,
"seq = vpar_seq_downleaves(vpar)
Return a Vecsmall of vertex numbers 1..#vpar in sequence down to each
childless vertex, for those childless in ascending order.
The return type is Vecsmall since this is a permutation of the integers
1..#vpar.

Suppose x is the smallest childless vertex number.  The sequence begins with
root down to x, inclusive.  Then sequence down to the next smallest number
childless vertex y, commencing at the top-most vertex not yet in the
sequence.  For a single tree this is some child of the first path, but if a
forest then y might be in another component tree and in that case down from
the root of its component.  And so on paths down to each childless vertex,
with those in ascending order.  This is a downwards sequence since each
parent precedes its children.

This is the vertex order described in Knuth volume 1 section 2.3.4.4
exercise ...

An equivalent definition is to sort vertex numbers in ascending order of
their smallest leaf (childless) descendant, and among those of equal
smallest descendant by ascending depth (so downwards).");
}

vpar_from_downleaves(vec) =
{
  \\ FIXME: not working

  \\ Could keep the attachments as a linked list instead of a List().
  \\ In both cases the number of attachments is at most #vpar, though the
  \\ List() might better suit cases where there's only a few final leaves.
  \\
  my(vpar=vector(#vec,i,-1),
     attachments=List([]),
     p);
  for(i=1,#vec,
     my(v=vec[i]);
     if(v,
        if(vpar[v]>=0,  \\ if already seen
           listput(attachments,vec[i-1]),
           vpar[v]=p));
     p=v);
  listput(attachments,p);

  \\ Vertices with parents not already determined are leaves.
  \\ Their parents are the attachments list, in sequence.
  my(i=0);
  \\ print(attachments);
  for(v=1,#vpar,
     \\ print("i="i);
     if(vpar[v]<0, vpar[v]=attachments[i++])); \\ pre-increment
  vpar;
}
{
addhelp(vpar_from_downleaves,
"vpar = vpar_from_downleaves(vec)
Return vpar from a vector of parents of downleaves order, ie. from a vec
which was formed vec=vecextract(vpar, vpar_seq_downleaves(vpar)).

vec is a vector of n integers.
Empty vec==[] is the empty tree vpar==[].
Otherwise must have vec[1]==0, then vec[2..n] each in the range 1..n for a
single tree or 0..n for a forest.
All labelled oriented trees (or forests) of n vertices are in one-to-one
correspondence with vectors of this form.

The initial vec[1]=0 is included so the empty tree vec==[] is distinguished
from a tree of 1 vertex vec=[0].

The second entry vec[2] is the root vertex number.  It can be made a fixed
value (eg. 1) to get all unrooted oriented trees.");
}


\\-----------------------------------------------------------------------------
\\ vpar_seq_downsuccessive() now option of vpar_seq_childless_paths()

vpar_seq_downsuccessive(vpar) =
{
  my(chain = vectorsmall(#vpar),
     seq   = chain,
     upto  = 0);
  for(i=1,#vpar,
     if(!chain[i],
        \\ print("i="i);
        my(v=i,p);
        while((p=vpar[v]),
              if(chain[p], break());
              \\ print(" chain down "p" -> "v);
              chain[p]=v; v=p);
        \\ print(" top at "v);
        while(seq[upto++]=v;
              \\ print(" seq "v);
              v!=i,
              v=chain[v]);
        chain[v]=1));
  seq;
}
{
addhelp(vpar_seq_downsuccessive,
"seq = vpar_seq_downsuccessive(vpar)
Return a Vecsmall of vertex numbers 1..#vpar in sequence of paths down to
ascending vertex numbers not already seen.  The return type is Vecsmall
since this is a permutation of the integers 1..#vpar.

The first path is from the root down to vertex 1, inclusive.  The next path
is down to the smallest vertex not already seen, starting from its highest
ancestor not already seen.  This is the vertex order of

    Tamas Fleiner, \"On Prufer Codes \", Egervary Research Group Technical
    Report TR-2005-16.  https://web.cs.elte.hu/egres/tr/egres-05-16.pdf

An equivalent definition is to sort vertex numbers in ascending order of
smallest descendant, and among those of equal descendant by ascending depth
(so down from the root).

vpar can be a forest, in which case the path down to vertex 1 etc is down
from the root of the component tree containing 1, etc.");
}

\\     2    seq 2,1,3
\\    /     vec 0,2,1
\\   1
\\  /
\\ 3
vpar_INTERNAL_seqfrom_downsuccessive(vec) =
{
  my(seq  = vectorsmall(#vec),
     seen = vectorsmall(#vec),
     upto = 0,
     v    = 0,
     p);
  for(i=1,#vec,
     v=if(i<#vec,vec[i+1],0);
     while(!v || seen[v], v=upto++);
     seq[i]=v;
     seen[v]=1);
  seq;
}
vpar_from_downsuccessive(vec) =
{
  \\ Path goes downwards so vec[i] contains the parent of vec[i+1].
  \\ Except if vec[i+1] has already been seen, ie. already been determined in
  \\ vpar, then that means the start of a new path.  In this case vec[i] is
  \\ the parent of the bottom of the path, ie. the successive vertices, which
  \\ is maintained in "upto".
  \\ vec[1]==0 always, since it is parent of the root for the first path
  \\ downwards, so it is unused.

  my(vpar = vector(#vec,v,-1),
     upto = 0);
  for(i=1,#vec,
     my(v=if(i<#vec,vec[i+1],0));
     while(!v || vpar[v]>=0, v=upto++);
     vpar[v]=vec[i]);
  vpar;
}
\\ vpar_from_downsuccessive(vec) = \
\\   vecextract(vec, vpar_INTERNAL_seqfrom_downsuccessive(vec)^-1);


      \\ ["downsuccessive",
      \\  vpar_seq_downsuccessive,
      \\  vpar_from_downsuccessive,
      \\  vpar_INTERNAL_seqfrom_downsuccessive,
      \\  'downwards, 0 ],



check_exact(vpar_seq_downsuccessive([2,3,4,5,6,0]), Vecsmall([6,5,4,3,2,1]));
check_exact(vpar_seq_downsuccessive([2,3,4,5,0]), Vecsmall([5,4,3,2,1]));
check_exact(vpar_seq_downsuccessive([2,3,4,0]), Vecsmall([4,3,2,1]));

check_exact(vpar_seq_downsuccessive([]), Vecsmall([]), \
            "vpar_seq_downsuccessive() empty");
check_exact(vpar_seq_downsuccessive([0]), Vecsmall([1]));
check_exact(vpar_seq_downsuccessive([0,1]), Vecsmall([1,2]), \
            "vpar_seq_downsuccessive() 2-->1");

\\ 1 --> 2  path root 2 down to 1
check_exact(vpar_seq_downsuccessive([2,0]), Vecsmall([2,1]));

{
  \\        2
  \\      / | \
  \\    1   4   3
  \\    |
  \\    5
  \\    |
  \\    6
  my(vpar = [2, 0, 2, 2, 1, 5]);
  check_exact(vpar_seq_downsuccessive(vpar), Vecsmall([2,1, 3,4,5,6]),
              "vpar_seq_downsuccessive()");
}

test_seq_downsuccessive_by_paths(vpar) =
{
  my(seq=List([]));
  for(v=1,#vpar,
     my(path=Vecrev(test_path_to_root_vertices(vpar,v)),
        set=Set(seq));
     for(i=1,#path,
        if(!setsearch(set,path[i]), listput(seq,path[i]))));
  Vecsmall(seq);
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_exact(vpar_seq_downsuccessive(vpar),
                 test_seq_downsuccessive_by_paths(vpar)));
}

\\ same as sort, as described in the help string
test_seq_downsuccessive_by_sort(vpar) =
{
  my(min_descendant = vpar_minmax_descendants_vector(vpar,min),
     depths         = vpar_depths_vector(vpar),
     keys = vector(#vpar,v, [min_descendant[v],depths[v]]));
  vecsort(keys,,1);
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_exact(vpar_seq_downsuccessive(vpar),
                 test_seq_downsuccessive_by_sort(vpar)));
}

{
  for(n=0,6,
     my(count=0);
     forvec(vec=vpar_forvec_from(n,0), \\ trees
           my(vpar=vpar_from_upascend(vec));
           my(seq =vpar_seq_downsuccessive(vpar),
              vec =vecextract(vpar,seq));

           my(vpar_again = vpar_from_downsuccessive(vec));
           check_exact(vpar_again,vpar,
                       "vpar_seq_downsuccessive() round trip");

           my(seq_again = vpar_INTERNAL_seqfrom_downsuccessive(vec));
           check_exact(seq_again,seq,
                       "vpar_INTERNAL_seqfrom_downsuccessive() round trip");

           vpar_again = vecextract(vec,seq^-1);
           check_exact(vpar_again,vpar,
                       "vpar_seq_downsuccessive() vpar by vecextract");

        ));
}



\\------------------------------------------------------------------------------

\\ func() changes to lexical variables don't persist beyond its call, so
\\ callback no good for accumulating results (except into global variables)
vpar_traverse_postorder(vpar,func) =
{
  my(num_children=vpar_num_children_vector(vpar));
  for(v=1,#vpar,
     my(l=v);
     while(num_children[l]==0,
           func(l);
           l=vpar[l]; \\ to parent
           if(!l,break());
           num_children[l]--));
}

\\-----------------------------------------------------------------------------
\\ porder

{
addhelp(vpar_seq_porder,
"seq = vpar_seq_porder(vpar,post=0)
Return a Vecsmall of the vertex numbers 1..#vpar in pre-order sequence.
The return is Vecsmall since it is a permutation of the integers 1..#vpar.

Pre-order is vertex v followed by the subtree of its smallest child vertex
number in pre-order, then next bigger child vertex subtree, etc.

Optional parameter \"post\" can be 1 for post-order (depth first) instead.
Post-order is the smallest child subtree in post-order, then next bigger
child subtree, etc, and finally v itself after all subtrees.

vpar can be a forest, in which case the component tree with smallest root is
first, then next bigger root, etc.");
}
\\-----------------------------------------------------------------------------

\\ Variant of dandelion which cycles all vertices in the runs between
\\ v<=min(ancestors).

  \\ my(seq=vectorsmall(#vpar,v,v),
  \\    v=1,prev=0);
  \\ while((v=vpar[v]), seq[v]=prev; prev=v);
  \\ print("root "prev);
  \\ v=prev; prev=1;
  \\ seq[1]=v;
  \\ my(m=v,top=1);
  \\ while(v>=1,
  \\       my(d=seq[v]);
  \\       print("at v="v" prev="prev" m="m);
  \\       if(v<m,
  \\          m=v;
  \\          print(" store top="top" "prev);
  \\          seq[top]=prev;
  \\          top=prev=v;
  \\          ,
  \\          print(" store prev, "v" "prev);
  \\          seq[v]=prev);
  \\       prev=v;
  \\       v=d);
  \\ seq;


\\ wrong bits of dandelion:

  \\ if(#vpar==0,Vecsmall([]),
  \\    vpar_INTERNAL_seq_rotate_up(vpar_minmax_upto_root_vector(vpar,1,min)));

  \\ if(#vpar==0,Vecsmall([]),
  \\    my(upto=vpar_minmax_upto_root_vector(vpar,1,max));
  \\    \\ upto[1]=1;
  \\    vpar_INTERNAL_seq_rotate_down(upto));


  \\ my(seq=vectorsmall(#vpar,v,v),
  \\    v=1,prev=0);
  \\ while((v=vpar[v]), seq[v]=prev; prev=v);
  \\ print("root "prev);
  \\ v=prev; prev=1;
  \\ seq[1]=v;
  \\ my(m=v,top=1);
  \\ while(v>=1,
  \\       my(d=seq[v]);
  \\       print("at v="v" prev="prev" m="m);
  \\       if(v<m,
  \\          m=v;
  \\          print(" store top="top" "prev);
  \\          seq[top]=prev;
  \\          top=prev=v;
  \\          ,
  \\          print(" store prev, "v" "prev);
  \\          seq[v]=prev);
  \\       prev=v;
  \\       v=d);
  \\ seq;
\\-----------------------------------------------------------------------------


vpar_count_free_rooted(n,forest=0) =
{
  \\ Joerg Arndt in A000081
  if(n==0,1,
     my(a=vector(n,i,1));
     for(i=1,n-1,
        a[i+1] = 1/i * sum(k=1,i, sumdiv(k,d, d*a[d]) * a[i+1-k]));
     a[n]);
}

{
  \\    n= 0  1  2  3  4  5   6
  my(want=[1, 1, 1, 2, 4, 9, 20, 48, 115, 286, 719, 1842, 4766, 12486, 32973,
           87811, 235381, 634847, 1721159, 4688676, 12826228, 35221832]);
  check_exact(vector(#want,n,n--; vpar_count_free_rooted(n)), want);
}



\\-----------------------------------------------------------------------------
\\ Count Unlabelled Trees

\\ p is a vector which is a partition (integers summing to some n).
test_partition_unduplicate(p) =
{
  my(ret=1,d=1);
  for(i=2,#p, if(p[i]==p[i-1], d++, ret*=d!;d=1));
  ret*d!;
}
check_exact(test_partition_unduplicate([1,2,3]), 1);
check_exact(test_partition_unduplicate([1,3,3]), 2!);
check_exact(test_partition_unduplicate([1,3,3,3,4,4,4,4]), 3!*4!);


\\-----------------------------------------------------------------------------
\\ Exponential Transform, func version

\\ func is a function to be called num=func(k) for how many labelled rooted
\\ trees of k vertices have some property.
\\ Return a vector a[1..n] where each a[i] is the number of labelled rooted
\\ forests of i vertices with this property.
\\
\\ Such a forest can be a single tree of n vertices, or a tree of t vertices
\\ which contains vertex 1, binomial choice of t-1 others, times a[n-t] ways
\\ for the rest of the vertices to make a forest.
\\
\\ These selections of t many to make a tree become a composition of n
\\ (partition with order), and multinomial to choose the ways vertices can
\\ be assigned to those compositions, but divide out num components
\\ factorial so the multinomial as want to order the components by smallest
\\ vertex they contain.
\\
vpar_INTERNAL_count_forests_by_trees_vector(n,func) =
{
  my(a=vector(n,i,1), f=vector(n));
  for(n=1,n,
     f[n] = func(n);
     a[n] = f[n] + sum(t=1,n-1, binomial(n-1,t-1) * f[t] * a[n-t]));
  a;
}

\\-----------------------------------------------------------------------------
\\ vpar_max_ancestor_of_vertex
\\ vpar_max_ancestors_vector
\\ prefer min/max single func

\\     vec = vpar_max_ancestor_of_vertex(vpar,v)
\\     vec = vpar_max_ancestors_vector(vpar)

\\ Inclusive of v or not?
\\ minmax maybe
vpar_max_ancestor_of_vertex(vpar,v) = \
  my(m=v); while((v=vpar[v]), m=max(m,v)); m;
{
addhelp(vpar_max_ancestor_of_vertex,
"ancestor = vpar_max_ancestor_of_vertex(vpar,v)
Return the biggest vertex number at or above v.
v itself is included, so if all above are smaller, or there are none above
at all, then the return is v.");
}

vpar_max_ancestors_vector(vpar) = \
  vpar_minmax_ancestors_vector(vpar,max);
{
addhelp(vpar_max_ancestors_vector,
"vec = vpar_max_ancestors_vector(vpar)
Return a vector where vec[v] is the biggest vertex number at or above v.
This is the same as vec[v] = vpar_max_ancestor_of_vertex(vpar,v) but
calculated all together.

vpar can have cycles (normally not allowed for vpar.gp functions).  In a
cycle all are ancestors of each other, so their biggest ancestor is the
biggest in the cycle.");
}

\\-----------------------------------------------------------------------------

vpar_subtree_nest(vpar,v,sort=0) =
{
  my(nest=apply(c->vpar_subtree_nest(vpar,c,sort), vpar_children_of_vertex(vpar,v)));
  if(sort,vecsort(nest),nest);
}
{
addhelp(vpar_subtree_nest,
"vec = vpar_subtree_nest(vpar,v)
Return a vector of nested vectors which is the tree structure below v.
See vpar_nest() for a description of the return.");
}




  \\ WRONG, children not necessarily seen in ascending order.
  \\ print("vpar_nest() "vpar" sort="sort);
  \\ my(num_children = vectorsmall(#vpar+1));
  \\ for(v=1,#vpar, num_children[vpar[v]+1]++);
  \\ my(nests=vector(#num_children,i, vector(num_children[i])));
  \\ print("initial num_children "Vec(num_children));
  \\ print("initial nests        "nests);
  \\
  \\ for(i=1,#vpar,
  \\    print("i="i);
  \\    my(v=i);
  \\    while(v && v<=i && num_children[v+1]==0,
  \\          print(" consider v="v);
  \\          nests[v+1]=if(sort,vecsort(nests[v+1]),Vecrev(nests[v+1]));
  \\          my(p=vpar[v]);
  \\          print(" store p="p" pos "num_children[p+1]" "nests[v+1]);
  \\          nests[p+1][num_children[p+1]] = nests[v+1];
  \\          num_children[p+1]--;
  \\          v=p));
  \\
  \\ print("return "if(sort,vecsort(nests[1]),Vecrev(nests[1])));
  \\ if(sort,vecsort(nests[1]),Vecrev(nests[1]));


\\-----------------------------------------------------------------------------
\\ vpar_INTERNAL_for_free_isomorphism()

\\ No, better to do the steps on both trees so can stop on earlier mismatch.
\\
\\ Return vpar changed suitably for testing isomorphism as a free tree.
\\ For each component tree add a new vertex made the parent of the centre
\\ vertex or vertices of that tree.
\\ Each component is always changed as otherwise ...
\\ Each uni-central component tree is re-rooted to its centre as root.
\\ Each bi-central component tree is re-rooted to a new vertex which is
\\ parent of the two centres.  (Separate new parent for each component tree.)
\\
vpar_INTERNAL_for_free_isomorphism(vpar) =
{
  \\ ENHANCE-ME: could stop if vpar1,vpar2 have different numbers of
  \\ centrals,bicentrals.
  my(c=vpar_centres_of_forest(vpar),
     bicentrals=sum(i=1,#c,#c[i]==2));

  \\ if any bicentrals then extra vertex above each component
  if(bicentrals,
     vpar=Vec(vpar,#vpar+vpar_num_roots(vpar)));

  \\ re-root to centre of each component, or if any bicentrals to a new
  \\ extra vertex above the centre of each component.
  for(i=1,#c,
     my(v=c[i][1], p=if(bicentrals,#vpar-i+1,0));  \\ desired parent for v
     while(v, my(new_v=vpar[v]); vpar[v]=p; p=v; v=new_v);
     if(#c[i]>1, vpar[c[i][2]]=#vpar-i+1));
  vpar;


  \\ vpar=Vec(vpar,#vpar+sum(i=1,#c,#c[i]==2));  \\ extra each bicentral component
  \\ my(c=vpar_centres(vpar);
  \\ if(#c==1,
  \\    vpar_reroot(vpar,c[1]);
  \\    ,
  \\    if(#c==2,
  \\       vpar=Vec(vpar,#vpar+1);
  \\       if(vpar[c[2]]==c[1], c=Vecrev(c));
  \\       vpar=vpar_reroot(vpar,c[2]);
  \\       vpar[c[1]]=#vpar;
  \\       vpar[c[2]]=#vpar);
}



check_exact(vpar_centres([2,0,1]), [1]);
check_exact(vpar_reroot([2,0,1],1), [0,1,1], "vpar_reroot()");

\\ 1 -> 2 -> 3  becomes 1 -> 2 <- 3    all unicentral
check_exact(vpar_INTERNAL_for_free_isomorphism([0,1,2]), [2,0,2], \
            "vpar_INTERNAL_for_free_isomorphism()");

\\ 3 -> 1 -> 2  becomes 3 -> 1 <- 2
check_exact(vpar_INTERNAL_for_free_isomorphism([2,0,1]), [0,1,1], \
            "vpar_INTERNAL_for_free_isomorphism()");

\\ unicental,bicentral both new above
\\                                    7           6
\\                                    |          / \
\\ 2 -> 1 <- 3   4 -> 5  becomes 2 -> 1 <- 3    4   5
check_exact(vpar_INTERNAL_for_free_isomorphism([0,1,1,5,0]), [7,1,1,6,6,0,0]);

\\                                      5
\\                                    /  \
\\ 1 -> 2 -> 3 -> 4  becomes   1 -> 2      3 <- 4
check_exact(vpar_INTERNAL_for_free_isomorphism([0,1,2,3]), [2,5,5,3,0], \
            "vpar_INTERNAL_for_free_isomorphism()");


\\ 1 -> 2 -> 3 -> 4  becomes   1 -> 2 -> 10 <- 3 <- 4
\\ 5 <- 6 <- 7 <- 8  becomes   5 -> 6 -> 9  <- 7 <- 8
check_exact(vpar_INTERNAL_for_free_isomorphism([0,1,2,3, 0,5,6,7]), \
            [2,10,10,3, 6,9,9,7, 0,0], \
            "vpar_INTERNAL_for_free_isomorphism()");


\\-----------------------------------------------------------------------------
\\ vpar_INTERNAL_strongly_monotone_transform()
\\ without row cumulative


\\ vpar_INTERNAL_strongly_monotone_transform() takes a vector "a" and
\\ returns a resulting vector of vectors
\\
\\     v = [ [x],
\\           [x,x],
\\           [x,x,x] ]
\\
\\ v[n][m] is count how many strongly monotone partitions of labelled
\\ vertices 1..n with have m as the maximum in the first partition block.
\\
\\ Each count has a[t] multiplied in so that each block of t many vertices
\\ is reckoned as having a[t] arrangements.  If all a[t]==1 then the result
\\ is just count of strongly monotone partitions, bigger like vpar_count(t)
\\ is for structure within the blocks such as a component tree of a forest.
\\
\\ Blocks are reckoned in order of their minimum vertex, so first block
\\ contains 1, second contains smallest not in the first, etc.
\\
\\ The sum is over t many vertices in the first block.  Its maximum is to
\\ be m.  The binomial chooses which of positions 2..m-1 are to make up the
\\ t many for this block.
\\
\\ h is the maximum vertex in the second block.  It must be > m, that being
\\ the strongly monotone condition.  The "recurse" to v[n-t][h-t] is count
\\ with the first block removed (and rest renumbered down to 1..n-t).
\\

  \\ special case for m==n taking simply a[n] since sum h=m+1 to n would be 0
  my(v=vector(#a),
     b=v);
  for(n=1,#a,
     v[n] = vector(n,m,  \\ m = 1 to n
                   if(m==n, a[n],
                      sum(t=min(2,m), m,
                            a[t]
                            * binomial(max(0,m-2),max(0,t-2))
                            * sum(h=m+1, n, v[n-t][h-t])))));
  v;


  \\ if(n==0, 1,
  \\    vecsum(vpar_INTERNAL_strongly_monotone_transform(vector(n,i,vpar_count(i)))
  \\           [n]));


  \\ if(n==0, 1,
  \\    vecsum(vpar_INTERNAL_strongly_monotone_transform(vector(n,i,1))
  \\           [n]));


\\-----------------------------------------------------------------------------

\\     bool = vpar_is_monotonic(vpar)

\\ this any good?
vpar_is_monotonic(vpar) = vpar_is_upwards(vpar) || vpar_is_downwards(vpar);
{
addhelp(vpar_is_monotonic,
"bool = vpar_is_monotonic(vpar)
Return 1 if vpar is labelled monotonic, meaning either upwards or downwards
per vpar_is_upwards() and vpar_is_downwards().  Return 0 if not");
}


\\-----------------------------------------------------------------------------
\\ vpar_count_min_ancestor_increasing()

  \\ i^(i-2) tree with root=i at top.
  \\ Or an existing tree of p many vertices to which put an initial path run
  \\ of i-p many with top=i, and any of a[p] such trees above.
  \\ For a forest p+1 places since path can be a new run in the forest.

  \\ if(n==0,1,
  \\    my(a=vector(n));
  \\    for(i=1,n,
  \\       a[i] = i^(i-2) + sum(p=1,i-1, a[p] * (p+forest) * (i-p)^(i-p-2)));
  \\    a[n]);

\\ Return the number of root=1 trees of n vertices.
\\ Same as vpar_count(n,-1) (which is not documented yet), but as a 1-arg
\\ function for use here.
vpar_INTERNAL_count_root1(n) = if(n<2,1,n^(n-2));

\\
\\
\\ Return the number of vpar trees made by attaching "func" pieces.  func is
\\ a function to be called num=func(t).  A subtree of t vertices is attached
\\ under a vpar of n-t vertices.  func(t) is how many such subtrees of t
\\ vertices.  Such a subtree is attached at any of the n-t existing vertices
\\ for a tree, or at n-t+1 places for a forest, the extra being subtree t
\\ with no parent so a new component tree of the forest.


(n-1)! is the tree as a single path with n at the bottom so max descendant
of all others, and those others arranged in (n-1)! permutations.  Otherwise
a tree of p vertices under which a final path of n-p vertices with n at the
bottom.  This is C(p) trees, path attached at any of their p vertices, and
(n-p-1)!  permutations of the path.


\\-----------------------------------------------------------------------------

\\ No, order within the size lists unspecified.

test_subtree_sizes_by_children_vectors(vpar) =
{
  my(vec=vpar_children_vector(vpar));
  for(v=1,#vec,
     vec[v]=apply(c->#vpar_subtree_vertices(vpar,c),vec[v]));
  vec;
}

{
  for(n=3,8,
     for(rep=1,if(n==3,1,10),
        my(vpar=vpar_make_random(n));   \\ forest
        my(got =vpar_children_subtree_sizes_vector(vpar),
           want=test_subtree_sizes_by_children_vectors(vpar));
        check_equal(got,want, "vpar_subtree_sizes_vector")));
}


\\-----------------------------------------------------------------------------
\\ vpar_reachability_matrix() maybe with directions

  \\ matrix(#vpar,#vpar,u,v,
  \\        if(vpar_path_length(vpar,u,v)>0, 1, 0));

         \\ if(directed==0, vpar_is_ancestor(vpar,u,v)||vpar_is_ancestor(vpar,v,u),
         \\    directed==1, vpar_is_ancestor(vpar,u,v),
         \\    directed==2, vpar_is_ancestor(vpar,v,u),
         \\    error()));

\\-----------------------------------------------------------------------------
\\ vpar_is_domset_flags() by second pass

  my(dominated=Vecsmall(flags,#vpar));  \\ all of flags itself dominated
  for(v=1,#flags,
     my(p=vpar[v]);
     if(p,
        dominated[p]+=flags[v];     \\ p dominated by v
        dominated[v]+=flags[p]));   \\ v dominated by p
  for(v=1,#vpar, if(!dominated[v],return(0)));
  1;





\\-----------------------------------------------------------------------------
\\ wrong bit of subtree heights vector

  my(heights=vector(#vpar));
  for(v=1,#vpar,
     if(vpar[v], heights[vpar[v]]=-1));  \\ set -1 anything with child
print(heights);
  for(i=1,#vpar,
     if(!heights[i],  \\ if childless
        print("i="i);
        my(v=i, h=-1);
        until(!(v=vpar[v]) || heights[v]>0, print("set "v); heights[v]=h++)));
  heights;


\\-----------------------------------------------------------------------------
\\ vpar_critical_ind_minimum_diff()

\\ Minimum diff among non-empties.
vpar_critical_ind_minimum_diff(vpar) =
{
  \\ print("vpar_critical_inddiff() "vpar);

  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     with           = vectorsmall(#vpar,v,1),
     none           = 2*#vpar,
     without_unseen = vectorsmall(#vpar,v,none),
     without_seen   = vectorsmall(#vpar,v,none),
     ret = 0);
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v],
        v_min = vecmin([with[v], without_seen[v],
                        if(without_unseen[v]==none,0,without_unseen[v])]));
     \\ print(" v="v" with "with[v]" without_seen "without_seen[v]" without_unseen "without_unseen[v]);

     if(p,
        with[p] += min(if(without_unseen[v]==none, 0, without_unseen[v]) - 1,
                       if(without_seen[v]==none, none, without_seen[v]));

        without_seen[p]
        = min(if(without_seen[p]==none, none,
                 without_seen[p] + v_min),      \\ extend
              if(without_unseen[p]==none, 0, without_unseen[p])
                 + with[v] - 1);                \\ build, new neighbour

        my(v_without = min(without_seen[v],
                           if(without_unseen[v]==none,0,without_unseen[v])));
        \\ print("  v_without "v_without"  p without_unseen "without_unseen[p]);
        without_unseen[p] = if(without_unseen[p]==none && v_without==none, none,
                               if(without_unseen[p]==none,0,without_unseen[p])
                               + v_without);

        \\ print("  set p="p" with "with[p]" without_seen "without_seen[p]" without_unseen "without_unseen[p]);
        ,
        ret += vecmin([with[v], without_seen[v],without_unseen[v]])));

  ret;
}

\\ 1-2 = -1 or 2-1 = 1
check_equal(vpar_critical_ind_minimum_diff(vpar_make_path(3)), 1-2); \\ -1
check_equal(vpar_critical_ind_minimum_diff(vpar_make_path(4)), 1-2); \\ -1
check_equal(vpar_critical_ind_minimum_diff(vpar_make_path(5)), 1-2); \\ -1

test_critical_ind_minimum_diff_by_sets(vpar) =
{
  \\ print("test_critical_ind_minimum_diff_by_sets() "vpar);
  my(sets=vpar_indsets(vpar));
  sets=select(set->#set, sets);    \\ non-empties
  \\ print("sets  "sets);

  my(diffs=apply(set->#set - #vpar_neighbours_of_set(vpar,set), sets));
  \\ print("diffs "diffs); if(#diffs, my(pos);vecmin(diffs,&pos);print(" smallest set "sets[pos]));

  if(#diffs, vecmin(diffs));
}
{
  for(forest=0,0,
     for(n=1,6-forest,
        forvec(vec=vpar_forvec_from(n,forest), \\ trees or forests
              my(vpar=vpar_INTERNAL_from_upascend(vec));
              check_equal(vpar_critical_ind_minimum_diff(vpar),
                          test_critical_ind_minimum_diff_by_sets(vpar),
                          "vpar_critical_ind_minimum_diff() vs test_critical_ind_minimum_diff_by_sets()");
     )));
}
{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     if(#vpar <= test_limit_sets,
        print(vpar);
        check_equal(vpar_critical_ind_minimum_diff(vpar),
                    test_critical_ind_minimum_diff_by_sets(vpar),
          "vpar_critical_ind_minimum_diff() vs test_critical_ind_minimum_diff_by_sets()")));
}


\\-----------------------------------------------------------------------------
\\ scratching around for claw count trees total -> forests total

  \\ subst
  check_equal((4*gB*gNumT')' - (4*gB*gNumT')*gNumT',
              (x*gB')'*(gNumT-1)/x);
  check_equal((4*gB*gNumT')'
              - (4*gB*gNumT')*gNumT'
              - (x*gB')'*(gNumT-1)/x,       0);
  check_equal(  4*gB' * gNumT'
              + 4*gB  * gNumT''
              - 4*gB  * gNumT' * gNumT'
              - (x*gB')'*(gNumT-1)/x,       0);
  check_equal(  4*gB' * gNumT'
              + 4*gB  * gNumT''
              - 4*gB  * gNumT' * gNumT'
              - x'*gB'*(gNumT-1)/x
              - x*gB''*(gNumT-1)/x,       0);
  check_equal(  4*gB' * gNumT'
              + 4*gB  * gNumT''
              - 4*gB  * gNumT' * gNumT'
              -   gB' * (gNumT-1)/x
              - x*gB''* (gNumT-1)/x,       0);

  check_equal(  4*gB' * gNumT'
              + 4*gB  * gNumT''
              - 4*gB  * gNumT' * gNumT'
              -   gB' /x * (gNumT-1)
              - x*gB''/x * (gNumT-1),       0);

  check_equal(  4*gB' * gNumT'
              + 4*gB  * gNumT''
              - 4*gB  * gNumT' * gNumT'
              -   gB' /x * gNumT
              - x*gB''/x * gNumT
              +   gB' /x
              + x*gB''/x,       0);

  check_equal(gF - intformal(gF*gNumT'),  intformal(gT'*gNumF));
  check_equal(intformal(gF' - gF*gNumT'),  intformal(gT'*gNumF));

  check_equal(gNumT', x*gNumF' + gNumF);

  check_equal(intformal(gF' - gF*(x*gNumF' + gNumF)),
              intformal(gT'*gNumF));

  check_equal(vector(limit,n,n--; vpar_num_claws_total(n,1)),
              vector(limit,n,n--;
                 sum(t=1,n, binomial(n-1,t-1) * (
                          vpar_num_claws_total(t) * vpar_count(n-t,1)
                        + vpar_num_claws_total(n-t,1) * vpar_count(t)))));

  check_equal(vector(limit,n,n--; vpar_num_claws_total(n,1)),
              vector(limit,n,n--;
                 sum(t=0,n-1, binomial(n-1,t) * (
                          vpar_num_claws_total(t+1) * vpar_count(n-t-1,1)
                        + vpar_num_claws_total(n-t-1,1) * vpar_count(t+1)))));

  check_equal(vector(limit,n,n--; vpar_num_claws_total(n+1,1)),
              vector(limit,n,n--;
                 sum(t=0,n, binomial(n,t) * (
                          vpar_num_claws_total(t+1) * vpar_count(n-t,1)
                        + vpar_num_claws_total(n-t,1) * vpar_count(t+1)))));
  check_equal(gT', test_func_to_egf(t->vpar_num_claws_total(t+1), limit));
  check_equal(gF',   gT' * gNumF
                   + gF  * gNumT'  );

  check_equal(intformal(gF'), intformal(   gT' * gNumF
                                         + gF  * gNumT' ) );
  check_equal(gF, intformal( gT' * gNumF)
                + intformal( gF  * gNumT' ) );
  check_equal(gF - intformal(gF*gNumT'), intformal(gT'*gNumF));
  check_equal(intformal(gF' - gF*gNumT'), intformal(gT'*gNumF));

  check_equal(gF' - gF*gNumT', gT'*gNumF);

  check_equal(intformal(gF'), intformal(   gT' * gNumB'
                                         + gF  * gNumT' ) );

  print(test_egf_terms(gF,10));

  print(test_egf_terms(gF', 10));
  print(test_egf_terms(  gT' * gNumF
                   + gF  * gNumT' , 10));
  print(vector(10,n,n--; F(n)));

  \\ check_equal(gF, gT*gNumT + O(x^10));
  print(test_egf_terms(gT*gNumT, 10));
  print(gF);

  print(gF/gT);
  print(test_egf_terms(gF/gT, 10));
  print(test_egf_terms(gNumT', 10));
  print(test_egf_terms(gF/gT/gNumT',10));
  print(test_egf_terms(4*gB*gNumT', 10));
  \\ 4*A000169 num trees

  \\ my(n=20,t=7); binomial(n,t)
  \\ my(n=20,t=7); binomial(n-1,t-1) * n/t

  \\ my(n=20,t=7); binomial(n,t) * binomial(t-1,3)
  \\ my(n=20,t=7); binomial(n,t) * (t-1)*(t-2)*(t-3)/6
  \\

\\-----------------------------------------------------------------------------
\\ vpar_is_equimatchable() smaller comments

\\ state[v] is the current state of it and subtrees under it.  A vertex v
\\ accumulates into state[p] changing that state to another.  Various
\\ combinations go to non-equimatchable for an early exit, whereas some
\\ child types remain acceptable.
\\
\\ The states arise first from a path length 1 to 5.  Vertices with various
\\ children are equivalent to those for equimatchable purposes.
\\
\\ State 4, which is path-4, is not equimatchable.  But if it gets a
\\ singleton, or singleton equivalent, under then it goes to state 2 which
\\ is equimatchable.  The simplest case is when path-4 gets singleton to
\\ become path-5, rooted at v=4, and path-5 is equimatchable.  With that
\\ root position it's state=2, not state=5.
\\
\\ All vertices start as state=1 singletons.  Those which are parents have
\\ subtrees added under them.
\\
\\              _                               1 = singleton [0]
\\             v |                              2 = path-2
\\        ----> 2                               3 = path-3
\\       /      ^                               4 = path-4
\\      /  /--> 4 loop or to 2 or to none       5 = path-5
\\     1 --
\\      \  \--> 3 loop path-2, or none
\\       \
\\        ----> 5  --> non
\\                 any


  my(others=List([]));
        listput(others,rel)));
  \\ print("others before undup "#others);
  others=Set(others);

  \\ for(i=1,#others, print("  "others[i]));

  check_equal(#others,0, "no others");

  others=setunion(others, [ [-2, -2, 0, unmatched] ]);
  others=setunion(others, [ [-3, -3, 0, unmatched] ]);
  others=setunion(others, [ [-4, -4, 0, unmatched] ]);

  others=setunion(others, [ [-2, -3, 0, unmatched] ]);
  others=setunion(others, [ [-3, -4, 0, unmatched] ]);
  others=setunion(others, [ [-4, -5, 0, unmatched] ]);

  my(any=concat(test_equimatchable_rel_list,others));
  for(p=1,#any,
     for(v=1,#any,
        \\ combinations of five with other, don't look again at combinations
        \\ five and five
        if(p<=#test_equimatchable_rel_list &&
           v<=#test_equimatchable_rel_list, next());

        my(rel=test_equimatchable_rel_add(any[p], any[v]));
        \\ print(rel);
        check_equal(test_equimatchable_rel_find(rel)
                    || setsearch(others,rel)
                    || test_equimatchable_rel_is_non(rel)
                    || rel == [-5, -5, 0, 'unmatched]
                    || rel == [-5, -6, 0, 'unmatched]
                    , 1,
                    "equimatchable others combinations");
     ));


\\-----------------------------------------------------------------------------
\\ Ordered Tree Depths Iteration
\\
\\ Think iterating the vpar's is more useful.  The work to iterate either is
\\ about the same, but with everything else acting on trees as vpar it
\\ should be better to give those.
\\
\\ Can depths=vpar_depths_vector(vpar) if really want the underlying depths.


\\     depths = vpar_preorder_depths_first(n)        iterate pre-order trees
\\     depths = vpar_preorder_depths_next(depths)    from their depths

\\ T. Beyer and S.M. Hedetniemi, "Constant Time Generation of Rooted Trees",
\\ SIAM Journal of Computing, volume 9, 1980, pages 706-712
\\
\\    unchanged  |
\\           q    p                 s[p]=s[q]
\\               s[i]               upwards is rolling copies q..p-1
\\  s[i- (p-q)]
\\ GP-Test  subst('i-('p-'q),'i,'p) == 'q

\\ initial [0,1,2,3,4]
\\ final   [0,1,1,1,1]
\\
vpar_preorder_depths_first(n) = vector(n,v,v-1);
{
addhelp(vpar_preorder_depths_first,
"depths = vpar_preorder_depths_first(n)
Return the first pre-order depths vector of n vertices.

This first vector is depths [0,1,2,...].  The vpar of these depths can be
made by vpar_make_preorder_depths().  It is a path-n with root=1 and others
successively below.

Further vectors are iterated with vpar_preorder_depths_next().");
}

vpar_preorder_depths_next(depths) =
{
  my(p=0,q=0);
  forstep(v=#depths,1,-1, if(depths[v]!=1, p=v;break()));
  forstep(v=p-1,1,-1, if(depths[v]<depths[p], q=v;break()));
  if(!(p&&q), return('none));
  \\ print(depths" p="p" q="q);
  until(q++; p++>#depths, depths[p]=depths[q]); \\ pre-increment p
  depths;
}
{
addhelp(vpar_preorder_depths_next,
"depths = vpar_preorder_depths_next(depths)
Return the next pre-order depths vector after the depths given.
If there is no next depths then return symbol 'none.
This iteration is per

    Beyer and Hedetniemi, \"Constant Time Generation of Rooted Trees\",
    SIAM Journal of Computing, volume 9, 1980, pages 706-712.

The firest depths is vpar_preorder_depths_first().  The vpar for each is
made by vpar_make_preorder_depths().  So a loop like

    n=6;  \\\\ vertices
    my(depths=vpar_preorder_depths_first(n));
    until((depths=vpar_preorder_depths_next(depths))=='none,
          my(vpar=vpar_make_preorder_depths(depths));
          \\\\ vpar pre-order
         );

This iterates what is effectively all rooted but unlabelled trees in the
sense that there is a single specific labelling.  The trees iterated here
are all representatives of all the distinct equivalence classes from
vpar_is_isomorphic().  See vpar_count_unlabelled() for the number of such
classes.

The next depths vector here is built by locating the last depths[p]!=1, and
preceding p the last smaller depths[q]<depths[p] with q<p.  Depths at and
after p are replaced by copies of the run from q inclusive to p exclusive.
The effect is to change any trailing run of leaf children of the root, and
the vertex before.");
}

\\ vpar_preorder_depths_first()
\\ vpar_preorder_depths_next()

check_equal(vpar_preorder_depths_first(5), [0,1,2,3,4]);
check_equal(vpar_preorder_depths_next([0,1,2,3,4]), [0,1,2,3,3]);
check_equal(vpar_preorder_depths_next([0,1,1,1,1]), 'none, \
            "vpar_preorder_depths_next() none");

test_decr(v) = apply(x->x-1,v);
test_incr(v) = apply(x->x+1,v);

\\ examples from Beyer and Hedetniemi page 708
check_equal(test_incr(vpar_preorder_depths_next(test_decr([1,2,3,2,2,2]))), \
            [1,2,2,2,2,2]);
check_equal(test_incr(vpar_preorder_depths_next(test_decr([1,2,3,4,2,2]))), \
            [1,2,3,3,3,3]);
\\
\\ typo in the paper, s(L(T)) should be one value shorter
check_equal(test_incr(vpar_preorder_depths_next \
                      (test_decr([1,2,3,4,3, 2,2,2, 2,2,2]))), \
            [1,2,3, 4,2,3, 4,2,3, 4,2]);
\\
check_equal(test_incr(vpar_preorder_depths_next \
                      (test_decr([1,2,3,4, 5,5, 2,2,2,2]))), \
            [1,2,3,4, 5,4, 5,4, 5,4]);



\\-----------------------------------------------------------------------------

vpar_num_bigger_siblings_of_vertex(vpar,v) =
{
  my(p=vpar[v]);
  sum(i=v,#vpar, vpar[i]==p);
}
{
addhelp(vpar_num_bigger_siblings_of_vertex,
"num = vpar_num_bigger_siblings_of_vertex(vpar,v)
Return the number of bigger siblings of vertex v.
Root vertices are reckoned as siblings of each other.

This is a reverse position of v among its siblings, so where last sibling is
numbered 0, second last 1, etc.");
}

vpar_num_bigger_siblings_vector(vpar) =
{
  my(upto=vectorsmall(#vpar+1,v,-1), \\ ready for pre-increment to 0 first
     ret=vector(#vpar));
  forstep(v=1,#vpar,-1, ret[v]=upto[vpar[v]+1]++);
  ret;
}
{
addhelp(vpar_num_bigger_siblings_vector,
"vec = vpar_num_bigger_siblings_vector(vpar)
Return a vector where vec[v] is the number of bigger siblings of vertex v.
Root vertices are reckoned as siblings of each other.
This is vec[v] = vpar_num_bigger_siblings_of_vertex(vpar,v) but calculated
all together.");
}


\\-----------------------------------------------------------------------------
\\ vpar_maximal_matchings_count() done just in integers for count,
\\ but can share the maximal matchpoly code.

vpar_maximal_matchings_count(vpar) =
{
  \\ with[v] = how many maximal matchings with v matched

  \\ without[v] = how many matchings where v not matched and the matchings
  \\     might or might not be maximal.  The non-maximals are only
  \\     non-maximal in respect of v.  If v is matched to a parent above it
  \\     then all of without[v] are maximal.
  \\ without_max[v] = how many matchings where v not matched and the
  \\     matchings are maximal.

  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     with           = vector(#vpar),
     without        = vector(#vpar,v,1),
     without_max    = without,
     ret = 1);
  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v],
        v_max = with[v] + without_max[v]);
     \\ print(" v="v" with "with[v]" without "without[v]" max "without_max[v]);

     if(p,
        with[p] = with[p] * v_max
                 + without[p] * without[v];  \\ new pair
        without[p] *= v_max;
        without_max[p] *= with[v];

        \\ print("  set p="p" with "with[p]" without "without[p]" max "without_max[p]);
        ,
        ret *= v_max);    \\ at root

     with[v]        =  \\ kill intermediate bignum values
     without[v]     =
     without_max[v] = 0);

  ret;
}


\\-----------------------------------------------------------------------------
\\ Sorted is vpar_automorphism_ids_vector().
\\ Is unsorted, by vertex number, anything useful?

\\     vec = vpar_lexicographic_ids_vector(vpar,sort=1)

vpar_lexicographic_ids_vector(vpar,sort=1) =
{
  my(n = #vpar,
     num_children = vpar_INTERNAL_num_children_vecsmall(vpar),
     id          = vector(n,i,1),
     seq         = vectorsmall(n),
     sub         = seq,   \\ sub[id] = id of last descent
     cumul       = seq,   \\ cumul[id] = id for descent by sub[id]
     idtree_upto = 1,     \\ id=1 for leaf
     phase       = 1,
     upto        = 0,
     idv         = seq,
     chain       = seq);

  for(v=1,n,
     if(num_children[v]==0,   \\ leaf vertices
        num_children[v]=-1;
        seq[upto++]=v));      \\ pre-increment
  \\ print("initial upto ",upto);

  while(upto<n,
        \\ print("phase="phase" to upto="upto);
        \\ print(" seq ",Vec(seq[phase..upto]));

        \\ next new id, so new ids added will be idtree_phase..idtree_upto
        my(idtree_phase=idtree_upto+1);

        my(new_upto = upto);
        for(t=phase,upto,
           my(v=seq[t], p=vpar[v]);
           if(p,
              \\ print(" t="t" i="i" v="v" vid="id[v]"  p="p" pid="id[p]);
              my(vid=id[v], pid=id[p]);
              id[p] = if(sub[pid]==vid,
                         cumul[pid];                  \\ old id
                         ,
                         sub[pid] = vid;              \\ descent
                         cumul[pid] = idtree_upto++); \\ pre-inc, new id

              if(num_children[p]--==0,  \\ pre-decrement
                 seq[new_upto++] = p)));

        phase = upto+1;
        upto=new_upto;

        \\ Bucket sort by id for sorted, by vertex number for unsorted.
        \\ print(" parents phase="phase" to upto="upto);
        \\ print(" seq1 ",Vec(seq[phase..upto]));
        for(t=phase,upto,
           my(v=seq[t], vid=if(sort,id[v],v));
           \\ print("  t="t" i="i" v="v" vid="vid);
           chain[v] = idv[vid]; idv[vid] = v);

        \\ seq in sorted order
        \\ print(" ids idtree_phase="idtree_phase" to idtree_upto="idtree_upto);
        new_upto = phase-1;  \\ ready for pre-increment
        for(t=idtree_phase,idtree_upto,
           my(v=idv[t]);
           while(v,
                 seq[new_upto++] = v;  \\ pre-increment
                 v = chain[v]));
       );
  id;
}
{
addhelp(vpar_lexicographic_ids_vector,
"vec = vpar_lexicographic_ids_vector(vpar,sort=1)
Return a vector where each vec[v] is an integer representing the
lexicographic class of the subtree at v.

Each vec[v] is an integer in the range 1 to at most #vpar, inclusive.
There may be gaps in the numbers used in the range.  The numbers cannot be
compared between different trees, not even between relabellings of the same
vpar.

The integers are ascending lexicographic class so an x < y means subtree
class x is lexicographically before subtree class y.  Lexicographic order
between two subtrees u and v means comparing child subtrees successively.
So compare first children u1,v1, and if equal then u2,v2, etc.  A shorter
but otherwise equal vector is reckoned smaller.  Children are taken by
ascending vertex number.

Optional parameter \"sort\" can be 1 to take children in lexicographic
order.  This ignores vertex numbers so gives subclass ids as for a rooted
but unlabelled tree.  The ids are isomorphism classes so that when
vec[u]==vec[v] it means the subtrees under u and v are isomorphic.

In all cases the effect of lexicographic order is to sort by subtree height,
then structure within equal heights.  Nothing in the id numbers shows those
heights, only that increasing id is increasing height.");
}

\\ vpar_lexicographic_ids_vector()

\\   2 <- 1 <- 3 <- 4
\\
check_equal(vpar_lexicographic_ids_vector([2,0,1,3],1), [3,4,2,1], \
            "vpar_lexicographic_ids_vector() path-4 reordered")

\\     1
\\    / \
\\   2   4     3,4 same
\\   |
\\   3
check_equal(vpar_lexicographic_ids_vector([0,1,2,1],1), [3,2,1,1])

{
  \\ id equality corresponds to isomorphic subtree

  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],

        ids=vpar_lexicographic_ids_vector(vpar,1),
        ids_m_eq=matrix(#vpar,#vpar,u,v, ids[u]==ids[v]),
        ids_m_cmp=matrix(#vpar,#vpar,u,v, lex(ids[u],ids[v])),

        \\ nests=vector(#vpar,v, test_subtree_nest_by_recurse(vpar,v,1)),
        \\ nests_m_cmp=matrix(#vpar,#vpar,u,v, lex(nests[u],nests[v])),

        subtrees = vector(#vpar,v, vpar_subtree(vpar,v)),
        subtrees_m_eq=matrix(#vpar,#vpar,u,v,
                             vpar_is_isomorphic(subtrees[u],subtrees[v])));

     check_equal(ids_m_eq, subtrees_m_eq, "ids_m vs subtrees_m");

     \\ if(ids_m_cmp != nests_m_cmp,
     \\    print(vpar);
     \\    vpar_view(vpar);
     \\   );
     \\ check_equal(ids_m_cmp, nests_m_cmp, "ids cmp");
     );
}


{
  my(n=4,
     sort=0);
  forvec(vec=vpar_forvec_from(n,0),  \\ trees
        my(vpar=vpar_from_upascend(vec));
        print(vpar);

        my(ids =vpar_lexicographic_ids_vector(vpar,sort));
        print(vpar"  "ids);
  );
}
quit


\\------------------------------------------------------------------------------

\\ WRONG  Must find a leaf as a starting point, or at root of degree d must
\\ allow for any one of the d to be its pairing
\\
vpar_has_perfect_matching_by_upwards(vpar) =
{
  print("vpar_has_perfect_matching "vpar);
  if(bittest(#vpar,0),return(0));
  my(seen = vectorsmall(#vpar));
  for(i=1,#vpar,
     if(!seen[i],
        my(v=i, phase=0);
        print(" v="v);
        while(1,
              print("  at v="v);
              if(!vpar[v] || seen[vpar[v]]==3,
                 if(v!=i,
                    seen[v]=1;
                    seen[i]=bitxor(phase,2);
                    print("  root "v" phase "phase" set "seen[v]" low "seen[i]));
                 break());
              seen[v]=3;
              phase=bitxor(phase,3);
              if(seen[vpar[v]],
                 print("  top "vpar[v]" phase "phase" seen "seen[vpar[v]]);
                 phase=bitxor(phase,seen[vpar[v]]);
                 seen[vpar[v]]=3;
                 seen[i]=phase;
                 print("  set low "seen[i]);
                 break());
              v=vpar[v]);

        \\ print("  low phase "phase);
        \\ v=i;
        \\ seen[v] = phase;
        \\ while((v=vpar[v]) && !seen[v], seen[v]=3);
        print("  seen "Vec(seen));
       ));
  my(phase=3);
  for(i=1,#vpar,
     (phase=bitand(phase,seen[i])) || return(0));
  1;
}

\\ WRONG  Cannot continue up from leaf, only where the pair reckoned removed
\\ makes it a further leaf.
\\
vpar_has_perfect_matching_by_upwards(vpar) =
{
  print("vpar_has_perfect_matching "vpar);
  if(bittest(#vpar,0),return(0));
  my(seen=vpar_INTERNAL_num_children_vecsmall(vpar));
  print(" num children "Vec(seen));
  for(i=1,#vpar,
     if(seen[i]==0,
        my(v=i, p);
        print(" v="i);
        while(v && (p=vpar[v]) && seen[v]>=0 && seen[p]>0,
              print("  pair v="v" p="p);
              seen[v]=-1;
              seen[p]=-1;
              v=vpar[p])));
  hammingweight(seen) == #vpar;
}

\\ Previous implementation using a sequence to go upwards.
vpar_has_perfect_matching(vpar) =
{
  if(bittest(#vpar,0),return(0));
  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     matched = vectorsmall(#vpar));
  for(i=1,#seq,
     my(v=seq[i], p);
     if(!matched[v],  \\ v unmatched, must match with parent p
        if(!(p=vpar[v]) || matched[p], return(0));
        matched[p]=1));
  1;
}

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i],
        got = vpar_matchnum_and_count(vpar));
     check_equal(vpar_has_perfect_matching_by_upwards(vpar),
                 vpar_has_perfect_matching(vpar));
  );
}



\\-----------------------------------------------------------------------------
\\     pos = vpar_childpos_of_vertex(vpar,v)
\\     vec = vpar_childpos_vector(vpar)
\\ counting from 1

vpar_childpos_of_vertex(vpar,v) =
{
  my(p=vpar[v]);
  sum(i=1,v, vpar[i]==p);
}
{
addhelp(vpar_childpos_of_vertex,
"pos = vpar_childpos_of_vertex(vpar,v)
Return the child position of vertex v among its siblings.
Siblings are numbered 1 for the smallest, 2 the next bigger, etc.
Root vertices are reckoned as siblings of each other.");
}

vpar_childpos_vector(vpar) =
{
  my(upto=vectorsmall(#vpar+1),
     ret=vector(#vpar));
  for(v=1,#vpar, ret[v]=upto[vpar[v]+1]++);
  ret;
}
{
addhelp(vpar_childpos_vector,
"vec = vpar_childpos_vector(vpar)
Return a vector where vec[v] is the child position of vertex v among its
siblings.  Siblings are numbered 1 for the smallest, 2 the next bigger, etc.
Root vertices are reckoned as siblings of each other.
This is vec[v] = vpar_childpos_of_vertex(vpar,v) but calculated all
together.");
}

\\ vpar_childpos_of_vertex()
\\ vpar_childpos_vector()

check_equal(vpar_childpos_vector(vpar_make_path(5)),[1,1,1,1,1]);
check_equal(vpar_childpos_vector(vpar_make_star(5)),[1,1,2,3,4]);
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_childpos_vector(vpar),
                 vector(#vpar,v, vpar_childpos_of_vertex(vpar,v))));
}

{
  for(forest=-1,1,
     for(n=1,3,
        my(total=0);
        forvec(vec=vpar_forvec_from(n,forest), \\ trees or forests
              my(vpar=vpar_INTERNAL_from_upascend(vec),
                 vec=vpar_childpos_vector(vpar));
              total += vecsum(vec));

        if(forest<1,
           check_equal(total, (3*(n-1)*n/2+1)/n * vpar_count(n,forest)),
           check_equal(total, 1/2*(3*n+1)*n/(n+1) * vpar_count(n,1)))));
}

\\ or number of smaller siblings is 0 upwards
\\ or num_other_siblings 0 upwards
\\    num_siblings       1 upwards
\\
\\ vpar_siblingpos_vector(vpar,down=0)  position 0 upwards
\\
\\ vpar_num_siblings_smaller_vector     position 0 upwards
\\ vpar_num_siblings_bigger_vector
\\ vpar_num_siblings_vector            excluding self so before+after
\\
\\ vpar_num_siblings_before_vector     position 0 upwards
\\ vpar_num_siblings_after_vector
\\ vpar_num_siblings_vector            excluding self so before+after
\\ vpar_num_siblings_before_of_vertex
\\ vpar_num_siblings_after_of_vertex
\\ vpar_num_siblings_of_vertex


\\-----------------------------------------------------------------------------
\\     bool = vpar_is_path(vpar,forest=0)
\\     count = vpar_count_paths(n,forest=0)
\\
\\ Don't rate specific test for paths, though the count of forests
\\ comprising only paths might be new.

\\ Or just vpar_is_isomorphic(vpar_make_path(n))
\\ Or vpar_is_free_isomorphic_path(vpar)
\\ FIXME: forest=-1 require v=1 root or just endpoint?  Or no such?
vpar_is_path(vpar,forest=0) =
{
  if(#vpar<=1, return(1));
  if(forest<1 && !vpar_is_tree(vpar), return(0));
  my(degrees=vectorsmall(#vpar,v, vpar[v]!=0)); \\ edge child -> parent
  for(v=1,#vpar,
     if(vpar[v],
        if(degrees[vpar[v]]++>2, return(0))));  \\ edge parent -> child

  (forest>=0 || degrees[1]==1);        \\ forest=-1 must v=1 endpoint
}
{
addhelp(vpar_is_path,
"bool = vpar_is_path(vpar,forest=0)
Return 1 if vpar is a path, or return 0 if not.
Optional parameter \"forest\" can be 1 to test for a forest of paths, or can
be -1 to test for a path of root=1.

Vertices can be in any order in the path and the root any one of them, So
the test is for free isomorphic to a path.  Likewise a forest of paths,
though the isomorphism would be to some collection of the same set of path
lengths.

The test is thus all vertex degrees <= 2, and for a tree a single component
so degrees >= 1 except empty vpar=[] and single vpar=[0] which are reckoned
paths in all cases.");
}

vpar_count_paths(n,forest=0) =
{
  if(n<=1, 1,
     forest==-1, (n-1)!,
     !forest,    (n! >> 1)*n,
     vpar_INTERNAL_exponential_transform(vector(n,i, if(i==1,1,(i!>>1)*i)))[n]);
}
{
addhelp(vpar_count_paths,
"count = vpar_count_paths(n,forest=0)
Return the number of vpar trees which are paths, per vpar_is_path(vpar,forest).

Optional parameter \"forest\" can be -1 for number of root=1 paths, or can
be 1 for number of forests with each component tree a path.

For trees this is permute labels, not also the reverse labelling, and
choose one of n root, so
    if(n<=1,1, n*n!/2) = 1,1,2,9,48,300,... (A074143)

For forests this is 1,1,3,16,109,891,... by usual exponential transform from
the trees count.");
}
\\ not in OEIS: 3, 16, 109, 891
\\ GP-Test  vector(6,n,n--; if(n<=1,1, n*n!/2)) == [1,1,2,9,48,300]
\\
\\ vector(20,n,n--; vpar_count_paths(n,1))
\\ vector(16,n,n++; vpar_count_paths(n)*(n-1)!)
\\ vpar_count_paths(2) == 2
\\ my(n=10); (vector(n,i, (i!>>1)*i))
\\ my(n=10); vpar_INTERNAL_exponential_transform(vector(n,i, (i!>>1)*i))
\\
\\ also permute within components ...
\\ vector(10,n,n--; sum(c=1,n, binomial(n,c)*c^(n-c)))


check_equal(vpar_is_path([]), 1, "vpar_is_path() empty");
check_equal(vpar_is_path([],1), 1);
check_equal(vpar_is_path([],-1), 1);

check_equal(vpar_is_path([0]), 1);
check_equal(vpar_is_path([0],1), 1);
check_equal(vpar_is_path([0],-1), 1);

check_equal(vpar_is_path([0,1]), 1);
check_equal(vpar_is_path([0,1],1), 1);
check_equal(vpar_is_path([0,1],-1), 1);
check_equal(vpar_is_path([2,0],-1), 1);

check_equal(vpar_is_path([0,0]), 0, "vpar_is_path() singletons");
check_equal(vpar_is_path([0,0],-1), 0);
check_equal(vpar_is_path([0,0],1), 1);

test_is_path_by_degrees(vpar,forest=0) =
{
  if(#vpar==0,return(1));
  if(forest<=0, vpar_is_tree(vpar) || return(0));
  my(degrees=vpar_degrees_vector(vpar));
  if(forest==-1, degrees[1]<=1 || return(0));
  vecmax(degrees) <= 2;
}
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     for(forest=-1,1,
        \\ print(vpar" forest="forest);
        check_equal(vpar_is_path(vpar,forest),
                    test_is_path_by_degrees(vpar,forest),
                    Str("vpar_is_path() vs test_is_path_by_degrees()"
                        " forest="forest));
        if(forest==0,
           check_equal(vpar_is_path(vpar,forest),
                       vpar_is_free_isomorphic(vpar,vpar_make_path(#vpar))));
        \\ if(forest==-1 && #vpar>=1,
        \\    check_equal(vpar_is_path(vpar,forest),
        \\                vpar_roots(vpar)==[1] &&
        \\                vpar_is_free_isomorphic(vpar,vpar_make_path(#vpar))));
     ));
}


\\ vpar_count_paths()

{
  \\ A074143 a(1) = 1; a(n) = n * sum {a(k) | k < n}.
  check_equal(vector(#A074143_samples,n, vpar_count_paths(n)),
              A074143_samples);
}
\\ note, not A198892 which begins 1, 1, 2, 9, 48, 300, 2280, ...

{
  my(limit = 10,
     L = O(x^limit),
     gT = test_func_to_egf(vpar_count_paths, limit),
     gF = test_func_to_egf(n->vpar_count_paths(n,1), limit));
  check_equal(exp(gT-1), gF);
  \\ print("gT      = "gT);
  \\ print("exp(gT) = "exp(gT-1));
  \\ print("gF      = "gF);

  my(gFactorials = test_func_to_egf(n->n!, limit),
     gFactorials_times_n = test_func_to_egf(n->n*n!, limit),
     gFactorials_times_n_over_2 = test_func_to_egf(n->n*n!/2, limit));
  check_equal(gFactorials, 1/(1-x) + L);
  check_equal(gFactorials', 1/(1-x)^2 + L);
  check_equal(gFactorials_times_n, x*gFactorials');
  check_equal(gFactorials_times_n, x/(1-x)^2 + L);
  check_equal(gFactorials_times_n_over_2, gFactorials_times_n/2);
  check_equal(gT, gFactorials_times_n_over_2 + 1 + 1/2*x);
  check_equal(gT, 1/2*x/(1-x)^2 + 1 + 1/2*x + L);
  check_equal(gF, exp(gFactorials_times_n_over_2 + 1/2*x));
  check_equal(gF, exp(x*gFactorials'/2 + 1/2*x));
  check_equal(gF, exp(x*gFactorials'/2) * exp(1/2*x+L));
  check_equal(gF, sqrt( exp(x*gFactorials') * exp(x+L) ));
  check_equal(gF, exp(gT-1-1/2*x) * exp(1/2*x+L));
  check_equal(gF, exp(1/2*x*(1 + 1/(1-x)^2) + L));
  check_equal(gF, exp(1/2*x*(1/(1-x)^2) + L) * exp(1/2*x*(1) + L));
  check_equal(gF, exp(1/2*x/(1-x)^2 + L) * exp(1/2*x + L));
  check_equal(1/2*x*(1 + 1/(1-x)^2),  x/2*(x^2 - 2*x + 2)/(1-x)^2);

  my(gx = test_func_to_egf(n->1/2^n, limit));
  check_equal(gx, exp(1/2*x + L));
}

\\ Wrong:
\\ test_count_paths_forest_by_binomial(n) =
\\ {
\\   \\ sum(k=1,n, 1/2^(n-k) * binomial(n,k) * (if(n==1,1,n*n!/2) -if(n==1,1/2)));
\\   sum(k=1,n, 1/2^(n-k) * binomial(n,k) * (if(k==1,1/2, k*k!/2)));
\\   sum(k=1,n, (1)^(n-k) * binomial(n,k) * (vpar_count(k) -if(k==1,1)));
\\ }
\\ vector(10,n, test_count_paths_forest_by_binomial(n))
\\ vector(10,n, vpar_count_paths(n,1))

{
  for(forest=-1,1,
     for(n=0,5,
        my(count=0);
        forvec(vec=vpar_forvec_from(n,forest),
              my(vpar=vpar_INTERNAL_from_upascend(vec));
              count += vpar_is_path(vpar,forest));
        check_equal(vpar_count_paths(n,forest), count,
                    Str("vpar_count_paths() vs count vpar_is_path()"
                        " n="n" forest="forest))));
}



\\-----------------------------------------------------------------------------

  \\ GP-Test  (6-1)*6 == 30
  \\ GP-Test  (3-1)*6 == 12
  \\ GP-Test  2^36-1 == 68719476735

  \\ 1,2,3 -> 1,4,8
  \\ 1,2,3 -> 1,3,6
  \\ GP-Test  my(len=1); len+bitnegimply(len>>1,len))  01 -> 00
  \\ 11 -> 01
  \\ 10 -> 10


\\-----------------------------------------------------------------------------
\\ vpar_automorphism_ids_vector() with its own code

vpar_automorphism_ids_vector(vpar) =
{
  my(num_children = vpar_INTERNAL_num_children_vecsmall(vpar),
     id    = vector(#vpar,i,1),
     seq   = vectorsmall(#vpar),
     sub   = seq,       \\ sub[id] = id of last descent
     cumul = seq,       \\ cumul[id] = id for descent by sub[id]
     idtree_upto = 1,   \\ id=1 for leaf
     phase = 1,
     upto  = 0,
     idv   = seq,
     chain = seq);

  \\ seq[1..upto] = childless vertices
  for(v=1,#vpar,
     if(num_children[v]==0,
        num_children[v]=-1;
        seq[upto++]=v));       \\ pre-increment

  while(phase<=upto,
        \\ print("phase="phase" to upto="upto);
        \\ print(" seq ",seq[phase..upto]);

        \\ next new idtree_upto, so new ids added are idtree_phase..idtree_upto
        my(idtree_phase=idtree_upto+1);

        my(new_upto=upto);
        for(t=phase,upto,
           my(v=seq[t], p=vpar[v]);
            \\ print(" t="t" i="i" v="v" vid="id[v]"  p="p, if(p,Str(" pid="id[p]),""));
           if(p,
              my(v_id=id[v], p_id=id[p]);
              if(sub[p_id]==v_id,
                 p_id = cumul[p_id];          \\ after p_id accumulates v_id
                 ,
                 sub[p_id] = v_id;
                 cumul[p_id] = idtree_upto++; \\ pre-increment, new id
                 \\ print("    new id "idtree_upto);
                 p_id = idtree_upto);
              id[p] = p_id;
              \\ print("    set parent "p" id "p_id);

              if(num_children[p]--==0,
                 seq[new_upto++] = p)));

        phase = upto+1;
        upto = new_upto;

        \\ bucket sort the parents by ascending id
        \\ print(" parents phase="phase" to upto="upto);
        \\ print(" seq ",seq[phase..upto]);
        for(t=phase,upto,
           my(v=seq[t], v_id=id[v]);
           chain[v] = idv[v_id]; idv[v_id] = v);

        \\ print(" ids idtree_phase="idtree_phase" to idtree_upto="idtree_upto);
        new_upto=phase-1;              \\ ready for pre-increment
        for(t=idtree_phase,idtree_upto,
           my(count=0,v=idv[t]);
           while(v,
                 seq[new_upto++] = v;  \\ pre-increment
                 v = chain[v];
                 count++)));
  id;
}

\\-----------------------------------------------------------------------------
\\ vpar_seq_uplex()
\\
\\ This form sorting within ids.
\\ And want to go by subtree classes not big ids made by lists.

vpar_seq_uplex(vpar) =
{
  my(num_children=vpar_INTERNAL_num_children_vecsmall(vpar),
     seq=vectorsmall(#vpar),
     id=vector(#vpar),
     upto=0,
     upto_id=0,
     phase=1);

  for(v=1,#vpar,
     if(num_children[v]==0,  \\ childless vertices
        num_children[v]=-1;
        seq[upto++]=v));     \\ pre-increment

  while(upto<#seq,
        my(l=List([]));
        for(i=phase,upto,
           my(v=seq[i], p=vpar[v]);
           if(p && num_children[p]--==0,
              listput(l,p);
              id[p] = id[p]*#vpar + id[v]));
        phase = upto+1;
        l=vecsort(l,(x,y)->lex(id[x],id[y]));
        for(i=1,#l,
           seq[upto++]=l[i]));
  seq;
}


\\------------------------------------------------------------------------------
\\ Vertex Stable Equimatchable
\\ BADLY WRONG - though gives a subset of stable equimatchable
\\
\\                      start=1
\\                  v=2      v=1       v=1      <-+ loop
\\            3  <------- 1 ------> 2 ------> 4 --+ v=1,5
\\                        |
\\                        | v=4
\\                        |
\\                        | <-+ loop
\\                        5 --+ v=4
\\
\\ 4*4 == 16 entries, values 0 to 3 inclusive
vpar_INTERNAL_is_vertex_stable_equimatchable_table = {Vecsmall([
      2,3,0,5,0,  \\ p=1 row
      4,0,0,0,0,  \\ p=2
      0,0,0,0,0,  \\ p=3
      4,0,0,0,4,  \\ p=4
      0,0,0,5,0   \\ p=5
\\    ---------
\\ v= 1 2 3 4 5
])};

vpar_is_vertex_stable_equimatchable(vpar) =
{
  print("vpar_is_vertex_stable_equimatchable() "vpar);

  my(seq=vpar_INTERNAL_seq_upwards(vpar),
     state = vectorsmall(#vpar,v,1));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     print(" v="v" state "state[v]" p="p,if(p,Str(" state "state[p]," to "vpar_INTERNAL_is_vertex_stable_equimatchable_table[5*state[p] + state[v] - 5]),""));
     if(p,
        (state[p] = vpar_INTERNAL_is_vertex_stable_equimatchable_table
                      [5*state[p] + state[v] - 5])
        || return(0)));
  1;
}


\\-----------------------------------------------------------------------------
\\ Automorphism Fragments

\\ premax in id tree (WRONG)

\\   31 =   vpar_seq_premax      7+24

              if(action==31,
                 \\ insert subtree v at the start of the chain of p, after p itself
                 \\ print("    p="p" chain "subtree_chain[p+1]" tail "subtree_tail[p+1]" = "subtree_chain[subtree_tail[p+1]+1]);
                 \\ print("    v="v" chain "subtree_chain[v+1]" tail "subtree_tail[v+1]);
                 if(subtree_tail[p+1]==p,
                    subtree_tail[p+1] = subtree_tail[v+1],  \\ p was alone
                    subtree_chain[subtree_tail[v+1]+1] = subtree_chain[p+1]);
                 subtree_chain[p+1] = v;
                 \\ print("    insert chain "Vec(subtree_chain));
                 \\ print("            tail "Vec(subtree_tail));
                 \\ print("          p chain "subtree_chain[p+1]" tail "subtree_tail[p+1]" = "subtree_chain[subtree_tail[p+1]+1]);
                 ,

  if(action==31 && #vpar,
     my(v=0); upto=0;
     while(v=subtree_chain[v+1], ret[upto++]=v));
  \\ print("  ret="ret);


\\-----------------------------------------------------------------------------

\\ WRONG
test_has_perfect_matching_by_leaf_distances(vpar) =
{
  my(degrees=vpar_degrees_vector(vpar));
  for(v=1,#degrees, if(degrees[v]==0,return(0)));

  my(leaves=select(d->d<=1, degrees, 1),
     path_lengths=vpar_path_lengths_matrix(vpar));
  for(i=1,#leaves,
     my(u=leaves[i]);
     for(j=i+1,#leaves,
        my(v=leaves[j]);
        if(path_lengths[u,v] < 0, next());
        if(path_lengths[u,v]%2==0,
           print("u="u" to v="v" length "path_lengths[u,v]);
           return(0))));
  1;
}


\\-----------------------------------------------------------------------------

\\ No, not leaves in parallel, one at a time.
\\
\\ indnum_by_leaves(vpar) =
\\ {
\\   my(indnum=0);
\\   while(#vpar,
\\         my(leaves=vpar_leaves(vpar),
\\            neighbours=vpar_neighbours_of_set(vpar,leaves));
\\         indnum += #leaves;
\\         vpar = vpar_delete_vertices(vpar, concat(leaves,neighbours)));
\\   indnum;
\\ }
\\ {
\\   for(i=1,#test_sample_forests,
\\      my(vpar=test_sample_forests[i]);
\\      check_equal(vpar_indnum(vpar),
\\                  indnum_by_leaves(vpar),
\\                  "vpar_indnum() vs indnum_by_leaves()");
\\   );
\\ }


\\-----------------------------------------------------------------------------
\\ Weigh Transform

\\ a parameter to vpar_INTERNAL_Euler_transform()

\\ vpar_INTERNAL_weigh_transform(a) takes vector a[] and returns a
\\ corresponding vector b[] which is the usual "weigh" transform of a[].
\\
\\ This transform is a[n] as number of free trees of n vertices, becoming
\\ b[n] number of free forests of n vertices made by choosing from the a[]
\\ trees and no duplicate trees in a forest.
\\
vpar_INTERNAL_weigh_transform(a) =
{
  my(b=vector(#a,i,1),
     c=vector(#a,n, -sumdiv(n,d, (-1)^(n/d)*d*a[d])));
  for(n=1,#a,
     b[n] = (c[n] + sum(k=1,n-1, c[k]*b[n-k])) / n);
  b;
}


\\-----------------------------------------------------------------------------
\\ vpar_to_unlabelled() with a sort func
\\
\\ Does the function closure copy the arrays it uses?
\\ Building vectors for comparing ought to be about equal speed anyway.

vpar_INTERNAL_premax(vpar,flags) =
{
  \\ print("vpar_INTERNAL_premax() n="#vpar" flags="flags"\n vpar="vpar);

  #vpar || return(if(flags, Vecsmall([]), []));

  \\ my(depths = vpar_depths_vector(vpar),
  \\    h = vecmax(depths),
  \\    row_head  = vectorsmall(#vpar),
  \\    row_chain = row_head);
  \\ for(v=1,#vpar,
  \\    my(d=depths[v]+1);
  \\    row_chain[d] = row_head[d];
  \\    row_head[d] = v);

  my(children_chain = vectorsmall(#vpar),
     subtree_chain  = vectorsmall(#vpar+1),
     subtree_tail   = vectorsmall(#vpar+1,i,i-1),
     id             = vector(#vpar,i,1));

  my(rows=vpar_row_vertices_vector(vpar),
     row=rows[#rows]);
  forstep(r=#rows,1,-1,
         \\ print(" row r="r" "row);
         \\ print("  ids "vecextract(id,row));

         \\ children_chain pushed in ascending order, so compare high to low
         for(i=1,#row,
            my(v=row[i], p=vpar[v]);
            \\ print("   chain push v="v" to p="p);
            if(p,
               children_chain[v]=children_chain[p];
               children_chain[p]=v);
            if(flags,
               if(subtree_tail[p+1]==p,
                  subtree_chain[subtree_tail[p+1]+1] = v;  \\ append
                  subtree_tail[p+1] = subtree_tail[v+1];
                  ,
                  subtree_chain[subtree_tail[v+1]+1] = subtree_chain[p+1];  \\ insert
                  subtree_chain[p+1] = v;
                 ));

            \\ subtree_chain[subtree_tail[p+1]+1] = v;  \\ append
            \\ subtree_tail[p+1] = subtree_tail[v+1];
         );

         if(r==1,break());

         row=rows[r-1];
         \\ print("  parents row "row);

         \\ print("  children_chain"); for(i=1,#row,my(v=row[i]); print("   p="v); print1("   "); if(!children_chain[v],print("  empty"), while(v=children_chain[v], print1("  v="v",id="id[v])); print()));

         my(func=(x,y)->
            while(!( !(x=children_chain[x]) + !(y=children_chain[y]) ),
                  my(c=lex(id[x],id[y]));
                  if(c, return(c)));
            lex(x,y));
         row = vecsort(row,func);
         \\ print("  next row sorted ascending "row);

         \\ ids for parents, same where equal
         my(upto=1);
         id[row[1]] = upto;
         \\ print("  set p="row[1]" id "id[row[1]]);
         for(i=2,#row,
            id[row[i]] = (upto += func(row[i],row[i-1]));
            \\ print("      p="row[i]" id "id[row[i]]);
         );
  );

  if(flags,
     \\ print("  subtree_chain "Vec(subtree_chain));
     \\ print("  starting v=0 chain="subtree_chain[1]);
     my(v=0,
        seq=vectorsmall(#vpar),
        upto=0);
     while(v=subtree_chain[v+1], seq[upto++]=v);
     \\ print(" final seq "Vec(seq));
     seq;
     ,
     id);
}



\\-----------------------------------------------------------------------------
\\ vpar_to_free() explicit swap of bicentral to bigger second half

  my(n=#vpar);
  vpar=vpar_to_unlabelled(vpar_INTERNAL_for_free_automorphism(vpar));
  \\ print(" relabelled "vpar" len "#vpar);
  if(#vpar==n,
     vpar,
     my(ret=vector(n),
        to=0,
        prev_r=0,
        prev_v=0,
        v=1);
     \\ print(" ret start  "ret);
     while(1,
        my(p);
        \\ print(" v="v" prev_r="prev_r"  prev_v = "prev_v);
        if(v>#vpar || (p=vpar[v])==0,
           if(prev_r,
              \\ print("  copy unicentral root="to+1);
              to++;  \\ root
              my(offset=prev_v-to);
              for(i=prev_v+1, v-1,
                 \\ print("   copy i="i" ret["to+1"] = "vpar[i]"-"offset);
                 ret[to++] = vpar[i]-offset);
           );
           if(v>#vpar,
              \\ print("  stop");
              break());

           \\ print("  now prev_r="v" prev_v="v+1);
           prev_r = v;
           prev_v = v++; \\ pre-increment
           v++;
           ,
           p==prev_r,
           \\ print("  swap p="p" prev_r="prev_r"  prev_v="prev_v);
           my(offset=v-to-2, b=v);

           my(r=to++);
           \\ print("   root r="r);
           ret[to++]=r;
           \\ print("   first top ret["to"] = "r" (would be v="v")");
           while(v++<=#vpar && vpar[v],
                 \\ print("   first ret["to+1"] = "vpar[v]"-"offset"  from v="v);
                 ret[to++]=vpar[v]-offset);

           offset = to - prev_v;
           \\ print("   second top prev_v="prev_v" r="r"  offset="offset);
           for(i=prev_v+1, b-1,
              my(p=vpar[i]);
              ret[to++] = if(p==prev_v, r, p+offset);
              \\ print("   second i="i" p="p"  ret["to"] = "ret[to]);
           );
           prev_r=0;
           \\ print("   now v="v);
           ,
           v++;
          );
     );

     \\ print(" ret "ret);
     ret);


\\-----------------------------------------------------------------------------
\\ vpar_relabel_uplexqueue()

\\ vpar = vpar_relabel_uplexqueue(vpar)      doesn't seem useful

\\ NOT GOOD
vpar_relabel_uplexqueue(vpar) = vpar_relabel_seq(vpar,vpar_seq_uplexqueue(vpar));

\\ distinct
\\ 1,2,5,15,52,203
\\ A000110 Bell

{
  my(forest=0);
  for(n=1,10,
     my(m=Map(),
        count=0);
     forvec(vec=vpar_forvec_from(n,forest),
           my(vpar=vpar_from_upascend(vec),
              relabelled=vpar_relabel_uplexqueue(vpar),
              seen=0);
           mapisdefined(m,relabelled,&seen);
           mapput(m,relabelled,seen+1);
           count++);
     m=Mat(m);
     print("labelled "count" distinct "matsize(m)[1]"  unlabelled "vpar_count_unlabelled(n));
  );
  quit;
}


\\-----------------------------------------------------------------------------
\\ vpar_children_subtree_sizes_vector()  -- not very useful

\\ Not sure the merit of these.
\\ Were used for testing, but not much more.

vpar_children_subtree_sizes_of_vertex(vpar,v) =
{
  my(num_below=vectorsmall(#vpar,i,1),
     seq=vpar_INTERNAL_seq_upwards(vpar),
     ret=0);
  for(i=1,#seq,
     my(u=seq[i], p=vpar[u]);
     if(u==v, break(),
        p==v, seq[ret++]=num_below[u],
        p,    num_below[p]+=num_below[u]));
  if(ret,Vec(seq,ret),[]);  \\ cannot Vec(seq,0) which gives
}
{
addhelp(vpar_children_subtree_sizes_of_vertex,
"vec = vpar_children_subtree_sizes_of_vertex(vpar,v)
Return a vector of subtree sizes of the children of v.
Each size counts the child and its descendants.
The order of the children sizes is unspecified.

The set of sizes is equivalent to
apply(c->vpar_subtree_size(vpar,c), vpar_children_of_vertex(vpar,v))
but not necessarily in this order.");
}

vpar_children_subtree_sizes_vector(vpar) =
{
  my(size_lists=vector(#vpar,i,List([])),
     seq=vpar_INTERNAL_seq_upwards(vpar));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v],
        l=(size_lists[v]=Vec(size_lists[v])));
     if(p, listput(size_lists[p],1+vecsum(l))));
  size_lists;
}
{
addhelp(vpar_children_subtree_sizes_vector,
"vec = vpar_children_subtree_sizes_vector(vpar)
Return a vector where each vec[v] is a vector of subtree sizes of each child
of v.  Each size counts the child and its descendants.
vpar can be a forest.

The order of the sizes in each vec[v] (which child they correspond to) is
unspecified.  This makes the sizes suitable for min/max and similar, but not
much more.");
}

check_equal(vpar_children_subtree_sizes_vector(vpar_make_complete_tree(2,2)), \
            [[3,3], [1,1], [1,1], [], [], [], []]);
check_equal(vpar_children_subtree_sizes_vector(vpar_make_binomial_tree(8)), \
            [[1,2,4], [], [1], [], [1,2], [], [1], []]);

check_equal(#vpar_make_complete_tree(2,2), 7);
\\ {
\\   my(vpar=vpar_make_complete_tree(2,2));
\\   check_equal(apply(vecsort, vpar_neighbour_subtree_sizes_vector(vpar)),\
\\               [[3,3], [1,1,4], [1,1,4], [6], [6], [6], [6]]);
\\ }

test_children_subtree_sizes_vector_by_individual(vpar) = \
  vector(#vpar,v,vpar_children_subtree_sizes_of_vertex(vpar,v));
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got =vpar_children_subtree_sizes_vector(vpar),
        want=test_children_subtree_sizes_vector_by_individual(vpar));
     check_equal(got,want, "vpar_children_subtree_sizes_vector"));
}

{
  my(vpar=vpar_make_path(5));
  check_equal(vpar_children_subtree_sizes_of_vertex(vpar,2),
              [3]);
}
{
  my(vpar=vpar_make_complete_tree(2,4));
  check_equal(vpar_children_subtree_sizes_of_vertex(vpar,2),
              [7,7]);
}

\\ this as shown in the addhelp() of vpar_children_subtree_sizes_of_vertex()
test_children_subtree_sizes_of_vertex_by_subtree_size(vpar,v) = \
  apply(c->vpar_subtree_size(vpar,c), vpar_children_of_vertex(vpar,v));

{
  for(i=1,#test_sample_trees,
     my(vpar=test_sample_trees[i]);
     if(#vpar==0,next());
     for(i=1,min(#vpar,5),
        my(v=vpar_random_vertex(vpar));
        my(got =vpar_children_subtree_sizes_of_vertex(Vecsmall(vpar),v));
        my(want=test_children_subtree_sizes_of_vertex_by_subtree_size(vpar,v));
        got=vecsort(got);
        want=vecsort(want);
        if (got!=want,
            vpar_view(vpar);
            print("vpar="vpar,
                  " v="v,
                  " children ",vpar_children_of_vertex(vpar,v)));
        check_equal(got,want,
                    Str("vpar_children_subtree_sizes_of_vertex() "
                        "vs test_children_subtree_sizes_of_vertex_by_subtree_size()"))));
}



\\-----------------------------------------------------------------------------
\\ vpar_neighbour_subtree_sizes_vector()  -- not very useful

vpar_neighbour_subtree_sizes_of_vertex(vpar,v) =
{
  my(vec=vpar_children_subtree_sizes_of_vertex(vpar,v));
  if(vpar[v],
     concat([#vpar-vecsum(vec)-1],vec),   \\ has a parent, its subtree size
     vec);
}
{
addhelp(vpar_neighbour_subtree_sizes_of_vertex,
"vec = vpar_neighbour_subtree_sizes_of_vertex(vpar,v)
Return a vector of subtree sizes of the neighbours of v.
Each size counts the neighbour and its distant vertices.
The order of the neighbour sizes is unspecified.");
}

vpar_neighbour_subtree_sizes_vector(vpar) =
{
  \\ assumes all the rest of the tree is above, ie. that vpar is a single
  \\ tree not a forest

  my(vec=vpar_children_subtree_sizes_vector(vpar));
  for(v=1,#vpar, if(vpar[v], vec[v]=concat([#vpar-vecsum(vec[v])-1],vec[v])));
  vec;
}
{
addhelp(vpar_neighbour_subtree_sizes_vector,
"vec = vpar_neighbour_subtree_sizes_vector(vpar)
Return a vector where vec[v] is a vector of subtree sizes of neighbours of v.
These are the child subtree sizes per vpar_children_subtree_sizes_vector()
and also the size of the subtree at parent, if v has a parent.
Each v itself is not counted in any of the sizes so total #vpar-1 at each v.
vpar must be a single tree, not a forest.
vpar can be a forest.

The order of the sizes in each vec[v] (which neighbour they correspond to) is
unspecified.  This makes the sizes suitable for min/max and similar, but not
much more.");
}

test_neighbour_subtree_sizes_vector_by_individual(vpar) = \
  vector(#vpar,v,vpar_neighbour_subtree_sizes_of_vertex(vpar,v));
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        got =vpar_neighbour_subtree_sizes_vector(vpar),
        want=test_neighbour_subtree_sizes_vector_by_individual(vpar));
     check_equal(got,want, "vpar_neighbour_subtree_sizes_vector"));
}


\\-----------------------------------------------------------------------------
\\ vpar_join() by apply

{
  my(n=#vpar);
  concat(vpar, apply(p->if(p==0,v,p+n), sub_vpar));
}


\\-----------------------------------------------------------------------------
\\ vpar_to_balanced_binary() other ways


  \\ my(seq=vpar_seq_preorder(vpar),
  \\    ret=0,
  \\    pos=2*#vpar,
  \\    prev=0);
  \\ for(i=1,#seq,
  \\    my(v=seq[i]);
  \\    while(vpar[v]!=prev, pos--; prev=vpar[prev]);
  \\    ret = bitor(ret, 1<<pos--);
  \\    prev=v);

  \\ print("depths ",vpar_depths_vector(vpar_relabel_preorder(vpar)));
  \\ print("ret "ret);

  \\ my(seq=vpar_INTERNAL_seq_upwards(vpar),
  \\    bits = vector(#vpar+1,i, if(i==1,0,1)),
  \\    len  = vectorsmall(#vpar+1,i,2));
  \\ for(i=1,#seq,
  \\    my(v=seq[i], p=vpar[v]+1);
  \\    \\ print(" at v="v" bits="digits(bits[v+1],2)" len "len[v+1]" parent "p-1" bits="digits(bits[p],2)" len "len[p]);
  \\    bits[p] = bitor(bits[p] << len[v+1], bits[v+1] << 1);
  \\    len[p] += len[v+1]);

  \\ print(" final "digits(bits[1],2));
  \\ bits[1];


\\-----------------------------------------------------------------------------
\\ vpar_from_perm_noup()
\\
\\ This is cycles as postorder per Dennis Walsh.  Not sure how useful it is
\\ on its own.  Maybe in conjunction with other perm <-> forest.  Would
\\ quite like some to dome out of say seq -> downwards forest, where the seq
\\ would be interpreted as a perm, or set of cycles.
\\
\\ e Oliveira and Las Vergnas "Parking Functions" has other more or less
\\ complicated ways to map n! permutations to specify a downwards.

vpar_from_perm_noup(perm,up=0) =
{
  \\ vpar[v] = perm[v] if it is perm[v] < v, otherwise try p=perm[perm[v]]
  \\ etc looking for <= v and set that vpar[v]=p.
  \\ If v is the biggest in its cycle then it is a root vpar[v]=0.
  \\
  \\ ENHANCE-ME: Find the next lowest in cycles by one pass.

  up=if(up,1,-1);  \\ 1 for up, -1 for down
  vector(#perm,i,
         my(v=i);
         while(1,
               v=perm[v];
               if(v==i, v=0;break());  \\ biggest of cycle is root
               if(lex(v,i)==up, break()));  \\ smaller or bigger than i
         v);

  \\ print("   perm "perm[v]" bigger");
  \\ print("i="i);
  \\ print(" at v="v" m="m" perm="perm[v]);
}
{
addhelp(vpar_from_perm_noup,
"vpar = vpar_from_perm_noup(perm,up=0)
perm is a permutation.  Return a downwards labelled vpar forest
(vpar_is_downwards()) by breaking upwards links in that permutation.

Each parent vpar[v] = perm[v] when that perm[v]<v so vpar is downwards.
If perm[v]>=v then try perm[perm[v]] and successive further perm until < v.
If v is the biggest in its perm cycle, so nothing smaller, then set it as
root vpar[v]=0.

Optional parameter \"up\" can be 1 for an upwards vpar (vpar_is_upwards())
by setting perm[v]>v and likewise seeking >v, instead of <v.

This is per Dennis Walsh http://frank.mtsu.edu/~dwalsh/STIRFORT.pdf
Each cycle in perm becomes a component tree of the forest, so single-cycle
permutations are trees.");
}

\\ vpar_to_perm(vpar,down=0) =
\\ {
\\   \\ vpar[v] = perm[v] if it is perm[v] <= v, otherwise try perm[perm[v]] etc.
\\   \\ If v is the biggest in its cycle then it is a root vpar[v]=0.
\\   \\
\\   \\ ENHANCE-ME: Find the next lowest in cycles by one pass.
\\   \\
\\   down=if(down,1,-1);
\\   vector(#perm,i,
\\          my(v=i);
\\          while(1,
\\                v=perm[v];
\\                if(v==i, v=0;break());  \\ biggest of cycle is root
\\                if(lex(v,i)==down, break()));  \\ smaller or bigger than i
\\          v);
\\
\\   \\ print("   perm "perm[v]" bigger");
\\   \\ print("i="i);
\\   \\ print(" at v="v" m="m" perm="perm[v]);
\\ }
\\ {
\\ addhelp(vpar_to_perm,
\\ "perm = vpar_to_perm(vpar,down=0)
\\ vpar is an upwards labelled forest (vpar_is_upwards()).
\\ Return a permutation uniquely corresponding to that forest.
\\
\\ Optional parameter \"down\" can be 1 for a downwards labelled vpar
\\ (vpar_is_downwards()).
\\
\\ See vpar_from_perm_noup() on how permutation corresponds to up/down forest.");
\\ }

'test_cyc_pow_perm;
kill(test_cyc_pow_perm);
if(! default(secure), \
   iferr(install(cyc_pow_perm, "GL", "test_cyc_pow_perm"), e, 0));
print("test_cyc_pow_perm is ", \
      type(test_cyc_pow_perm),"  ",test_cyc_pow_perm);

\\ v is a vector of Vecsmalls [ Vecsmall([1,2,3]), Vecsmall([4,5,6]) ]
\\ each element is a cycle in the resulting permutation
\\ all 1..n must appear somewhere in the cycles
\\
\\ ENHANCE-ME: is there a direct library cyc_to_perm() ?
\\
test_cycles_to_perm(cycles) =
{
  if(type(test_cyc_pow_perm)=="t_CLOSURE",
     test_cyc_pow_perm(cycles,1),
     test_cycles_to_perm_by_compose(cycles));
}
test_cycles_to_perm_by_compose(cycles) =
{
  my(n=sum(i=1,#cycles, #cycles[i]),
     perm=vectorsmall(n));
  for(i=1,#cycles,
     my(c=cycles[i]);
     for(j=1,#c,
        perm[c[j]] = c[if(j==#c,1,j+1)]));
  perm;
}
check_equal(test_cycles_to_perm([ Vecsmall([1,2]) ]), \
            Vecsmall([2,1]), \
            "test_cycles_to_perm() 1,2");

{
  \\ Dennis Walsh, "Counting Forests with Stirling and Bell Numbers"
  \\ http://frank.mtsu.edu/~dwalsh/STIRFORT.pdf
  \\ example 5 permutation

  my(cycles=[ Vecsmall([1,6,2,8]), Vecsmall([7,3,5,4,9]) ],
     want_perm=Vecsmall([6,8,5,9,4,2,3,1,7]));
  check_equal(test_is_perm(want_perm), 1);
  check_equal(want_perm[1],6);
  check_equal(want_perm[6],2);

  check_equal(test_cycles_to_perm(cycles), want_perm,
              "test_cycles_to_perm() 9");
  check_equal(test_cycles_to_perm_by_compose(cycles), want_perm,
              "test_cycles_to_perm_by_compose() 9");

  check_equal(vpar_INTERNAL_perm_order(want_perm), 20);
  if(test_cyc_pow_perm!='test_cyc_pow_perm,
     my(id=test_identity_perm(#want_perm));
     check_equal(test_cyc_pow_perm(cycles,0), id,
                 "test_cyc_pow_perm() 0");
     check_equal(test_cyc_pow_perm(cycles,20), id);
     );
}

{
  \\ Dennis Walsh, "Counting Forests with Stirling and Bell Numbers"
  \\ http://frank.mtsu.edu/~dwalsh/STIRFORT.pdf
  \\ example 5 permutation to forest

  \\                v = 1 2 3 4 5 6 7 8 9
  my(perm            = Vecsmall([6,8,5,9,4,2,3,1,7]),
     want_vpar       =          [6,8,5,9,9,8,9,0,0]);
  check_equal(perm^-1, Vecsmall([8,6,7,5,3,1,9,2,4]));
  check_equal(vpar_seq_preorder(want_vpar),  Vecsmall([8,2,6,1,9,4,5,3,7]));
  check_equal(vpar_seq_postorder(want_vpar), Vecsmall([2,1,6,8,4,3,5,7,9]));
  check_equal(test_is_perm(perm), 1);
  vpar_validate(want_vpar);
  check_equal(vpar_is_upwards(want_vpar), 1);

  my(vpar=vpar_from_perm_noup(perm,1));
  vpar_validate(vpar);
  check_equal(vpar_is_upwards(vpar), 1,
              "vpar_from_perm_noup() example 5 is upwards");
  check_equal(vpar, want_vpar, "vpar_from_perm_noup() example 5 upwards");

  vpar=vpar_from_perm_noup(perm,0);
  vpar_validate(vpar);
  check_equal(vpar, [0,1,0,3,4,2,3,1,7],
              "vpar_from_perm_noup() example 5 downwards");
  check_equal(vpar_is_downwards(vpar),1,
              "vpar_from_perm_noup() example 5 is downwards");
}

{
  \\ Dennis Walsh, "Counting Forests with Stirling and Bell Numbers"
  \\ http://frank.mtsu.edu/~dwalsh/STIRFORT.pdf
  \\ example 6 tree

  my(perm=test_cycles_to_perm([ Vecsmall([5,1,2,6,3,9,8,4,7,10]) ]));
  check_equal(test_is_perm(perm), 1);
  check_equal(perm,    Vecsmall([2,6,9,7, 1,3,10, 4, 8, 5]));
  check_equal(perm^-1, Vecsmall([5,1,6,8,10,2, 4, 9, 3, 7]));

  my(vpar=vpar_from_perm_noup(perm,1));
  vpar_validate(vpar);
  check_equal(vpar,             [2,6,9,7, 6,9,10,10,10, 0]);
  check_equal(vpar_is_upwards(vpar),1, "monotonic");

  vpar=vpar_from_perm_noup(perm,0);
  vpar_validate(vpar);
  check_equal(vpar, [0,1,1,1,1,3,5,4,8,5]);
  check_equal(vpar_is_downwards(vpar),1);
}

{
  \\ vpar_from_perm_noup() over all permutations
  for(up=0,1,
     for(n=0,4,
        my(l=List([]));
        for(p=0,n!-1,
           my(perm=numtoperm(n,p),
              vpar=vpar_from_perm_noup(perm,up));
           vpar_validate(vpar);
           check_equal(if(up,vpar_is_upwards(vpar),
                             vpar_is_downwards(vpar)),   1);
           listput(l,vpar));
        check_equal(#Set(l),#l))); \\ all vpar unique
}

\\---------------
\\ vpar_downwards_ith() vs vpar_from_perm_noup()

{
  my(forest=1);
  for(n=4,4,
     for(i=1,n!,
        my(vpar=vpar_downwards_ith(n,i,forest),
           perm=Vecsmall(numtoperm(n,i-1))^-1,
           vpar_perm=vpar_from_perm_noup(perm));

        print(vpar"  "vpar_perm" perm="perm);
     );
  );
  quit;
}

\\---------------
{
  \\ Antonio Guedes de Oliveira, Michel Las Vergnas, "Parking Functions and
  \\ Labeled Trees" Seminaire Lotharingien de Combinatoire, volume 65, 2011,
  \\ article B65e.
  \\
  \\ p(T) = reversed preorder
  \\
  \\ sigma    = [a1,...,an] permutation
  \\ sigmabar = [a1,...,an,n+1]
  \\ tree T = phi(sigma) with n+1 vertices
  \\ edge i to k for i<k and a[k] smallest in sigma after a[i]
  \\ In perm, perm[i] next elem perm[j] > perm[i] put edge i to j.
  \\
  \\ desc(i) = descendant vertices of i
  \\ dt(i) = subtree size below i
  \\ dt(T) = subtree sizes vector
  \\
  \\ dp(i) of perm = number of entries immediately following i which have
  \\ perm[j] < perm[i]
  \\ dp(sigma) of perm = vector of dp(i) called "depth"
  \\
  \\ A perm converted has smallers as subtree and next bigger as parent.

  my(vpar=[10,7,7,9,7,3,10,3,1,0]);
  vpar_validate(vpar);
  check_equal(#vpar, 10);

  check_equal(vpar_seq_preorder(vpar),
              Vecsmall([10,1,9,4,7,2,3,6,8,5]),
              "vpar_seq_preorder() 10");
  check_equal(Vecrev(vpar_seq_preorder(vpar)),
              [5,8,6,3,2,7,4,9,1,10]);

  \\   9        8
  \\   |      / | \
  \\   4    2   5   7
  \\   |           / \
  \\   1          3   6
  \\
  my(perm=Vecsmall([1,8,5,2,7,4,6,9,3]));  \\ sigma
  my(vpar=test_perm_to_vpar(perm));
  check_equal(vpar, [4, 8, 7, 9, 8, 7, 8, 0, 0]);
  check_equal(vpar_seq_preorder(vpar),  Vecsmall([8,2,5,7,3,6,9,4,1]));
  check_equal(Vecrev(vpar_seq_preorder(vpar)),
              [1,4,9,6,3,7,5,2,8],
              "T represented linearly");
  check_equal(vpar_seq_postorder(vpar), Vecsmall([2,5,3,6,7,8,1,4,9]));

  check_equal(vpar_from_perm_noup(perm,1),    [0, 8, 5, 8, 7, 8, 8, 9, 0],
              "vpar_from_perm_noup() 10");
  check_equal(vpar_from_perm_noup(perm^-1,1), [0, 4, 9, 6, 9, 7, 9, 9, 0], "inverse");
  check_equal(vpar_from_perm_noup(Vecrev(perm),1),    [3, 9, 6, 0, 7, 9, 0, 0, 0]);
  check_equal(vpar_from_perm_noup(Vecrev(perm^-1),1), [8, 0, 5, 7, 0, 0, 9, 9, 0]);
  check_equal(vpar_from_perm_noup(Vecsmall(Vecrev(perm))^-1,1),
                                               [9, 6, 9, 0, 7, 9, 0, 0, 0]);
}

\\-----------------------------------------------------------------------------
\\ vpar_from_upascend()

\\ ahead=1 childless vertices case
  \\ if(1&&ahead,
  \\    my(vpar=vector(#vec,v,-1));
  \\    if(#vpar,
  \\       my(childless=vpar_childless_vertices(vec),
  \\          upto=#childless+1, step=1, v);
  \\       print(" childless "childless);
  \\       forstep(i=#vpar-1,1,-1,
  \\          print("at i="i" vec="if(i<=#vec,vec[i])" upto="upto" vpar="vpar);
  \\          v=if(i>=1 && i<=#vpar, vec[i], 0);
  \\          if(!v || vpar[v]>=0,
  \\             my(t=if(i<#vpar,vec[i+1],0));
  \\             v=childless[upto--];
  \\             print(" next childless "upto);
  \\            );
  \\          print(" store vpar["v"] = "if(i<#vpar,vec[i+1],0));
  \\          vpar[v] = if(i<#vpar,vec[i+1],0);
  \\       \\ if(vec[i], vpar[vec[i]]=-2);
  \\       );
  \\       print("final upto="upto" "vpar);
  \\       vpar[childless[1]] = vec[1]);
  \\    return(vpar);
  \\   );
  \\

\\ ahead=1 experiment
  \\ if(0&&ahead,
  \\    my(vpar=vector(#vec,v,-1),
  \\       upto=#vpar, step=1, v);
  \\    forstep(i=#vpar-1,1,-1,
  \\       print("at i="i" vec="if(i<=#vec,vec[i])" upto="upto" vpar="vpar);
  \\       v=if(i>=1 && i<=#vpar, vec[i], 0);
  \\       if(!v || vpar[v]>=0,
  \\          my(t=if(i<#vpar,vec[i+1],0));
  \\          while(vpar[upto--]!=-1||upto>=t,); print(" decrease to upto "upto" vs t="t);
  \\          v=upto);
  \\       print(" store vpar["v"] = "if(i<#vpar,vec[i+1],0));
  \\       vpar[v] = if(i<#vpar,vec[i+1],0);
  \\       \\ if(vec[i], vpar[vec[i]]=-2);
  \\    );
  \\    print("final upto="upto);
  \\    while(vpar[upto-=step]>=0,); print(" advance upto "upto);
  \\    vpar[upto]=vec[1];
  \\    return(vpar);
  \\   );

\\ ahead=0 case Cho et al vec[i] and vec[i+1]
  \\ if(!ahead,
  \\    \\ at a vec[i] which is 0 or already seen, search down "upto" for the
  \\    \\ biggest unseen
  \\    my(vpar=vector(#vec,v,-1),
  \\       upto=#vec, v);
  \\    forstep(i=#vpar-1,0,-1,
  \\           vpar[if(!i || !(v=vec[i]) || vpar[v]>=0 || (ahead&&i<upto),
  \\                   while(vpar[upto]>=0, upto--); upto,
  \\                   v)] = vec[i+1]);
  \\    return(vpar);
  \\   );
  \\


\\-----------------------------------------------------------------------------
\\ vpar_INTERNAL_from_upascend()

\\ vec[1..n] is vertex parent numbers of vertices taken in postorder sequence,
\\ as obtained for instance by vec=vecextract(vpar,vpar_seq_postorder(vpar)).
\\ Return the original vpar.
\\ vec==[] is the empty tree.
\\ vec==[0] is the tree of one vertex.
\\ vpar_seq_postorder() always has a root visited last, so vec[#vec]==0 for
\\ a non-empty tree.
\\ ENHANCE-ME: Not sure what name yet for this kind of reversal.
\\
vpar_INTERNAL_from_upascend(vec) =
{
  my(num_children=vpar_INTERNAL_num_children_vecsmall(vec),
     vpar=vector(#vec),
     upto=0);   \\ position in vec[]
  for(i=1,#vec, \\ vertex number
     my(v=i);
     while(num_children[v]==0,
           num_children[v]=-1;
           my(p=vec[upto++]);  \\ pre-increment
           (vpar[v]=p) || break();
           num_children[p]--;
           v=p));
  vpar;
}


{
  my(vpar=[0,0]);    \\ two isolated vertices
  vpar_validate(vpar);
  my(seq=vpar_INTERNAL_seq_upwards(vpar));
  check_equal(seq,Vecsmall([1,2]));
  my(can=vecextract(vpar,seq));
  check_equal(can,[0,0]);
  check_equal(vpar_INTERNAL_from_upascend(can),vpar);
}
{
  my(vpar=[0,1]);    \\ path-2
  vpar_validate(vpar);
  my(seq=vpar_INTERNAL_seq_upwards(vpar));
  check_equal(seq,Vecsmall([2,1]));
  my(can=vecextract(vpar,seq));
  check_equal(can,[1,0]);
  check_equal(vpar_INTERNAL_from_upascend(can),vpar);
}
{
  my(vpar=[2,0]);    \\ path-2 reverse
  vpar_validate(vpar);
  my(seq=vpar_INTERNAL_seq_upwards(vpar));
  check_equal(seq,Vecsmall([1,2]), "seq");
  my(can=vecextract(vpar,seq));
  check_equal(can,[2,0]);
  check_equal(vpar_INTERNAL_from_upascend(can),vpar);
}

{
  my(n=4,
     seen=List([]));
  forvec(vec=vpar_forvec_from(n,1), \\ forests
        my(vpar=vpar_INTERNAL_from_upascend(vec));
        check_equal(setsearch(seen,vpar),0, "postorder not already seen");
        check_equal(#vpar,n);
        listinsert(seen,vpar, setsearch(seen,vpar,1));
        vpar_validate(vpar);
        \\ print("vpar "vpar);
        my(vec_again=vecextract(vpar,vpar_INTERNAL_seq_upwards(vpar)));
        \\ print("vec_again "vec_again);
        check_equal(vec_again,vec, "postorder vec and back again"));
  check_equal(#seen,(n+1)^(n-1));
}


\\ {
\\ addhelp(vpar_INTERNAL_seq_upwards,
\\ "seq = vpar_INTERNAL_seq_upwards(vpar,v)
\\ Return a Vecsmall of the vertex numbers 1..#vpar in a post-order sequence.
\\ The return is Vecsmall since it is a permutation of the integers
\\ 1..#vpar.
\\ This is the sequence of Neville's third canonical representation for
\\ trees.
\\
\\ The sequence starts with the smallest number leaf, which is then reckoned
\\ removed from the tree.  If its parent is now a leaf then it is taken and
\\ reckoned removed, and so on up until a non-leaf.  At that point the next
\\ smallest number leaf remaining is taken, and again up possible chain of
\\ parents, etc.
\\
\\ The effect is that each child is before its parent (so post-order),
\\ children of a given parent are in increasing order, and the last child
\\ is immediately followed by its parent."); }


\\-----------------------------------------------------------------------------
\\ vpar_seq_minmax_at_height()
\\
\\ Vertices sorted by vpar_minmax_height_descendants_vector(), not reversible, not
\\ particularly interesting.

vpar_seq_minmax_at_height(vpar,func,down=0) = \
  vpar_INTERNAL_seq_bucket_sort(vpar_minmax_height_descendants_vector(vpar,func),down);

{
  \\ Deo and Micikevicius sample tree, with 11 as the root
  my(vpar=[11,9,10,7,7,7,1,3,4,4,0]);
  check_equal(vpar_minmax_height_descendants_vector(vpar,min),
              [8,2,8,8,5,6,8,8,2,8,8]);
  check_equal(vpar_minmax_height_descendants_vector(vpar,min,1),
              [8,0,8,8,0,0,8,0,2,8,8]);
}

  \\ Deo and Micikevicius sample tree, with 11 as the root
  check_equal(vpar_seq_minmax_at_height(vpar,min), \
              Vecsmall([2, 9, 5, 6, 1, 3, 4, 7, 8, 10, 11]), \
              "vpar_seq_minmax_at_height()");
  check_equal(vpar_seq_minmax_at_height(vpar,max), \
              Vecsmall([2, 9, 5, 6, 1, 3, 4, 7, 8, 10, 11]), \
              "vpar_seq_minmax_at_height()");

\\-----------------------------------------------------------------------------
\\ vpar_seq_minmax_childless()
\\
\\ Vertices sorted by vpar_minmax_childless_descendants_vector(), not reversible, not
\\ particularly interesting.

vpar_seq_minmax_childless(vpar,func,down=0) = \
  vpar_INTERNAL_seq_bucket_sort(vpar_minmax_childless_descendants_vector(vpar,func),down);

\\-----------------------------------------------------------------------------
\\ vpar_seq_minmax_descendant()
\\ vpar_seq_minmax_ancestor()

vpar_seq_minmax_descendant(vpar,func,down=0) = \
  vpar_INTERNAL_seq_bucket_sort(vpar_minmax_descendants_vector(vpar,func),down);
vpar_seq_minmax_ancestor(vpar,func,down=0) = \
  vpar_INTERNAL_seq_bucket_sort(vpar_minmax_ancestors_vector(vpar,func),down);


\\-----------------------------------------------------------------------------
\\ vpar_to_balanced_binary()

\\ by depths
vpar_preorder_bitcode(vpar) =
{
  my(depths=vpar_depths_vector(vpar_relabel_preorder(vpar)),
     bits=vectorsmall(2*#vpar-1),
     pos=0);
  for(i=2,#depths,
     if(depths[i]<=depths[i-1],
        pos += depths[i]-depths[i-1]+1);
     bits[pos++]=1);
  Vec(bits);
}

check_equal(vpar_preorder_bitcode([0,1,2]), [1,1,0,0,0]);
check_equal(vpar_preorder_bitcode([0,1,1]), [1,0,1,0,0]);


\\-----------------------------------------------------------------------------
\\ vpar_seq_lexmin()

\\ WRONG
test_lexmin_seq_by_keys(vpar) =
{
  my(seq=vpar_seq_upascend(vpar),
     keys=vector(#vpar,v),
     depths=vpar_depths_vector(vpar),
     children=vpar_children_vector(vpar));
  for(i=1,#seq,
     my(p=seq[i]);
     my(children_keys=vecextract(keys,children[p]));
     children_keys = vecsort(children_keys,lex);
     keys[p] = [ depths[p], -#children[p], children_keys ]);
  for(v=1,#vpar,
     keys[v] = concat(keys[v], [v]));  \\ by vertex number among equals
  \\ print(keys);
  vecsort(keys,lex,1);
}
test_to_lexmin_by_keys(vpar) = \
  vpar_relabel_perm(vpar,test_lexmin_seq_by_keys(vpar)^-1);



  check_equal(test_lexmin_seq_by_keys(vpar), want_seq,
             "test_lexmin_seq_by_keys() forest 3");

  \\ check_equal(test_to_lexmin_by_keys(vpar), want_vpar,
  \\             "test_to_lexmin_by_keys() forest 6");
  \\ check_equal(test_lexmin_seq_by_keys(vpar), want_seq,
  \\            "test_lexmin_seq_by_keys() forest 6");

  check_equal(got_seq,
              test_lexmin_seq_by_keys(vpar),
              "vpar_seq_lexmin() vs test_lexmin_seq_by_keys() 16");


\\ NO, WRONG
test_lexmin_perm_by_sort(vpar) =
{
  \\ print("test_lexmin_perm_by_sort() "vpar);
  if(#vpar,
     my(rows=vpar_row_vertices_vector(vpar),
        children_vector=vpar_children_vector(vpar),
        perm=vectorsmall(#vpar));
     \\ print("children "children_vector);
     forstep(d=#rows,1,-1,
            for(i=1,#rows[d],
               my(v=rows[d][i]);
               children_vector[v] = vecsort(children_vector[v],
                                            (x,y)->lex(#children_vector[y],
                                                       #children_vector[x]))));
     \\ print("sorted   "children_vector);

     my(upto=0);
     for(d=1,#rows,
        my(s=vecsort(rows[d],
                     (x,y)->
                     my(l=if(vpar[x],lex(perm[vpar[x]],perm[vpar[y]])));
                     while(!l,
                           l=lex(#children_vector[y],#children_vector[x]);
                           #children_vector[x] || break();
                           #children_vector[y] || break();
                           x=children_vector[x][1];
                           y=children_vector[y][1]);
                     l));
        for(i=1,#rows[d],
           perm[s[i]] = upto++);
        \\ print(" row d="d"  "rows[d]" sorted "s"  perm "Vec(perm));
     );
     perm;
     ,
     Vecsmall([]));

  \\ Or descending into num children each time.
  \\ Is this actually right though?
  \\
  \\ if(#vpar,
  \\    my(num_children=vpar_INTERNAL_num_children_vecsmall(vpar),
  \\       c = (u,v)->my(l=lex(vpar[u],vpar[v]));
  \\                  while(!l,
  \\                        l=lex(num_children[v],num_children[u]);
  \\                        v=vpar[v] || break; u=vpar[u]);
  \\                  if(l,l, lex(u,v)),
  \\       rows=vpar_row_vertices_vector(vpar));
  \\    for(d=1,#rows,
  \\       rows[d] = vecsort(rows[d],c);
  \\       print(" row d="d"  "rows[d]);
  \\    );
  \\    Vecsmall(concat(rows));
  \\    ,
  \\    Vecsmall([]));
}

  print(vpar_relabel_perm(vpar,test_lexmin_perm_by_sort(vpar)));
  check_equal(want_seq,
              test_lexmin_perm_by_sort(vpar)^-1,
              "test_lexmin_perm_by_sort() 10");

     print(vpar);
     check_equal(got_seq, test_lexmin_perm_by_sort(vpar)^-1,
                 "vpar_seq_lexmin() vs test_lexmin_perm_by_sort()");

     \\ print(vpar);
     \\ check_equal(got_vpar, test_to_lexmin_by_premax(vpar),
     \\             "vpar_to_lexmin() vs test_to_lexmin_by_premax()");


\\-----------------------------------------------------------------------------
\\ Automorphisms List

vpar_automorphisms(vpar) =
{
 \\ WRONG
  print("vpar_automorphisms() "vpar);
  my(num=vpar_num_automorphisms(vpar),
     ret=vector(num),
     gens=vpar_automorphism_generators(vpar),
     upto=1);
  print(" num "num);
  print(" gens "gens);
  ret[1] = vpar_INTERNAL_identity_perm(#vpar);
  for(i=1,#gens,
     print(" gens "i" = "gens[i]" at upto="upto);
     my(p=gens[i]);
     my(end=p^-1);
     print("  end "end);
     while(1,
           print("  p="p" at upto="upto);
           for(j=1,upto,
              print("   j="j" to ret "upto+1);
              ret[upto++] = p*ret[j]);
           if(p==end,break);
           p *= gens[i];
          ));
  ret;
}


vpar_INTERNAL_automorphisms() separate code for vpar_automorphism_generators()

     flags==132,       \\ vpar_automorphism_generators()
     print(" generators");
     print("  subtree_chain "Vec(subtree_chain));
     print("  subtree_tail  "Vec(subtree_tail));

     my(gen_pos = vectorsmall(#vpar+1));
     rep = prev = gen_pos;
     ret = List([]);
     forstep(i=#seq,1,-1,     \\ roots downwards
        my(v=seq[i],
           p=vpar[v],
           prev_v = prev[p+1],
           same=(prev_v && id[v] && id[prev_v]==id[v]));
        rep[p+1] = if(same,rep[p+1]+1,0);
        if(same,
           print("  v="v" prev="prev_v" under p="p);
           my(pos=if(rep[p+1]<=2,
                     listput(ret,if(rep[p+1]==1,
                                    vpar_INTERNAL_identity_perm(#vpar),
                                    ret[gen_pos[p+1]]));  \\ copy
                     gen_pos[p+1]=#ret;
                     ,
                     gen_pos[p+1]));

           \\ apply swap trees x<->y to permutation at ret[pos]
           my(x=prev_v, y=v, tail=subtree_tail[x+1]);
           pos=#ret;
           while(1,
                 print("   pair "x" <-> "y"    (to tail "tail")");
                 my(t=ret[pos][x]); ret[pos][x]=y; ret[pos][y]=t;
                 if(x==tail, break());
                 id[x]=0;
                 x=subtree_chain[x+1];
                 y=subtree_chain[y+1]);
          print("   is "Vec(ret[pos]));
          );
        prev[p+1]=v;
     );
     Vec(ret);
     ,

{
  \\ gens order 2, 5
  \\ total num 120
  my(vpar=[0,1,1,1,1,1]);
  check_equal(vpar_num_automorphisms(vpar), 5!);
  my(gens=vpar_automorphism_generators(vpar));
  check_equal(vpar_INTERNAL_perm_order(gens[1]), 2);
  check_equal(vpar_INTERNAL_perm_order(gens[2]), 5);
  check_equal(vpar_INTERNAL_perm_order(gens[1]*gens[2]), 4);
  \\ print(gens[1]);
  \\ print(gens[1]*gens[2]);
  check_equal(vpar_INTERNAL_perm_order(gens[1]*gens[2]^2), 6);
  check_equal(vpar_INTERNAL_perm_order(gens[1]*gens[2]^3), 6);
  check_equal(vpar_INTERNAL_perm_order(gens[1]*gens[2]^4), 4);
  check_equal(vpar_INTERNAL_perm_order(gens[2]^2*gens[1]*gens[2]^-2), 2);
  check_equal(vpar_INTERNAL_perm_order(gens[2]^3*gens[1]*gens[2]^-3), 2);
  check_equal(vpar_INTERNAL_perm_order(gens[2]^4*gens[1]*gens[2]^-4), 2);
  check_equal(vpar_INTERNAL_perm_order(gens[1]*gens[2]*gens[1]), 5);
}
\\-----------------------------------------------------------------------------
\\     v = vpar_random_root(vpar)
\\     v = vpar_random_childless(vpar)
\\     v = vpar_random_parent(vpar)
\\     v = vpar_random_child(vpar)         "nonroot"
\\
\\ not sure very useful

vpar_random_root(vpar) =
{
  my(m=vpar_num_roots(vpar));
  if(m,
     m=random(m);
     for(v=1,#vpar,
        if(!vpar[v] && m--<0, return(v))));
}
{
addhelp(vpar_random_root,
"v = vpar_random_root(vpar)
Return a random root vertex of vpar.
This is a uniform random choice among the roots.
If empty vpar=[] then return 0 since there are no vertices.

The opposite, a random non-root, is vpar_random_child().");
}

\\ FIXME: name random non-root ?
vpar_random_child(vpar) =
{
  my(m=vpar_num_edges(vpar));
  if(m,
     m=random(m);
     for(v=1,#vpar,
        if(vpar[v] && m--<0, return(v))));
}
{
addhelp(vpar_random_child,
"v = vpar_random_child(vpar)
Return a random child vertex of vpar, ie. a vertex which has a parent, so
any non-root.  This is a uniform random choice among child vertices.
This also represents a random edge, from child v its parent vpar[v].

If vpar has no child vertices, because it is empty vpar=[] or entirely
singletons, then return 0.

The opposite, a random non-child, is vpar_random_root().");
}

\\ flags is a vector of booleans 0 or !=0.
\\ Return index i which is a uniform random choice from among those flags[i].
\\ So flags treated as a set this is a random choice from the set.
vpar_INTERNAL_random_of_flags(flags,zero=0) =
{
  my(m=hammingweight(flags));
  if(zero, m=#flags-m);
  if(m,
     m=random(m);
     for(v=1,#flags, if(lex(flags[v],0)!=zero && m--<0, return(v))));
}

\\ not sure how useful
vpar_random_parent(vpar) = \
  vpar_INTERNAL_random_of_flags(vpar_INTERNAL_num_children_vecsmall(vpar));
{
addhelp(vpar_random_parent,
"v = vpar_random_parent(vpar)
Return a random parent vertex of vpar, ie. a vertex which has a parent, so
any non-root.  This is a uniform random choice among parent vertices.

If vpar has no parents, because it is empty vpar=[] or entirely singletons,
then return 0.

The opposite, a random non-parent, is vpar_random_childless().");
}

\\ not sure how useful
vpar_random_childless(vpar) = \
  vpar_INTERNAL_random_of_flags(vpar_INTERNAL_num_children_vecsmall(vpar),1);
{
addhelp(vpar_random_childless,
"v = vpar_random_childless(vpar)
Return a random childless vertex of vpar, being those 1..#vpar which do not
appear anywhere in the vpar vector.  This is a uniform random choice among
childless vertices.
The empty vpar=[] has no vertices at all so for it return 0.

The opposite, a random vertex with a child, is vpar_random_parent().");
}


\\-----------------------------------------------------------------------------
\\ vpar_random_childless()

check_equal(vpar_random_childless([]),0, "vpar_random_childless() empty");
check_equal(vpar_random_childless([0]),1, "vpar_random_childless() singleton");

\\ vpar_random_childless() but calling to test_random()
test_INTERNAL_random_of_flags = \
  eval(test_random_to_test_random(vpar_INTERNAL_random_of_flags));
test_vpar_random_childless = \
  eval(test_string_replace(Str(vpar_random_childless), \
                           "vpar_INTERNAL_random_of_flags(", \
                           "test_INTERNAL_random_of_flags("));
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        v=vpar_random_childless(vpar));
     check_equal(vpar_is_childless(vpar,v), 1,
                 "vpar_random_childless() is childless");

     my(got=vpar_random_childless(vpar));
     if(#vpar,
        check_equal(vpar_is_childless(vpar,got), 1,
                    "vpar_random_childless() vs vpar_is_childless()"),
        check_equal(got,0, "vpar_random_childless() is 0 on empty"));

     my(childless=vpar_childless_vertices(vpar));
     if(#vpar==0,
        test_random_list      =List([]);
        test_random_limit_list=List([]);
        my(v=test_vpar_random_childless(vpar));
        check_equal(#test_random_list, 0, "test_random_list consumed");
        check_equal(#test_random_limit_list, 0, "test_random_limit_list consumed");
        check_equal(v,0, "vpar_random_childless() of empty");
        ,
        for(j=1,#childless,
           test_random_list      =List([j-1]);
           test_random_limit_list=List([#childless]);
           my(v=test_vpar_random_childless(vpar));
           check_equal(#test_random_list, 0, "test_random_list consumed");
           check_equal(#test_random_limit_list, 0, "test_random_limit_list consumed");
           check_equal(v,childless[j], "vpar_random_childless() vertex");
        )));
}


\\-----------------------------------------------------------------------------
\\ vpar_random_root()

check_equal(vpar_random_root([]),0, "vpar_random_root() empty 0");

\\ vpar_random_root() but calling to test_random()
test_vpar_random_root = eval(test_random_to_test_random(vpar_random_root));
{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);

     my(got=vpar_random_root(vpar));
     if(#vpar,
        check_equal(vpar[got],0, "vpar_random_root() is root"),
        check_equal(got,0, "vpar_random_root() is 0 on empty"));

     my(roots=vpar_roots(vpar));
     if(#vpar==0,
        test_random_list      =List([]);
        test_random_limit_list=List([]);
        my(v=test_vpar_random_root(vpar));
        check_equal(#test_random_list, 0, "test_random_list consumed");
        check_equal(#test_random_limit_list, 0, "test_random_limit_list consumed");
        check_equal(v,0, "vpar_random_root() of empty");
        ,
        for(j=1,#roots,
           test_random_list      =List([j-1]);
           test_random_limit_list=List([#roots]);
           my(v=test_vpar_random_root(vpar));
           check_equal(#test_random_list, 0, "test_random_list consumed");
           check_equal(#test_random_limit_list, 0, "test_random_limit_list consumed");
           check_equal(v,roots[j], "vpar_random_root() vertex");
        )));
}


\\-----------------------------------------------------------------------------
\\ vpar_random_parent()

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        v=vpar_random_parent(vpar));
     if(test_is_all_singletons(vpar),
        check_equal(v,0);
        ,
        check_equal(v!=0, 1);
        check_equal(vpar_vertex_is_parent(vpar,v), 1,
                    "vpar_random_parent() is a parent");
       ));
}


\\-----------------------------------------------------------------------------
\\ vpar_random_child()

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i],
        v=vpar_random_child(vpar));
     if(test_is_all_singletons(vpar),
        check_equal(v,0);
        ,
        check_equal(v!=0, 1);
        check_equal(vpar[v]!=0, 1, "vpar_random_child() has a parent");
        );
  );
}


\\-----------------------------------------------------------------------------
\\ What was this?

vpar_INTERNAL_subtree_vertices_by_ids(vpar,v,ids) =
{
  my(c=vpar_children_of_vertex(vpar,v));
  c=vecsort(c,(x,y)->lex(ids[x],ids[y]));
  c=apply(v->vpar_subtree_vertices_by_ids(vpar,v,ids), c);
  concat([v],concat(c));
}


\\-----------------------------------------------------------------------------
\\ vpar_dreadnaut_roots()
\\ without ranges


\\-----------------------------------------------------------------------------
\\ vpar_unlabelled_prev()

\\ WRONG, cannot increase a subtree when it is part of a higher one equal to
\\ is prev sibling
vpar_unlabelled_prev(vpar,forest=0) =
{
  print("vpar_unlabelled_prev() "vpar);
  my(g=#vpar, p);
  while((p=g)>1,
        (g=vpar[p]) || forest || break;
        my(q=p-1);
        print(" try p="p" g="g);
        if(q==g,
           print("  p no sibling");
           next);
        while(vpar[q]!=g, q=vpar[q]);
        my(len=p-q);  \\ size of q subtree
        print("  q="q" len="len);
        if(p+len <= #vpar,
           print("  p subtree size "#vpar-p+1" bigger than len");
           next);
        for(i=p+1,#vpar, 
           if(vpar[i]!=vpar[i-len]+len,
              print("  p different at i="i"  "vpar[i]" vs "vpar[i-len]+len);
              next(2)));
        my(r);
        while((r=q-len) >= 1 && vpar[r]==g,
              print("  r="r);
              for(i=q+1,p-1,
                 if(vpar[i]!=vpar[i-len]+len,
                    print("  r different at i="i"  "vpar[i]" vs "vpar[i-len]+len);
                    break(2)));
              print("   same");
              p=q; q=r);

        r = !forest;
        print("  found p="p" q="q"  fill "r);
        vpar[p]=q; 
        for(i=p+1,#vpar, vpar[i]=r);
        return(vpar);
        );
  print(" none");
  'none;
}



\\-----------------------------------------------------------------------------
\\ vpar_preorder_prev() with inlined sibling search

vpar_preorder_prev(vpar,forest=0) =
{
  forstep(v=#vpar,2,-1,
         my(p=v-1);
         \\ print("v="v" p="p" vpar[v]="vpar[v]);
         if(vpar[v]!=p,
            while(vpar[p]!=vpar[v], p=vpar[p]);
            \\ print(" drop to p="p" for v="v);
            vpar[v]=p;
            forest=!forest;  \\ 1 for tree, 0 for forest
            for(i=v+1,#vpar, vpar[i]=forest);
            return(vpar)));
  'none;
}


\\-----------------------------------------------------------------------------

\\ test_perm_order(perm) = Pari library perm_order() when
\\ possible to install() it, otherwise some code here.
\\ perm must be Vecsmall.
\\ The library doesn't check the type of perm and passing non-Vecsmall
\\ (including say full Vec) will segfault.
'test_perm_order;
kill(test_perm_order);
{
  if(! if(! default(secure),
          iferr(install(perm_order, "lG", "test_perm_order"); 1,
                e, 0)),
     test_perm_order
     = (perm)->prod(i=1,#perm,
                    my(t=i, cycle=0);
                    until(t==i,
                          if(cycle++>#perm, error());  \\ catch not cyclic
                          my(c=perm[t]); perm[t]=t; t=c);  \\ zap when seen
                    cycle));
}


\\-----------------------------------------------------------------------------
\\ vpar_free_next() forest bits

  \\ L1 height is attained by the first vertex in L1 which is not a descent
  \\ (ie. v-1 isn't its parent)

  \\ L2 height is at first non-descent after m.
  \\ L2_height = after L2 prospective reduction.  If L2 height is attained
  \\ at p then 1 smaller due to p moving up.
  \\
  print(" L2 height default p-m = "L2_height", after prospective step");
  for(v=m+1,p-1,
     if(vpar[v]!=v-1,   \\ not path down from m
        L2_height = v-m;
        break()));
  forstep(v=#vpar,1,-1,
     if(vpar[v] && (forest || vpar[vpar[v]]),
        e=p=v;break));

     \\ if(L1_height==L2_height && L2_size < L1_size,
     \\    p = m-1;
     \\    print("  equal height but L2 size too small,  copy L1 from p="p);
     \\    );

     \\ if(L1_height==L2_height && l < 2*L1_size,
     \\    e = #vpar - ((l-3)>>1);
     \\    print("  partial makes L2 too small, set e="e);
     \\    ,
     \\    L1_height!=L2_height && l < L1_size + L2_height + 1,
     \\    e = max(#vpar - ((l-3)>>1),
     \\            #vpar - L2_height + 1);
     \\    print("  partial chops L2 height, set e="e);
     \\    ,
     \\    l <= L1_size + L2_height,
     \\    e = #vpar - L1_height;
     \\    print("  truncates, set e="e" r="r);
     \\   );

     if(l < L1_height+2,
        p = #vpar - ((l-3)>>1);
        print("  L1 height partial, path down from p="p);
        for(v=p,#vpar, vpar[v]=r; r=v);
        return(vpar);
        );

        \\ for(v=m,p-forest,
        \\    print("  cmp vpar["v-offset"]="vpar[v-offset]" and vpar["v"]~"if(vpar[v]==r,r+1,vpar[v]-offset));
        \\    if(c = lex(vpar[v-offset], if(vpar[v]==r,r+1,vpar[v]-offset)),
        \\       break));


\\-----------------------------------------------------------------------------
\\ Separate test funcs.

\\     bool = vpar_is_automorphisms_solvable(vpar)
\\     bool = vpar_is_automorphisms_supersolvable(vpar)
\\     bool = vpar_is_automorphisms_2group(vpar)
\\     bool = vpar_is_automorphisms_Abelian(vpar)

\\   bits 1+2   rep_limit = low two bits 1,2,3, or 0 for no limit
\\ 1070 = vpar_is_automorphisms_supersolvable() 2+4+8   +32          +1024
\\   45 = vpar_is_automorphisms_2group()        1+4+8   +32         
\\ 1069 = vpar_is_automorphisms_Abelian()       1+4+8   +32          +1024

vpar_is_automorphisms_solvable(vpar) = \
  vpar_INTERNAL_automorphisms(vpar, 47);
{
addhelp(vpar_is_automorphisms_solvable,
"bool = vpar_is_automorphisms_solvable(vpar)
Return 1 if the automorphism group of vpar is solvable.
In vpar this means isomorphic sibling subtrees in sets of at most 4 each.

Tree automorphism groups are direct products or wreath products of symmetric
groups S(n).  Both such products are solvable if and only if their terms
are.  S(2), S(3), S(4) are the only solvable symmetrics, hence at most 4
same siblings.");
}

vpar_is_automorphisms_supersolvable(vpar) = \
  vpar_INTERNAL_automorphisms(vpar, 1070);
{
addhelp(vpar_is_automorphisms_supersolvable,
"bool = vpar_is_automorphisms_supersolvable(vpar)
Return 1 if the automorphism group of vpar is supersolvable.
In vpar this means isomorphic sibling subtrees in sets of at most 3 each,
or if such subtrees contain nested isomorphics then at most 2 there and
within.

Tree automorphism groups are direct products or wreath products of symmetric
groups S(n).  Direct products are supersolvable if and only if the terms
are, which means S(2) and S(3) only.  Wreath product G wr H is supersolvable
(per Durbin) if and only if G is nilpotent (vpar_is_automorphisms_2group())
and H either Abelian or certain conditions on derived H'.  So at most S(2)
within G, and only S(2) for H (the further conditions not allowing S(3) or
more).");
}

vpar_is_automorphisms_2group(vpar) = \
  vpar_INTERNAL_automorphisms(vpar, 45);
{
addhelp(vpar_is_automorphisms_2group,
"bool = vpar_is_automorphisms_2group(vpar)
Return 1 if the automorphism group of vpar is a 2-group, meaning the number
of automorphisms is a power 2^k.

In vpar this means isomorphic sibling subtrees in sets of at most 2 each.
A vertex can have 1 other sibling with the same (isomorphic) subtree, but no
more than 1 other.

This is also trees whose automorphism group is nilpotent.  Finite 2-groups
are a subset of nilpotent.  For tree automorphisms, there are no extras
nilpotent.

Each automorphism has permorder() a power of 2 (divisor of the group size
in the usual way).  See vpar_is_automorphisms_Abelian() for a tighter test
demanding each automorphism exactly 2.");
}
\\ For nilpotent equivalent to 2-groups:
\\ Tree automorphism groups are direct products or wreath products of
\\ symmetric groups S(n).  Direct product is nilpotent if and only if the
\\ terms are, and only symmetric S(2) is nilpotent.  Wreath product is
\\ nilpotent if and only if the terms are p-groups of the same prime p.
\\ Here this means only p=2.

vpar_is_automorphisms_Abelian(vpar) = \
  vpar_INTERNAL_automorphisms(vpar, 1069);
{
addhelp(vpar_is_automorphisms_Abelian,
"bool = vpar_is_automorphisms_Abelian(vpar)
Return 1 if the automorphism group of vpar is Abelian.

In vpar this means isomorphic sibling subtrees in sets of at most 2 each,
and no nesting.  A vertex can have 1 other sibling with the same
(isomorphic) subtree, but no more than 1 other, and when it does have 1
other those subtrees must be asymmetric (vpar_is_asymmetric()).

The number of automorphisms (vpar_num_automorphisms()) is a power of 2, and
each automorphism is an involution, ie. permorder() == 2.");
}
\\ Tree automorphism groups are direct products or wreath products of
\\ symmetric groups S(n).  Direct product is Abelian if and only if the
\\ terms are, and only S(2) is Abelian.  Wreath product G wr H is never
\\ Abelian, as g*h or h*g put g in a different copy of G.

{
  my(vpar=vpar_make_complete_tree(2,2),   \\ S2 wr S2
     perms=vpar_automorphisms(vpar));
  check_equal(#perms, 8);
  check_equal(vpar_is_automorphisms_2group(vpar), 1,
             "vpar_is_automorphisms_2group() complete tree 2,2, yes");
  check_equal(test_perms_is_2group(perms), 1,
             "test_perms_is_2group() complete tree 2,2, yes");
  check_equal(test_perms_is_nilpotent(perms), 1,
             "test_perms_is_nilpotent() complete tree 2,2, yes");

  vpar = vpar_to_unlabelled(vpar);
  check_equal(vpar_is_automorphisms_2group(vpar), 1,
             "vpar_is_automorphisms_2group() complete tree 2,2, yes");
}


\\-----------------------------------------------------------------------------

vpar_is_preorder_by_rev(vpar,post=0) =
{
  \\ print("vpar_is_preorder_by_rev() "vpar);
  my(m=vectorsmall((#vpar >> 1) + 1),
     d=1,
     step=if(post,1,-1));
  forstep(v=if(post,1,#vpar), if(post,#vpar,1), step,
     \\ print(" at v="v" p="vpar[v]"  d="d" m="Vec(m)[1..d]);
     if(v == m[d],
        \\ print("  pop m==v");
        d--);
     if(lex(v,vpar[v]) == step || lex(vpar[v],m[d]) == step,
        \\ print("  out of range");
        return(0));
     if(lex(m[d],vpar[v]) == step,
        \\ print("  push m "vpar[v]);
        m[d++]=vpar[v]);
  );
  1;
}

\\-----------------------------------------------------------------------------
\\ vpar_is_preorder_by_rev()

check_equal(vpar_is_preorder_by_rev([]), 1, "vpar_is_preorder() empty");
check_equal(vpar_is_preorder_by_rev([0]), 1);
check_equal(vpar_is_preorder_by_rev([0,1]), 1);
check_equal(vpar_is_preorder_by_rev([2,0]), 0);
check_equal(vpar_is_preorder_by_rev([0,1,2, 0,4]), 1);
check_equal(vpar_is_preorder_by_rev([0,1,2, 1,2]), 0);

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(vpar_is_preorder_by_rev(vpar),
                 vpar_is_preorder(vpar));
  );
}



\\-----------------------------------------------------------------------------
\\ setproduct by explicit code

vpar_INTERNAL_setproduct(u,v) =
{
  my(ret=vector(#u*#v),pos=0);
  for(i=1,#u,
     for(j=1,#v,
        ret[pos++]=setunion(u[i],v[j])));  \\ pre-increment
  ret;
}



\\-----------------------------------------------------------------------------
\\ vpar_siblings_cycles()
\\
\\     perm = vpar_siblings_cycles(vpar,down=0)
\\
\\ Think not much use for this.  Cute to make a type of perm out of the
\\ vpar, but vpar_next_sibling_vector() should be more use.

\\ Could similar:
\\ perm = vpar_orbit_cycles(vpar,down=0)  perm[v] = next in orbit
\\ perm = vpar_free_orbit_cycles(vpar) 

vpar_siblings_cycles(vpar,down=0) = \
  vpar_INTERNAL_next_sibling_vector(vpar,vpar_INTERNAL_identity_perm,down);
{
addhelp(vpar_siblings_cycles,
"perm = vpar_siblings_cycles(vpar,down=0)
Return a Vecsmall perm where sets of siblings are in cycles.
Each perm[v] = next bigger sibling of v, or if none bigger then back to the
smallest sibling of v.

Optional parameter \"down\" can be 1 to go downwards perm[v] = next smaller
sibling of v, or if no smaller then cycle up to the biggest.");
}

check_equal(vpar_siblings_cycles([]), Vecsmall([]), "vpar_siblings_cycles() empty");
check_equal(vpar_siblings_cycles([0,0]), Vecsmall([2,1]), \
            "vpar_siblings_cycles() 2 singletons");
check_equal(vpar_siblings_cycles([0,1,0,0]), Vecsmall([3,2,4,1]), \
            "vpar_siblings_cycles() path-2 and 2 singletons, up");
check_equal(vpar_siblings_cycles([0,1,0,0],1), Vecsmall([4,2,1,3]), \
            "vpar_siblings_cycles() path-2 and 2 singletons, down");
check_equal(vpar_siblings_cycles([3,3, 0, 3]), Vecsmall([2,4, 3, 1]));

test_sibling_cycles_by_next(vpar,down=0) =
{
  my(first_sibling=vpar_minmax_sibling_vector(vpar,if(down,max,min)),
     next_sibling=vpar_next_sibling_vector(vpar,down));
  vectorsmall(#vpar,v, if(next_sibling[v],next_sibling[v],first_sibling[v]));
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     for(down=0,1,
        check_equal(vpar_siblings_cycles(vpar,down),
                    test_sibling_cycles_by_next(vpar,down),
                    "vpar_siblings_cycles() vs test_sibling_cycles_by_next()");
     ));
}


\\-----------------------------------------------------------------------------
\\ vpar_subtree_heights_vectors_count() variants

vpar_subtree_heights_vectors_count(n,forest=0) =
{
  \\ code following formula by Vladimir Kruchinin in A005651
  forest>0 || n--;
  if(n<=0, 1,
     my(a=vector(n,m, vector(m,i,1)));
     for(n=2,n,
        for(m=1,n-1,
           a[n][m] = 1 + sum(k=m,n>>1, a[n-k][k] * binomial(n,k))));
     a[n][1] * if(forest,1,n+1));
}

vpar_subtree_heights_vectors_count(n,forest=0) =
{
  \\ Sum and sumdiv as per Peter Luschny in A182926.
  \\ Here b[n]=A182926 used to make a[n]=A005651.
  \\
  \\ The formula by Vladeta Jovovic in A005651 is essentially the same, but
  \\ the b(k) there is fractions so here instead b(k)*(k-1)!, which is A182926.
  \\ Think it should be more efficient to stay in integers.

  \\ n-1 for trees and root=1 trees
  if((n-=(forest<=0)) < 1,  1, 
     my(a=vector(n)); a[1]=1;
     my(b=a,
        f=a);   \\ f[n]=n! factorials
     for(n=2,n,
        f[n] = f[n-1]*n;
        b[n] = sumdiv(n,d, f[n-1] * d / f[d]^(n/d));
        a[n] = b[n] + sum(k=1,n-1, binomial(n-1,k-1) * b[k] * a[n-k]));
     if(forest,a[n],a[n]*(n+1)));
}


\\-----------------------------------------------------------------------------
\\ vpar_from_balanced_binary()

  \\ Similar to vpar_make_preorder_depths(), but depth is net 0 bits working
  \\ from low of b, which means high to low of binary(b) vector.
  \\ When b[i]==0 it means v is one deeper, just incr d.
  \\ When b[i]==1 it means v is at depth d-1.  Follow linked list of those
  \\ at depth d (if any) setting parent v.  Then v prepended to chain d-1,
  \\ except for root has no chain.
  \\
  \\ Might like to bittest() instead of b in a binary vector.
  \\ But how to get the bit length other than #binary(b) ?

  \\ b=binary(b);
  \\ \\ print("vpar_from_balanced_binary()  "b);
  \\ my(v=#b>>1,
  \\    vpar=vector(v),
  \\    d=0);                 \\ depth of v
  \\ forstep(i=#b,1,-1,
  \\    print(" i="i" v="v" d="d" bit "b[i]"  vpar "vpar);
  \\    if(b[i],
  \\       my(t=vpar[d], u);  \\ traverse linked list setting parent v
  \\       while(u=t,
  \\             \\ print("  chain u="u" parent v="v);
  \\             t=vpar[u];
  \\             vpar[u]=v);
  \\       vpar[d]=0;  \\ zap to empty list
  \\ 
  \\       if(d--, vpar[v]=vpar[d]; vpar[d]=v);  \\ prepend v to chain d
  \\       v--;
  \\       ,
  \\       d++));
  \\ vpar;

  \\ \\ b[] is bits of b.
  \\ \\ Then re-used as b[d+1] = last vertex at depth d, eg. b[1]=last root.
  \\ \\ b[1]=1 always so can skip that.
  \\ \\
  \\ \\ Might like to bittest() instead of b as binary vector.
  \\ \\ Where is the bit length other than #binary(b) ?
  \\ \\
  \\ b=binary(b);
  \\ \\ print("vpar_from_balanced_binary()  "b);
  \\ my(vpar=vector(#b>>1),
  \\    v=1,
  \\    d=1);                 \\ depth of v
  \\ for(i=2,#b,
  \\    if(b[i],
  \\       v++;
  \\       \\ print(" i="i" v="v" parent "if(d,b[d],"X"));
  \\       if(d, vpar[v] = b[d]);   \\ can d==0 when forest
  \\       b[d++]=v;
  \\       ,
  \\       \\ print(" i="i" decr to d="d-1);
  \\       d--));
  \\ vpar;

  \\ b=binary(b);
  \\ print("vpar_from_balanced_binary()  "b);
  \\ my(vpar=vector(#b>>1), v=1, p=1);
  \\ for(i=2,#b-1,
  \\    if(b[i],
  \\       print(" i="i" v="v+1" p="p);
  \\       vpar[v++]=p; p=v;
  \\       ,
  \\       p=vpar[p]));
  \\ vpar;


\\-----------------------------------------------------------------------------

test_highest_0s_run_extents(n) =
{
  my(pos=0, hi=-1, lo=0);
  while(1,
        while(bitand(n,1), n>>=1; pos++);   \\ strip low 1s
        n || break;
        lo=pos;   \\ lowest 0
        while(!bitand(n,1), hi=pos; n>>=1; pos++));  \\ count and strip low 0s
  [hi,lo];
}
check_equal(test_highest_0s_run_extents(test_from_binary(0)), [-1,0]);
check_equal(test_highest_0s_run_extents(test_from_binary(1)), [-1,0]);
check_equal(test_highest_0s_run_extents(test_from_binary(10)), [0,0]);
check_equal(test_highest_0s_run_extents(test_from_binary(111000111)), [5,3]);
check_equal(test_highest_0s_run_extents(test_from_binary(1100101011)), [7,6]);
{
  for(rep=1,20,
     my(b=random(2^64),
        e=test_highest_0s_run_extents(b));
     check_equal(test_highest_0s_run_length(b),
                 e[1]-e[2]+1,
       "test_highest_0s_run_length() vs test_highest_0s_run_extents()"));
}



{
  \\ balanced binary how many highest 00..001 can be flipped to 100..00

  my(forest=1);
  for(n=2,min(12,#A014138_samples),
     my(vpar=vpar_preorder_first(n),
        total=0);
     until((vpar=vpar_preorder_next(vpar,forest))=='none,  \\ increasing lex()
           my(b=vpar_to_balanced_binary(vpar),
              e=test_highest_0s_run_extents(b),
              hi=e[1], lo=e[2]);
           for(i=hi,lo,
              my(x = (1<<(i+1)) - (1<<(lo-1)),
                 c = bitxor(b,x));
              print(test_to_binary(b));
              print(test_to_binary(c)"    "test_is_balanced_binary(c));
              printf("%*s\n", 2*n, test_to_binary(x));
              print();
              total += test_is_balanced_binary(c)));
     print1(total", ");
  );
}

\\-----------------------------------------------------------------------------
\\ vpar_is_horizpos_vector()

\\ by biggest and smallest both calculated

  \\ \\ print("vpar_is_horizpos_vector() "vec);
  \\ if(#vec,
  \\    forest>=0 || vec[1]==0 || return(0);  \\ root=1 must have v=1 pos=0
  \\ 
  \\    my(l=vecmax(vec));
  \\    \\ print(" l="l);
  \\ 
  \\    \\ maximum pos range 0 to n-1 forest, 0 to n-2 tree
  \\    l < max(1,#vec-(forest<=0)) || return(0);
  \\ 
  \\    my(s=vectorsmall(l+1), \\ s[pos+1] = smallest vertex of pos
  \\       b=s);               \\ b[pos+1] = biggest  vertex of pos
  \\    for(i=1,#vec,  \\ low to high, b[] last of
  \\       vec[i]>=0 || return(0);
  \\       b[vec[i]+1] = i);
  \\    \\ print(" b="b);
  \\ 
  \\    for(i=1,l,          
  \\       b[i] || return(0));  \\ no gaps in pos values
  \\ 
  \\    forstep(i=#vec,1,-1,  \\ high to low, b[] last of
  \\       s[vec[i]+1] = i);
  \\    \\ print(" s="s);
  \\ 
  \\    \\ tree with l>=1 must have >=2 many pos=0, as the root cannot have
  \\    \\ siblings
  \\    !l || forest>0 || b[1]!=s[1] || return(0);
  \\ 
  \\    \\ biggest b[pos+1] can be sibling of smallest s[pos], provided that b
  \\    \\ is bigger than that s
  \\    for(i=1,l,
  \\       \\ print(" i-1="i-1" b "b[i+1]" cf prev s "s[i]);
  \\       if(b[i+1]<s[i], return(0))));
  \\ 
  \\ \\ print(" yes");
  \\ 1;

\\-----------------------------------------------------------------------------
\\ vpar_unsigned_Laplacian_poly()
\\ -- done as parameter

\\ any[v] = with vertex size 1, no adjacent edges
\\          without vertex size 0, one adjacent edge
\\ without[v] = 1 way no vertex, no adjacent edge
\\
\\ without[p] *= any[v]
\\ 
\\ any[p] = any[p]*any[v]
\\        - without[p]*without[v]      
\\
\\       without       any
\\          |           |
\\       without       any
\\ 
\\ vpar_unsigned_Laplacian_poly([0]) == 'x   without 1
\\  
\\ vpar_unsigned_Laplacian_poly([0,1])
\\  1---2                    x+1  x+1
\\  x^2 + 2*x                 1    1
\\  2 vertices  1 way
\\  1 vertices  2 ways
\\  without x + 1
\\
\\ vpar_unsigned_Laplacian_poly([0,1,2])
\\  1---2---3                x+1  x+2  x+1
\\  x^3 + 4*x^2 + 3*x         1    1    1
\\  without x^2 + 3*x + 1
\\
\\  3 vertices   1 way
\\  2 vertices   4 ways
\\  1 vertex     3 ways
\\  0 vertices   no ways
\\
\\  1---2         3
\\  x^2 + 2*x    x+1
\\    x + 1       1
\\
\\ vpar_unsigned_Laplacian_poly([0,1,2,3])
\\  1---2---3---4  
\\  x^4 + 6*x^3 + 10*x^2 + 4*x
\\ 1+2+3 + 1+2 + 1 == 10
\\ 1 + 2 + 2 + 1 == 6
\\ 

vpar_unsigned_Laplacian_matrix(vpar) =
{
  my(m = vpar_adjacency_matrix(vpar), p);  \\ 1 where edge
  for(v=1,#vpar,
     if((p=vpar[v]),
        m[v,v]++;  \\ degrees of v and p on diagonal
        m[p,p]++));
  m;
}
vpar_unsigned_Laplacian_poly(vpar,var='x) =
{
  my(with=vpar_num_children_vector(vpar),     \\ start as num_children
     seq=vpar_INTERNAL_seq_upwards(vpar,with),
     without  = vector(#vpar,v,1),   \\ without[v] = 1
     ret      = var/var);            \\ 1 as polynomial

  \\ with[v] = 'x + degree[v], by degree = num children + (1 if parent!=0)
  for(v=1,#vpar,
     with[v] = var + (with[v] + sign(vpar[v])));

  for(i=1,#seq,
     my(v=seq[i],
        p=vpar[v]);
     if(p,
        with[p] = with[p]*with[v] - without[p]*without[v];
        without[p] *= with[v];
        ,
        ret *= with[v]);  \\ at root

     with[v]    =     \\ kill intermediate polys
     without[v] = 0);

  ret;
}

{
  for(i=1,#test_sample_forests,
     my(vpar=test_sample_forests[i]);
     check_equal(Vec(vpar_unsigned_Laplacian_poly(vpar)),
                 abs(Vec(vpar_Laplacian_poly(vpar))),
                 "vpar_Laplacian_poly() termwise abs of vpar_Laplacian_poly()"));
}

\\ Choose #vpar-n vertices and one edge adjacent to each.
\\ Edges chosen cannot overlap.
\\
\\ This is not just an edge set, since can choose same set of edges in
\\ different ways,
\\     1--- 2 --- 3--- 4
\\     C E        C E
\\     1 ---2 --- 3 ---4
\\        E C        E C
\\
\\ Choice of edge and assign a direction, but only one outwards from any vertex.
\\

\\-----------------------------------------------------------------------------
\\ vpar_make_random_preorder() by unrank

vpar_make_random_preorder(n,forest=0) = \
  vpar_preorder_ith(n, random(vpar_count_ordered(n,forest))+1, forest);


{
  \\ WRONG
  \\ rand(A*B) >= C*D becomes rand(A)>=C and rand(B)>=D

  my(A=5,B=13,C=7,D=11);
  for(a=0,A-1,
     for(b=0,B-1,
        for(c=0,C-1,
           for(d=0,D-1,
              print(a" "b" "c" "d" products "a*b" "c*d);
              check_equal(a*b >= c*d,
                          a>=c && b>=d)))));
}

\\-----------------------------------------------------------------------------
\\ test_count_preorder_childful_vector_by_gf()
\\ working but not useful bits

test_Bq(q,limit) = sum(i=0,limit-1, 1/x^i * binomial(q+i,q));
check_equal(test_Bq(3,4), 1 + 4/x + binomial(5,3)/x^2 + binomial(6,3)/x^3);

test_E(p,q,limit) = sum(i=0,limit-1, x^i * binomial(p+q-1+i,q-1));

{
  \\    N  *
  \\    | \
  \\    N
  \\    | \
  \\    N
  \\    | \
  \\    L
  my(p=3,q=1);
  check_equal(x^p*test_Bq(q-1,6), 1+x+x^2+x^3 + 1/x+1/x^2);
  check_equal(test_E(p,q,4), 1 + x + x^2 + x^3);
}
{
  my(p=3,q=2);
  check_equal(x^p*test_Bq(q-1,4), 4 + 3*x + 2*x^2 + x^3);
  check_equal(test_E(p,q,4), 4 + 5*x + 6*x^2 + 7*x^3);
}
{
  my(p1=3,q1=1, p2=3,q2=1, limit=8);
  check_equal(polcoeff(x^p1*test_Bq(q1,p1+1) * test_E(p2,q2,limit), 0), 4);
  check_equal(test_count_preorder_childful_vector_by_search([1,1,1,0,1,1,1,0],1),4);
}
{
  my(p1=3,q1=2, p2=3,q2=1, limit=8);
  check_equal(polcoeff(x^p1*test_Bq(q1,p1+1) * test_E(p2,q2,limit), 0), 10);
  check_equal(test_count_preorder_childful_vector_by_search([1,1,1,0,0,1,1,1,0],1), 10);
}
{
  my(p1=3,q1=3, p2=3,q2=1, limit=8);
  check_equal(polcoeff(x^p1*test_Bq(q1,p1+1) * test_E(p2,q2,limit), 0), 20);
  check_equal(test_count_preorder_childful_vector_by_search([1,1,1,0,0,0,1,1,1,0],1), 20);
}
\\ {
\\   my(p1=3,q1=2, p2=3,q2=2, p3=3,q3=2, limit=18);
\\   check_equal(vpar_count_ordered(3+2+3+2+3+2,1), 9694845);
\\   \\ check_equal(test_count_preorder_childful_vector_by_search([1,1,1,0,0,1,1,1,0,0,1,1,1,0,0],1),
\\   \\             846);
\\   check_equal(polcoeff(x^p1*test_Bq(q1,p1+1) * x^p2*test_Bq(q2,p2+1) * test_E(p3,q3,limit), 0),
\\               846);
\\ }


\\-----------------------------------------------------------------------------
\\ vpar_depth_of_vertex()
\\ Cyclic code with a "cycle" option to count the cycle (proper ancestors),
\\ or not count so all cyclics are depth 0.

vpar_INTERNAL_depth_of_vertex(vpar,v,cycle) =
{
  \\ u is a search up from v looking for a root, ie. vpar[u]==0.
  \\ h is a half-speed search up.
  \\ If find u==h then they are part of a cycle.
  \\ Cycle length is by working upwards until u==h again.
  \\ The non-cycle length up from v is then found by re-traversing upwards
  \\ with u ahead by the cycle length, so that find u==v at the first
  \\ cyclic.
  \\
  \\ Time taken is linear in the returned depth d and the cycle length c (if
  \\ cyclic).  The first loop has two stopping conditions, either u==h or u
  \\ through #vpar many vertices.  The former catches small d and c, the
  \\ latter caps at #vpar if big c (instead of 2 cycles around big c).
  \\
  \\ The depth d at which u==h found does not say anything about the desired
  \\ depth.  Different depth reach u==h with the same d.  For example
  \\
  \\    5 <-- 4
  \\      \   |       from u=1 reach u=4,h=4 at d=6, desiring depth=2
  \\       -> 3
  \\          |       from u=2 reach u=5,h=5 at d=6, desiring depth=2
  \\          2
  \\          |
  \\          1
  \\
  \\ The d with u==h is a multiple of the cycle length c though.  If the
  \\ desired depth is t, and du=2*dh steps up by u and h, then have
  \\ dh-t == du-t mod c so that u==h, and hence dh==2*dh mod c and
  \\ dh==0 mod c.  What multiple of c is du or dh depends on t, so doesn't
  \\ help to find c.

  \\ print("vpar_depth_of_vertex() v="v);
  my(u=v, h=v);
  for(d=0,#vpar,
     (u=vpar[u]) || return(d);
     d++;
     (u=vpar[u]) || return(d);
     if((h=vpar[h])==u, break));

  \\ print(" cyclic at u="u" h="h);
  my(c=1);
  h=u; while((u=vpar[u])!=h, c++);
  \\ print(" cycle length c="c);

  u=v; for(i=1,c, u=vpar[u]);  \\ ahead by c steps
  for(d=0,#vpar,
     if(u==v, return(d + if(cycle,c-1)));      \\ equal at first cyclic
     u=vpar[u]; v=vpar[v]);
}

\\-----------------------------------------------------------------------------
\\ vpar_path_lengths_matrix()
\\ Allowing cycles.
\\ Upwards to and around cycles probably right, rest going back down is wrong.

{
  \\ all functional digraphs
  for(n=0,6,
     forvec(vpar=vector(n,i,[0,n]),
           check_equal(vpar_path_lengths_matrix(vpar),
                       test_path_lengths_matrix_by_vector(vpar))));
}


vpar_path_lengths_matrix_cyclic(vpar) =
{
  print("vpar_path_lengths_matrix_cyclic() "vpar);

  \\ ret[u,v]=-2 undetermined
  \\ ret[u,v]=-1 known u,v not reachable (different components)
  \\ ret[u,v]>=0 distance to v, by shortest path
  \\
  my(ret=matrix(#vpar,#vpar,i,j, -1),
     top=vector(#vpar,v, vpar[v]==0));
  for(u=1,#vpar,
     print(" u="u" ret "ret);
  
    \\ Upwards from u, store ret[u,v] = h distance from u.
    \\ Cycle noticed by ret[u,v] already set.
    \\
    my(v=u,   \\ v working upwards
       h=-1); \\ distance above v
    while(1,
          ret[u,v]=h++;
          if(!(v=vpar[v]),
             break);
          print("  up v="v);

          if(ret[u,v]>=0 && ret[u,v]!=h+1,  \\ already set, cycle
             print("  "ret);
             print("  cycle h="h" prev "ret[u,v]", length="h-ret[u,v]+1);
             my(c=h-ret[u,v]+1);
             for(i=1,c, top[v]=1; ret[v,v]=0; v=vpar[v]);  \\ all above cycle seen
             my(t=v);
             for(x=1,c>>1,    \\ distance v to t, upto floor(cycle length / 2)
                print("    x="x);
                t=vpar[t];    \\ extend how far ahead t
                for(i=1,c, ret[v,t]=ret[t,v]=x; v=vpar[v]; t=vpar[t]));
             print("    gives "ret);

             if(!top[u],
                print("  at t="t);
                for(x=1,(c-1)>>1,    \\ distance v to t, upto floor(cycle length / 2)
                   t=vpar[t];
                   print("    set u="u" t="t" was "ret[u,t]" set "h-1);
                   ret[u,t]=h--);
                print("    gives "ret);
               );
             break;
             \\ my(c = (h-ret[u,v]+1)>>1);
             \\ for(i=1,c, v=vpar[v]);
             \\ h -= c-1;
             \\ print("  half around to v="v" h="h", by c="c);
             \\ for(i=1,c, ret[u,v=vpar[v]]=h--;
             \\    print("  set v="v" to h="h);
             \\ )
            );

          if(ret[v,v]==0,
             h++;
             print("  seen, copy its above, h="h);
             my(t=v);
             while(ret[u,t]<0,
                   print("    copy t="t"  "ret[v,t]"+"h);
                   ret[u,t]=ret[v,t]+h;
                   (t=vpar[t]) || break);
             break);
         ));

  print(" above "ret);

  \\ downwards set ret[u,v] for u descendant of v and all reachable from v
  my(seq=vpar_seq_successive_paths(vpar,1));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     if(p,
        for(u=1,#vpar,
           if(ret[u,v]==-1 && ret[u,p]>=0,
              ret[u,v] = ret[u,p]+1))));

  \\ for(v=1,#vpar,
  \\    if(!top[v],
  \\       my(t=v, h=0);
  \\       while(!top[t], t=vpar[t]; h++);
  \\       print("  v="v" t="t" h="h);
  \\       for(u=1,#vpar,
  \\          if(ret[u,v]==-1 && ret[u,t]>=0,
  \\             print("    have u="u" to t="t" dist "ret[u,t]" set u to v="v" dist "ret[u,t]+h);
  \\             ret[u,v] = ret[v,u] = ret[u,t]+h));
  \\      top[v]=1));

  \\ for(u=1,#vpar,
  \\    print(" u="u" ret "ret);
  \\ 
  \\   \\ Upwards from each vertex.
  \\   \\ Fill ret[u,v]=-1 on the way up.  This is right setting if stop at a root,
  \\   \\ so target v is not reachable.
  \\   \\ If find ret[u,v]==-1 then already known target not reachable.
  \\   \\ This includes in a different component which is cyclic, since the
  \\   \\ ret[u,v]=-1 stored along the way is re-encountered.
  \\   \\ If ret[v]>=0 found then re-traverse and fill with that plus distance below.
  \\ 
  \\   for(i=1,#vpar,
  \\      if(ret[u,i]==-2,
  \\         print("  i="i" below");
  \\         v=i; h=1;
  \\         while(h++; ret[u,v]=ret[v,u]=-1; (v=vpar[v]),
  \\               if(ret[u,v]!=-2,
  \\                  print("    stop v="v" ret "ret[u,v]);
  \\                  if(ret[u,v]>=0,
  \\                     print("    found v="v" ret="ret[u,v]" h="h);
  \\                     h += ret[u,v];
  \\                     v=i;
  \\                     until(ret[u,v=vpar[v]]>=0, ret[u,v]=ret[v,u]=h--);
  \\                     print("    filled "ret);
  \\                    );
  \\                  break))));
  \\    );
  \\ 

  print("  return "ret);
  ret;
}


\\-----------------------------------------------------------------------------
\\ Indnum and Domnum Cyclics

vpar_INTERNAL_cyclic_num(vpar,num_children,exclude,ret,path,cycle) =
{
  \\ print("vpar_INTERNAL_cyclic_num() ret="ret);
  \\ print("  num_children "num_children);
  \\ print("  exclude      "exclude);
  if(hammingweight(num_children),
     for(i=1,#vpar,
        if(num_children[i],
           my(v=i, c=0, prev);
           until(prev=exclude[v],
                 \\ print("  flag v="v);
                 v=vpar[v];
                 c++;
                 if(v==i,
                    \\ print("  cycle c="c);
                    ret += cycle(c); next(2));
                 num_children[v]=0);

           \\ print("  subpaths from v="v);
           while(1,
                 v=vpar[v];
                 if(exclude[v],
                    if(!prev,
                       \\ print("  v="v" subpath c="c" is "path(c));
                       ret += path(c));
                    if(!num_children[v],
                       \\ print("  stop excluded flagged v="v);
                       break);
                    ,
                    \\ print("  v="v" not excluded");
                    if(prev, c=0); c++);
                 num_children[v]=0;
                 prev=exclude[v]))));
  \\ print("  ret "ret);
  ret;
}

vpar_indnum(vpar) =
{
  \\ As per
  \\
  \\   Mitchell, Cockayne, Hedetniemi, "Linear Algorithms on Recursive
  \\   Representations of Trees", Journal of Computer and System Sciences,
  \\   volume 18, 1979, pages 76-85.
  \\   https://www.sciencedirect.com/science/article/pii/0022000079900539
  \\
  \\ at each leaf, take that leaf and exclude its parent if it has a parent.
  \\ (Their Beta0(G) = indnum.)
  
  \\ print("vpar_indnum() "vpar);
  my(num_children=vpar_INTERNAL_num_children_vecsmall(vpar),
     exclude=vectorsmall(#vpar),
     ret=0, v, p);
  for(i=1,#vpar,
     \\ print("  at i="i" ret="ret"  num_children "num_children[i]);
     v=i;
     while(!num_children[v] && v<=i,
           \\ print("  v="v" exclude="exclude[v]);
           p=vpar[v];
           if(!exclude[v], ret++; if(p, exclude[p]=1));
           if(p, num_children[v=p]--, break)));
  
   vpar_INTERNAL_cyclic_num(vpar,num_children,exclude,ret,
                            vpar_INTERNAL_indnum_of_path,
                            vpar_INTERNAL_indnum_of_cycle);
}

vpar_domnum(vpar) =
{

  \\ print("vpar_domnum() "vpar);
  my(num_children=vpar_INTERNAL_num_children_vecsmall(vpar),
     state=vectorsmall(#vpar),
     ret=0, v, p);
  for(i=1,#vpar,
     \\ print("  at i="i" ret="ret"  num_children "num_children[i]);
     v=i;
     while(!num_children[v] && v<=i,
           \\ print("  v="v" state="state[v]" parent p="vpar[v]);
           p=vpar[v];
           my(t=vpar_INTERNAL_domnum_table[state[v] + if(p,3*state[p]) + 1]);
           ret += bitand(t,1);
           if(p, state[p] = t>>1; num_children[v=p]--,
                 break)));
  
  vpar_INTERNAL_cyclic_num(vpar,num_children,state,ret,
                           vpar_INTERNAL_domnum_of_path,
                           vpar_INTERNAL_domnum_of_path);
}


\\-----------------------------------------------------------------------------
\\ decide instead cycle upwards like path upwards

\\ Return a vpar with a cycle of c many vertices and a tail of t many
\\ vertices below it.
test_make_cycle_and_tail(n,t) = \
  vector(n+t,i, if(i<n,i+1, i==n,1, i-1));
check_equal(test_make_cycle_and_tail(1,0), [1]);
check_equal(test_make_cycle_and_tail(2,0), [2,1]);
check_equal(test_make_cycle_and_tail(3,0), [2,3,1]);
check_equal(test_make_cycle_and_tail(1,2), [1, 1,2]);
check_equal(test_make_cycle_and_tail(2,2), [2,1, 2,3]);
check_equal(test_make_cycle_and_tail(3,2), [2,3,1, 3,4]);
check_equal(test_make_cycle_and_tail(3,6), [2,3,1, 3,4,5,6,7,8]);


\\-----------------------------------------------------------------------------
\\ equimatchable renumbered

{
  my(newnum=Vecsmall([2,3,4,1,5]), undo=newnum^-1);
  my(lookup(p,c) = vpar_INTERNAL_is_equimatchable_table[5*(p-1)+c]);
  if(0,
     for(p=1,5,
        for(c=1,5,
           my(p=undo[p], c=undo[c],
              to=lookup(p,c),
              e=if(to,newnum[to],0));
           print1(e, if(c==5,";\n",",")))));;
  my(once=matrix(5,5,p,c, lookup(p,c)),
     twice=matrix(5,5,p,c,
                  my(to=lookup(p,c));
                  if(to,lookup(to,c))));
  \\ print("once:");
  \\ printp(once);
  \\ print("twice:");
  \\ printp(twice);
  \\ print("diff:");
  \\ printp(once-twice);
  check_equal(once != twice, 1);
}

\\-----------------------------------------------------------------------------
\\ what was this for?

\\ A067078 a(1) = 1, a(2) = 2, a(n) = (n-1)*a(n-1) - (n-2)*a(n-2).
A067078(n) =
{
  n>=1 || error();
  if(n==1,1, n==2,2, (n-1)*A067078(n-1) - (n-2)*A067078(n-2));
}
{
  check_equal(vector(#A067078_samples,n, A067078(n)),
              A067078_samples);
}
\\ vector(#A067078_samples,n, A067078(n))
\\ A067078_samples

\\-----------------------------------------------------------------------------
